@@ -9,6 +9,7 @@
 #include "station.h"
 #include "pathfind.h"
 #include "economy.h"
+#include "airport.h"
 
 // remove some day perhaps?
 static Player *_cur_ai_player;
@@ -245,6 +246,9 @@ static int AiChooseTrainToReplaceWith(Player *p, Vehicle *v)
 static int AiChooseShipToReplaceWith(Player *p, Vehicle *v)
 {
 	error("!AiChooseShipToReplaceWith");
+
+	/* maybe useless, but avoids compiler warning this way */
+	return 0;
 }
 
 static void AiHandleGotoDepot(Player *p, int cmd)
@@ -263,11 +267,11 @@ static void AiHandleGotoDepot(Player *p, int cmd)
 	}
 } 
 
-static void AiRestoreVehicleOrders(Vehicle *v, uint16 *bak)
+static void AiRestoreVehicleOrders(Vehicle *v, BackuppedOrders *bak)
 {
-	uint16 ord;
+	uint16 *os = bak->order, ord;
 	int ind = 0;
-	while ((ord = *bak++) != 0) {
+	while ((ord = *os++) != 0) {
 		if (DoCommandByTile(0, v->index + (ind << 16), ord, DC_EXEC, CMD_INSERT_ORDER) == CMD_ERROR)
 			break;
 		ind++;
@@ -277,7 +281,7 @@ static void AiRestoreVehicleOrders(Vehicle *v, uint16 *bak)
 static void AiHandleReplaceTrain(Player *p)
 {
 	Vehicle *v = p->ai.cur_veh;
-	uint16 orderbak[41];
+	BackuppedOrders orderbak[1];
 	int veh;
 	uint tile;
 
@@ -304,7 +308,7 @@ static void AiHandleReplaceTrain(Player *p)
 static void AiHandleReplaceRoadVeh(Player *p)
 {
 	Vehicle *v = p->ai.cur_veh;
-	uint16 orderbak[41];
+	BackuppedOrders orderbak[1];
 	int veh;
 	uint tile;
 
@@ -331,7 +335,7 @@ static void AiHandleReplaceAircraft(Player *p)
 {
 	Vehicle *v = p->ai.cur_veh;
 	int veh;
-	uint16 orderbak[41];
+	BackuppedOrders orderbak[1];
 	uint tile;
 
 	if (!IsAircraftHangarTile(v->tile) && !(v->vehstatus&VS_STOPPED)) {
@@ -341,7 +345,6 @@ static void AiHandleReplaceAircraft(Player *p)
 
 	veh = AiChooseAircraftToReplaceWith(p, v);
 	if (veh != -1) {
-
 		BackupVehicleOrders(v, orderbak);
 		tile = v->tile;
 
@@ -405,6 +408,16 @@ typedef struct FoundRoute {
 	void *to;
 } FoundRoute;
 
+static City *AiFindRandomCity() {
+	City *c = DEREF_CITY(RandomRange(_total_cities));
+	return (c->xy != 0) ? c : NULL;
+}
+
+static Industry *AiFindRandomIndustry() {
+	Industry *i = DEREF_INDUSTRY(RandomRange(_total_industries));
+	return (i->xy != 0) ? i : NULL;
+}
+
 static void AiFindSubsidyIndustryRoute(FoundRoute *fr)
 {
 	int i;
@@ -430,16 +443,16 @@ static void AiFindSubsidyIndustryRoute(FoundRoute *fr)
 		return;
 	fr->cargo = cargo;
 
-	fr->from = from = &_industries[s->from];
+	fr->from = from = DEREF_INDUSTRY(s->from);
 
 	if (cargo == CT_GOODS || cargo == CT_FOOD) {
 		to_cit = DEREF_CITY(s->to);
-		if (to_cit->population < (cargo == CT_FOOD ? 200 : 900))
+		if (to_cit->population < (uint32)(cargo == CT_FOOD ? 200 : 900))
 			return; // error
 		fr->to = to_cit;
 		to_xy = to_cit->xy;
 	} else {
-		to_ind = &_industries[s->to];
+		to_ind = DEREF_INDUSTRY(s->to);
 		fr->to = to_ind;
 		to_xy = to_ind->xy;
 	}
@@ -491,8 +504,8 @@ static void AiFindRandomIndustryRoute(FoundRoute *fr)
 	r = Random();
 
 	// pick a source
-	fr->from = i = &_industries[(uint16)r * lengthof(_industries) >> 16];
-	if (i->xy == 0)
+	fr->from = i = AiFindRandomIndustry();
+	if (i == NULL)	
 		return;
 
 	// pick a random produced cargo
@@ -508,16 +521,16 @@ static void AiFindRandomIndustryRoute(FoundRoute *fr)
 
 	if (cargo != CT_GOODS && cargo != CT_FOOD) {
 		// pick a dest, and see if it can receive
-		i2 = &_industries[(r>>16) * lengthof(_industries) >> 16];
-		if (i == i2 || i2->xy == 0 || !(i2->accepts_cargo[0] == cargo || i2->accepts_cargo[1] == cargo || i2->accepts_cargo[2] == cargo))
+		i2 = AiFindRandomIndustry();
+		if (i2 == NULL || i == i2 || !(i2->accepts_cargo[0] == cargo || i2->accepts_cargo[1] == cargo || i2->accepts_cargo[2] == cargo))
 			return;
 
 		fr->to = i2;
 		fr->distance = GetTileDist(i->xy, i2->xy);
 	} else {
 		// pick a dest city, and see if it's big enough
-		city = DEREF_CITY((r>>16) * lengthof(_cities) >> 16);
-		if (city->population < (cargo == CT_FOOD ? 200 : 900))
+		city = AiFindRandomCity();
+		if (city == NULL || city->population < (uint32)(cargo == CT_FOOD ? 200 : 900))
 			return;
 		
 		fr->to = city;
@@ -535,12 +548,12 @@ static void AiFindRandomPassengerRoute(FoundRoute *fr)
 
 	r = Random();
 
-	fr->from = source = DEREF_CITY((uint16)r * lengthof(_cities) >> 16);
-	if (source->xy == 0 || source->population < 400)
+	fr->from = source = AiFindRandomCity();
+	if (source == NULL || source->population < 400)
 		return;
 	
-	fr->to = dest = DEREF_CITY((r >> 16) * lengthof(_cities) >> 16);
-	if (source == dest || dest->xy == 0 || dest->population < 400)
+	fr->to = dest = AiFindRandomCity();
+	if (dest == NULL || source == dest || dest->population < 400)
 		return;
 
 	fr->distance = GetTileDist(source->xy, dest->xy);
@@ -554,6 +567,7 @@ static bool AiCheckIfRouteIsGood(Player *p, FoundRoute *fr, byte bitmask)
 	TileIndex from_tile, to_tile;
 	Station *st;
 	int dist, cur;
+	uint same_station = 0;
 
 	// Make sure distance to closest station is < 37 pixels.
 	from_tile = GET_CITY_OR_INDUSTRY_TILE(fr->from);
@@ -565,7 +579,14 @@ static bool AiCheckIfRouteIsGood(Player *p, FoundRoute *fr, byte bitmask)
 		if (cur < dist) dist = cur;
 		cur = GetTileDist1D(to_tile, st->xy);
 		if (cur < dist) dist = cur;
+		if (to_tile == from_tile && st->xy == to_tile)
+		    same_station++;
 	}
+	
+	// To prevent the AI from building ten busstations in the same city, do some calculations
+	//  For each road or airport station, we want 350 of population!
+	if ((bitmask == 2 || bitmask == 4) && same_station > 2 && ((City *)(fr->from))->population < same_station * 350)
+	    return false;
 
 	if (dist != 0xFFFF && dist > 37)
 		return false;
@@ -1246,11 +1267,6 @@ static void AiWantLongRoadPassengerRoute(Player *p)
 	p->ai.timeout_counter = 0;
 }
 
-static City *AiFindRandomCity() {
-	City *c = DEREF_CITY(RandomRange(lengthof(_cities)));
-	return (c->xy != 0) ? c : NULL;
-}
-
 static void AiWantPassengerRouteInsideCity(Player *p)
 {
 	int i;
@@ -1396,7 +1412,7 @@ static void AiWantOilRigAircraftRoute(Player *p)
 		c = AiFindRandomCity();
 		if (c != NULL) {
 			// Find a random oil rig industry
-			in = &_industries[RandomRange(lengthof(_industries))];
+			in = DEREF_INDUSTRY(RandomRange(lengthof(_industries)));
 			if (in != NULL && in->type == IT_OIL_RIG) {
 				if (GetTileDist(c->xy, in->xy) < 60)
 					break;
@@ -1548,7 +1564,7 @@ static int32 AiDoBuildDefaultRailTrack(TileIndex tile, const AiDefaultBlockData
 		// This will seldomly overflow for valid reasons. Mask it to be on the safe side.
 		uint t = TILE_MASK(tile + p->tileoffs);
 
-		_cleared_city = NULL;	
+		_cleared_city = NULL;
 
 		if (p->mode < 2) {
 			if (p->mode == 0) {
@@ -1625,8 +1641,10 @@ clear_city_stuff:;
 		p++;
 	}
 
-	if (city != NULL && rating > city->ratings[_current_player]) {
-		return CMD_ERROR;
+	if (!(flag & DC_EXEC)) {
+		if (city != NULL && rating > city->ratings[_current_player]) {
+			return CMD_ERROR;
+		}
 	}
 
 	return total_cost;
@@ -1932,6 +1950,7 @@ static void FORCEINLINE AiCheckBuildRailBridgeHere(AiRailFinder *arf, TileIndex
 		// Allow bridges directly over bottom tiles
 		flag = arf->ti.z == 0;
 		for(;;) {
+			if (tile_new < -_tileoffs_by_dir[dir2]) return; // Wraping around map, no bridge possible!
 			tile_new = TILE_MASK(tile_new + _tileoffs_by_dir[dir2]);
 			FindLandscapeHeightByTile(&arf->ti, tile_new);
 			if (arf->ti.tileh != 0 || arf->ti.type == MP_CLEAR || arf->ti.type == MP_TREES) {
@@ -1943,7 +1962,7 @@ static void FORCEINLINE AiCheckBuildRailBridgeHere(AiRailFinder *arf, TileIndex
 			flag = true;
 		}
 
-		// Build bridge?
+		// Is building a (rail)bridge possible at this place (type doesn't matter)?
 		if (DoCommandByTile(tile_new, tile, arf->player->ai.railtype_to_use<<8, 
 			DC_AUTO, CMD_BUILD_BRIDGE) == CMD_ERROR)
 				return;
@@ -2100,14 +2119,19 @@ static void AiBuildRailConstruct(Player *p)
 
 	if (arf.best_ptr[0]&0x80) {
 		int i;
+		int32 bridge_len = GetBridgeLength(arf.bridge_end_tile, p->ai.cur_tile_a);
 		
-		// Figure out what bridge type to build
-		i = 10 + (p->ai.railtype_to_use << 8);
-		for(;;) {
-			int32 cost = DoCommandByTile(arf.bridge_end_tile, p->ai.cur_tile_a, i, DC_AUTO, CMD_BUILD_BRIDGE);
-			if ((cost != CMD_ERROR && cost < p->player_money>>5) || (i&255)==0)
-				break;
-			i--;
+		/*	Figure out what (rail)bridge type to build
+				start with best bridge, then go down to worse and worse bridges
+				unnecessary to check for worse bridge (i=0), since AI will always build that.
+				AI is so fucked up that fixing this small thing will probably not solve a thing
+		*/
+		for(i = 10 + (p->ai.railtype_to_use << 8); i != 0; i--) {
+			if (CheckBridge_Stuff(i, bridge_len)) {
+				int32 cost = DoCommandByTile(arf.bridge_end_tile, p->ai.cur_tile_a, i, DC_AUTO, CMD_BUILD_BRIDGE);
+				if (cost != CMD_ERROR && cost < (p->player_money >> 5))
+					break;
+			}
 		}
 
 		// Build it
@@ -2800,6 +2824,7 @@ static void FORCEINLINE AiCheckBuildRoadBridgeHere(AiRoadFinder *arf, TileIndex
 		// Allow bridges directly over bottom tiles
 		flag = arf->ti.z == 0;
 		for(;;) {
+			if (tile_new < -_tileoffs_by_dir[dir2]) return; // Wraping around map, no bridge possible!		
 			tile_new = TILE_MASK(tile_new + _tileoffs_by_dir[dir2]);
 			FindLandscapeHeightByTile(&arf->ti, tile_new);
 			if (arf->ti.tileh != 0 || arf->ti.type == MP_CLEAR || arf->ti.type == MP_TREES) {
@@ -2813,7 +2838,7 @@ static void FORCEINLINE AiCheckBuildRoadBridgeHere(AiRoadFinder *arf, TileIndex
 			flag = true;
 		}
 
-		// Build bridge?
+		// Is building a (rail)bridge possible at this place (type doesn't matter)?
 		if (DoCommandByTile(tile_new, tile, 0x8000, DC_AUTO, CMD_BUILD_BRIDGE) == CMD_ERROR)
 			return;
 		AiBuildRoadRecursive(arf, tile_new, dir2);
@@ -2961,19 +2986,26 @@ do_some_terraform:
 
 	if (arf.best_ptr[0]&0x80) {
 		int i;
+		int32 bridge_len;
 		p->ai.cur_tile_a = arf.bridge_end_tile;
-		
-		// Figure out what bridge type to build
-		i = 10 + (0x80<<8);
-		for(;;) {
-			int32 cost = DoCommandByTile(tile, p->ai.cur_tile_a, i, DC_AUTO, CMD_BUILD_BRIDGE);
-			if ((cost != CMD_ERROR && cost < p->player_money>>5) || !(i&255))
-				break;
-			i--;
+		bridge_len = GetBridgeLength(tile, p->ai.cur_tile_a); // tile
+
+		/*	Figure out what (road)bridge type to build
+				start with best bridge, then go down to worse and worse bridges
+				unnecessary to check for worse bridge (i=0), since AI will always build that.
+				AI is so fucked up that fixing this small thing will probably not solve a thing
+		*/
+		for(i = 10; i != 0; i--) {                      
+			if (CheckBridge_Stuff(i, bridge_len)) {
+				int32 cost = DoCommandByTile(tile, p->ai.cur_tile_a, i + (0x80 << 8), DC_AUTO, CMD_BUILD_BRIDGE);
+				if (cost != CMD_ERROR && cost < (p->player_money >> 5))
+					break;
+			}
 		}
 
 		// Build it
-		DoCommandByTile(tile, p->ai.cur_tile_a, i, DC_AUTO | DC_EXEC, CMD_BUILD_BRIDGE);
+		DoCommandByTile(tile, p->ai.cur_tile_a, i + (0x80 << 8), DC_AUTO | DC_EXEC, CMD_BUILD_BRIDGE);
+
 		p->ai.state_counter = 0;
 	} else if (arf.best_ptr[0]&0x40) {
 		// tunnel
@@ -3175,42 +3207,90 @@ static void AiStateDeleteRoadBlocks(Player *p)
 static bool AiCheckIfHangar(Station *st)
 {
 	uint tile = st->airport_tile;
-	return (_map5[tile] == 0x20 || _map5[tile] == 0x41);
+	// HANGAR of airports
+	// 0x20 - hangar large airport (32)
+	// 0x41 - hangar small airport (65)
+	return (_map5[tile] == 32 || _map5[tile] == 65);
 }
 
 static void AiStateAirportStuff(Player *p)
 {
 	Station *st;
+	byte acc_planes;
 	int i;
 	AiBuildRec *aib;
 	byte rule;
 
+	// Here we look for an airport we could use instead of building a new
+	// one. If we find such an aiport for any waypoint,
+	// AiStateBuildDefaultAirportBlocks() will kindly skip that one when
+	// building the waypoints.
+
 	i = 0;
 	do {
+		// We do this all twice - once for the source (city in the case
+		// of oilrig route) and then for the destination (oilrig in the
+		// case of oilrig route).
+		aib = &p->ai.src + i;
+
 		FOR_ALL_STATIONS(st) {
-			aib = &p->ai.src + i;
+			// Dismiss ghost stations.
+			if (st->xy == 0)
+				continue;
 
-			if (st->xy != 0 &&
-					// Has airport
-					st->facilities&8 &&
-					// ??
-					!(p->ai.build_kind==0 && st->airport_flags&0x40) &&
-					// ??
-					!(i==0 && st->airport_flags&0x40) &&
-					// Owned by current player
-					!(st->owner!=OWNER_NONE && st->owner != _current_player) &&
-					// Not too far away
-					GetTileDist1D(st->airport_tile, aib->spec_tile) <= aib->rand_rng) {
-				
-				rule = 2;
-				if (!(st->airport_flags&0x40)) {
-					rule = AiCheckIfHangar(st);
-				}
+			// Is this an airport?
+			if (!(st->facilities & FACIL_AIRPORT))
+				continue;
 
-				aib->cur_building_rule = rule;
-				aib->use_tile = st->airport_tile;
-				break;
+			// Do we own the airport? (Oilrigs aren't owned, though.)
+			if (st->owner != OWNER_NONE && st->owner != _current_player)
+				continue;
+
+			acc_planes = GetAirport(st->airport_type)->acc_planes;
+
+			// Dismiss heliports, unless we are checking an oilrig.
+			if (acc_planes == HELICOPTERS_ONLY && !(p->ai.build_kind == 1 && i == 1))
+				continue;
+
+			// Dismiss country airports if we are doing the other
+			// endpoint of an oilrig route.
+			if (acc_planes == AIRCRAFT_ONLY && (p->ai.build_kind == 1 && i == 0))
+				continue;
+
+			// Dismiss airports too far away.
+			if (GetTileDist1D(st->airport_tile, aib->spec_tile) > aib->rand_rng)
+				continue;
+
+			// It's ideal airport, let's take it!
+
+			/* XXX: This part is utterly broken - rule should
+			 * contain number of the rule appropriate for the
+			 * airport type (country, city, ...), see
+			 * _airport_default_block_data (rule is just an index
+			 * in this array). But the only difference between the
+			 * currently existing two rules (rule 0 - city and rule
+			 * 1 - country) is the attr field which is used only
+			 * when building new airports - and that's irrelevant
+			 * for us. So using just about any rule will suffice
+			 * here for now (some of the new airport types would be
+			 * broken because they will probably need different
+			 * tileoff values etc), no matter that
+			 * AiCheckIfHangar() makes no sense. --pasky */
+			if (acc_planes == HELICOPTERS_ONLY) {
+				/* Heliports should have maybe own rulesets but
+				 * OTOH we don't want AI to pick them up when
+				 * looking for a suitable airport type to build.
+				 * So any of rules 0 or 1 would do for now. The
+				 * original rule number was 2 but that's a bug
+				 * because we have no such rule. */
+				rule = 1;
+			} else {
+				rule = AiCheckIfHangar(st);
 			}
+
+			aib->cur_building_rule = rule;
+			aib->use_tile = st->airport_tile;
+			break;
 		}
 	} while (++i != p->ai.num_build_rec);
 
@@ -3256,11 +3336,16 @@ static bool AiCheckAirportResources(TileIndex tile, const AiDefaultBlockData *p,
 	return true;
 }
 
-static int AiFindBestDefaultAirportBlock(TileIndex tile, byte cargo, int32 *cost)
+static int AiFindBestDefaultAirportBlock(TileIndex tile, byte cargo, byte heli, int32 *cost)
 {
 	int i;
 	const AiDefaultBlockData *p;
 	for(i=0; (p = _airport_default_block_data[i]) != NULL; i++) {
+		// If we are doing a helicopter service, avoid building
+		// airports where they can't land.
+		if (heli && GetAirport(p->attr)->acc_planes == AIRCRAFT_ONLY)
+			continue;
+
 		*cost = AiDoBuildDefaultAirportBlock(tile, p, 0);
 		if (*cost != CMD_ERROR && AiCheckAirportResources(tile, p, cargo))
 			return i;
@@ -3297,7 +3382,7 @@ static void AiStateBuildDefaultAirportBlocks(Player *p)
 			aib->use_tile = AdjustTileCoordRandomly(aib->spec_tile, aib->rand_rng);
 
 			// check if the aircraft stuff can be built there.
-			rule = AiFindBestDefaultAirportBlock(aib->use_tile, aib->cargo, &cost);
+			rule = AiFindBestDefaultAirportBlock(aib->use_tile, aib->cargo, p->ai.build_kind, &cost);
 
 #if 0
 			if (!IS_TILETYPE(aib->use_tile, MP_STREET) &&
@@ -3704,13 +3789,13 @@ static void AiHandleTakeover(Player *p)
 
 	{
 		uint asked = p->bankrupt_asked;
-		Player *pp, *best_pl;
+		Player *pp, *best_pl = NULL;
 		int32 best_val = -1;
 		uint old_p;
 
 		// Ask the guy with the highest performance hist.
 		FOR_ALL_PLAYERS(pp) {
-			if (pp->name_1 != 0 &&
+			if (pp->is_active &&
 					!(asked&1) &&
 					pp->bankrupt_asked == 0 &&
 					best_val < pp->old_economy[1].performance_history) {
