@@ -40,7 +40,7 @@ static void CcBuildAircraft(bool success, uint tile, uint32 p1, uint32 p2)
 			_backup_orders_tile = 0;
 			RestoreVehicleOrders(v, _backup_orders_data);
 		}
-		ShowAircraftViewWindow(v);	
+		ShowAircraftViewWindow(v);
 	}
 }
 
@@ -82,11 +82,11 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 				if (HASBIT(e->player_avail, _local_player)) {
 					if (sel==0) selected_id = engine_id;
 					if (IS_INT_INSIDE(--pos, -4, 0)) {
-						DrawString(x+62, y+7, _engine_name_strings[engine_id], sel==0 ? 0xC : 0x10);
+						DrawString(x+62, y+7, GetCustomEngineName(engine_id), sel==0 ? 0xC : 0x10);
 						DrawAircraftEngine(x+29, y+10, engine_id, SPRITE_PALETTE(PLAYER_SPRITE_COLOR(_local_player)));
 						y += 24;
 					}
-					sel--; 
+					sel--;
 				}
 			} while (++engine_id, ++e,--num);
 
@@ -101,12 +101,12 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 				SET_DPARAM16(3, _aircraft_num_mail[selected_id - AIRCRAFT_ENGINES_INDEX]);
 				SET_DPARAM32(4,_aircraft_running_cost[selected_id - AIRCRAFT_ENGINES_INDEX] * _price.aircraft_running >> 8);
 
-				e = &_engines[selected_id];	
+				e = &_engines[selected_id];
 				SET_DPARAM16(6, e->lifelength);
 				SET_DPARAM8(7, e->reliability * 100 >> 16);
 				ConvertDayToYMD(&ymd, e->intro_date);
 				SET_DPARAM16(5, ymd.year + 1920);
-				
+
 				DrawString(2, 111, STR_A007_COST_SPEED_CAPACITY_PASSENGERS, 0);
 			}
 		}
@@ -121,7 +121,7 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 				SetWindowDirty(w);
 			}
 		} break;
-					
+
 		case 5: { /* build */
 			int sel_eng = WP(w,buildtrain_d).sel_engine;
 			if (sel_eng != -1)
@@ -131,7 +131,7 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 		case 6:	/* rename */
 			WP(w,buildtrain_d).rename_engine = WP(w,buildtrain_d).sel_engine;
 			ShowQueryString(
-				_engine_name_strings[WP(w,buildtrain_d).sel_engine],
+				GetCustomEngineName(WP(w,buildtrain_d).sel_engine),
 				STR_A039_RENAME_AIRCRAFT_TYPE,
 				31,
 				160,
@@ -140,7 +140,7 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 			break;
 		}
 		break;
-	
+
 	case WE_4:
 		if (w->window_number != 0 && !FindWindowById(WC_VEHICLE_DEPOT, w->window_number)) {
 			DeleteWindow(w);
@@ -154,7 +154,7 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 		memcpy(_decode_parameters, b, 32);
 		DoCommandP(0, WP(w,buildtrain_d).rename_engine, 0, NULL, CMD_RENAME_ENGINE | CMD_MSG(STR_A03A_CAN_T_RENAME_AIRCRAFT_TYPE));
 	} break;
-	}	
+	}
 }
 
 static const Widget _new_aircraft_widgets[] = {
@@ -185,7 +185,7 @@ static void ShowBuildAircraftWindow(uint tile)
 	w = AllocateWindowDesc(&_new_aircraft_desc);
 	w->window_number = tile;
 	w->vscroll.cap = 4;
-	
+
 	if (tile != 0) {
 		w->caption_color = _map_owner[tile];
 	} else {
@@ -206,7 +206,7 @@ static void AircraftRefitWndProc(Window *w, WindowEvent *e)
 {
 	switch(e->event) {
 	case WE_PAINT: {
-		Vehicle *v = &_vehicles[w->window_number];	
+		Vehicle *v = &_vehicles[w->window_number];
 		const byte *b;
 		int sel;
 		int x,y;
@@ -218,7 +218,7 @@ static void AircraftRefitWndProc(Window *w, WindowEvent *e)
 		DrawWindowWidgets(w);
 
 		DrawString(1, 15, STR_A040_SELECT_CARGO_TYPE_TO_CARRY, 0);
-		
+
 		cargo = -1;
 		x = 6;
 		y = 25;
@@ -232,7 +232,7 @@ static void AircraftRefitWndProc(Window *w, WindowEvent *e)
 			}
 			sel--;
 			DrawString(x,y,_cargoc.names_s[*b], color);
-			y += 10;			
+			y += 10;
 		} while (*++b != 0xFF);
 
 		WP(w,refit_d).cargo = cargo;
@@ -261,7 +261,7 @@ static void AircraftRefitWndProc(Window *w, WindowEvent *e)
 			} break;
 		case 4: /* refit button */
 			if (WP(w,refit_d).cargo != 0xFF) {
-				Vehicle *v = &_vehicles[w->window_number];	
+				Vehicle *v = &_vehicles[w->window_number];
 				if (DoCommandP(v->tile, v->index, WP(w,refit_d).cargo, NULL, CMD_REFIT_AIRCRAFT | CMD_MSG(STR_A042_CAN_T_REFIT_AIRCRAFT)))
 					DeleteWindow(w);
 			}
@@ -293,8 +293,8 @@ static void ShowAircraftRefitWindow(Vehicle *v)
 	Window *w;
 
 	DeleteWindowById(WC_VEHICLE_REFIT, v->index);
-	
-	_alloc_wnd_parent_num = v->index;	
+
+	_alloc_wnd_parent_num = v->index;
 	w = AllocateWindowDesc(&_aircraft_refit_desc);
 	w->window_number = v->index;
 	w->caption_color = v->owner;
@@ -319,7 +319,7 @@ static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 			StringID str;
 
 			SET_DPARAM16(1, year);
-			
+
 			str = STR_0199_YEAR;
 			if (year != 1) {
 				str++;
@@ -366,7 +366,7 @@ static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 
 			do {
 				if (v->subtype <= 2) {
-					SET_DPARAM16(0, _engine_name_strings[v->engine_type]);
+					SET_DPARAM16(0, GetCustomEngineName(v->engine_type));
 					SET_DPARAM16(1, 1920 + v->build_year);
 					SET_DPARAM32(2, v->value);
 					DrawString(60, y, STR_A011_BUILT_VALUE, 0);
@@ -463,7 +463,7 @@ static void ShowAircraftDetailsWindow(Vehicle *v)
 	DeleteWindowById(WC_VEHICLE_ORDERS, veh);
 	DeleteWindowById(WC_VEHICLE_DETAILS, veh);
 
-	_alloc_wnd_parent_num = veh;	
+	_alloc_wnd_parent_num = veh;
 	w = AllocateWindowDesc(&_aircraft_details_desc);
 	w->window_number = veh;
 	w->caption_color = v->owner;
@@ -497,7 +497,7 @@ static void AircraftViewWndProc(Window *w, WindowEvent *e)
 		{
 			uint tile = v->tile;
 			if (IS_TILETYPE(tile, MP_STATION) &&
-					(_map5[tile]==0x20 || _map5[tile]==0x41) &&
+					(_map5[tile] == 32 || _map5[tile] == 65) &&
 					v->vehstatus&VS_STOPPED)
 						disabled = 0;
 		}
@@ -549,13 +549,13 @@ static void AircraftViewWndProc(Window *w, WindowEvent *e)
 		DrawStringCentered(125, 105, str, 0);
 		DrawWindowViewport(w);
 	} break;
-		
+
 	case WE_CLICK: {
 		Vehicle *v = &_vehicles[w->window_number];
 
 		switch(e->click.widget) {
 		case 4: /* start stop */
-			DoCommandP(v->tile, v->index, 0, NULL, CMD_START_STOP_AIRCRAFT | CMD_MSG(STR_A016_CAN_T_STOP_START_AIRCRAFT)); 
+			DoCommandP(v->tile, v->index, 0, NULL, CMD_START_STOP_AIRCRAFT | CMD_MSG(STR_A016_CAN_T_STOP_START_AIRCRAFT));
 			break;
 		case 5: /* center main view */
 			ScrollMainWindowTo(v->x_pos, v->y_pos);
@@ -574,12 +574,12 @@ static void AircraftViewWndProc(Window *w, WindowEvent *e)
 			break;
 		}
 	} break;
-	
+
 	case WE_DESTROY:
 		DeleteWindowById(WC_VEHICLE_ORDERS, w->window_number);
-		DeleteWindowById(WC_VEHICLE_REFIT, w->window_number);		
+		DeleteWindowById(WC_VEHICLE_REFIT, w->window_number);
 		DeleteWindowById(WC_VEHICLE_DETAILS, w->window_number);
-		break;	
+		break;
 	}
 }
 
@@ -618,12 +618,12 @@ static void DrawAircraftDepotWindow(Window *w)
 	/* determine amount of items for scroller */
 	num = 0;
 	FOR_ALL_VEHICLES(v) {
-		if (v->type == VEH_Aircraft && v->subtype <= 2 && v->vehstatus&VS_HIDDEN && 
+		if (v->type == VEH_Aircraft && v->subtype <= 2 && v->vehstatus&VS_HIDDEN &&
 				v->tile == (TileIndex)tile)
 					num++;
 	}
 	SetVScrollCount(w, (num + 3) >> 2);
-	
+
 	SET_DPARAM16(0, _map2[tile]);
 	DrawWindowWidgets(w);
 
@@ -639,12 +639,12 @@ static void DrawAircraftDepotWindow(Window *w)
 				--num < 0 && num >=	-8) {
 
 			DrawAircraftImage(v, x+12, y, WP(w,traindepot_d).sel);
-			
+
 			SET_DPARAM16(0, v->unitnumber);
 			DrawString(x, y+2, (uint16)(v->max_age-366) >= v->age ? STR_00E2 : STR_00E3, 0);
 
 			DrawSprite( (v->vehstatus & VS_STOPPED) ? 0xC12 : 0xC13, x, y+12);
-			
+
 			if ((x+=74) == 2+74*4) {
 				x -= 74*4;
 				y += 24;
@@ -658,7 +658,7 @@ static int GetVehicleFromAircraftDepotWndPt(Window *w, int x, int y, Vehicle **v
 	Vehicle *v;
 	uint tile;
 	int pos;
-	
+
 	xt = x / 74;
 	xm = x % 74;
 	if (xt >= 4)
@@ -674,7 +674,7 @@ static int GetVehicleFromAircraftDepotWndPt(Window *w, int x, int y, Vehicle **v
 	tile = w->window_number;
 	FOR_ALL_VEHICLES(v) {
 		if (v->type == VEH_Aircraft &&
-				v->subtype <= 2 && 
+				v->subtype <= 2 &&
 				v->vehstatus&VS_HIDDEN &&
 				v->tile == (TileIndex)tile &&
 				--pos < 0) {
@@ -717,7 +717,7 @@ static void AircraftDepotClickAircraft(Window *w, int x, int y)
 
 	default:
 		NOT_REACHED();
-	}	
+	}
 }
 
 static const Widget _aircraft_depot_widgets[] = {
@@ -772,12 +772,12 @@ static void AircraftDepotWndProc(Window *w, WindowEvent *e)
 				ShowAircraftViewWindow(v);
 			}
 		} break;
-					
+
 		case 3:
 			if (!HASBIT(w->disabled_state, 3) &&
 					WP(w,traindepot_d).sel != INVALID_VEHICLE)	{
 				Vehicle *v;
-				
+
 				HandleButtonClick(w, 3);
 
 				v = &_vehicles[WP(w,traindepot_d).sel];
@@ -785,7 +785,7 @@ static void AircraftDepotWndProc(Window *w, WindowEvent *e)
 
 				_backup_orders_tile = v->tile;
 				BackupVehicleOrders(v, _backup_orders_data);
-	
+
 				if (!DoCommandP(v->tile, v->index, 0, NULL,  CMD_SELL_AIRCRAFT | CMD_MSG(STR_A01C_CAN_T_SELL_AIRCRAFT)))
 					_backup_orders_tile = 0;
 			}
@@ -869,7 +869,7 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 
 			SetVScrollCount(w, num);
 		}
-		
+
 		/* draw the widgets */
 		{
 			Player *p = DEREF_PLAYER(w->window_number);
@@ -885,15 +885,15 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 			byte owner = (byte)w->window_number;
 			int x = 2;
 			int y = 15;
-			
+
 			FOR_ALL_VEHICLES(v) {
 				if (v->type == VEH_Aircraft && v->subtype <= 2 && v->owner == owner &&
 						--pos < 0 && pos >= -4) {
 					StringID str;
-					
+
 					DrawAircraftImage(v, x + 19, y + 6, INVALID_VEHICLE);
 					DrawVehicleProfitButton(v, x, y+13);
-					
+
 					SET_DPARAM16(0, v->unitnumber);
 					if (IsAircraftHangarTile(v->tile)) {
 						str = STR_021F;
@@ -905,16 +905,16 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 					SET_DPARAM32(0, v->profit_this_year);
 					SET_DPARAM32(1, v->profit_last_year);
 					DrawString(x+19, y + 28, STR_0198_PROFIT_THIS_YEAR_LAST_YEAR, 0);
-					
+
 					if (v->string_id != STR_SV_AIRCRAFT_NAME) {
 						SET_DPARAM16(0, v->string_id);
 						DrawString(x+19, y, STR_01AB, 0);
 					}
 
 					DrawSmallSchedule(v, x+136, y);
-					
+
 					y += 36;
-				}				
+				}
 			}
 		}
 		break;
@@ -925,7 +925,7 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 			int sel;
 			Vehicle *v;
 			byte owner;
-	
+
 			sel = (e->click.pt.y - 14) / 36;
 
 			if ((uint)sel >= 4)
@@ -934,7 +934,7 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 			owner = (byte)w->window_number;
 			FOR_ALL_VEHICLES(v) {
 				if (v->type == VEH_Aircraft && v->subtype <= 2 && v->owner == owner &&
-						--sel < 0) {	
+						--sel < 0) {
 					ShowAircraftViewWindow(v);
 					break;
 				}
@@ -952,7 +952,7 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 					ShowBuildAircraftWindow(tile);
 					return;
 				}
-				
+
 				tile = TILE_MASK(tile + 1);
 			} while(tile != _last_built_aircraft_depot_tile);
 			ShowBuildAircraftWindow(0);
