@@ -354,7 +354,7 @@ year_4:
 		} else {
 			p->money64 = p->player_money = 100000000;
 			ChangeOwnershipOfPlayerItems(owner, 0xFF);
-			p->name_1 = 0;
+			p->is_active = false;
 		}
 	}
 }
@@ -466,6 +466,9 @@ StringID GetNewsStringBankrupcy(NewsItem *ni)
 	default:
 		NOT_REACHED();
 	}
+  
+	/* useless, but avoids compiler warning this way */
+	return 0;
 }
 
 static void PlayersGenStatistics()
@@ -485,7 +488,7 @@ static void PlayersGenStatistics()
 		return;
 
 	FOR_ALL_PLAYERS(p) {
-		if (p->name_1 != 0) {
+		if (p->is_active) {
 			memmove(&p->old_economy, &p->cur_economy, sizeof(p->old_economy));
 			memset(&p->cur_economy, 0, sizeof(p->cur_economy));
 
@@ -551,7 +554,7 @@ static void PlayersPayInterest()
 	int interest = _economy.interest_rate * 54;
 
 	FOR_ALL_PLAYERS(p) {
-		if (p->name_1 == 0)
+		if (!p->is_active)
 			continue;
 
 		_current_player = p->index;
@@ -620,7 +623,7 @@ void StartupEconomy()
 
 	_economy.interest_rate = _opt.diff.initial_interest;
 	_economy.infl_amount = _opt.diff.initial_interest;
-	_economy.infl_amount_pr = _opt.diff.initial_interest - 1;
+	_economy.infl_amount_pr = max(0, _opt.diff.initial_interest - 1);
 	_economy.max_loan_unround = _economy.max_loan = _opt.diff.max_loan * 1000;
 	_economy.fluct = (byte)(Random()) + 168;
 }
@@ -639,7 +642,7 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 	if (s->age < 12) {
 		if (!(s->cargo_type == CT_PASSENGERS || s->cargo_type == CT_MAIL)) {
 			SET_DPARAM16(1, STR_2029);
-			i = &_industries[s->from];
+			i = DEREF_INDUSTRY(s->from);
 			tile = i->xy;
 			SET_DPARAM16(2, i->city->citynametype);
 			SET_DPARAM32(3, i->city->citynameparts);
@@ -648,7 +651,7 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 
 			if (s->cargo_type != CT_GOODS && s->cargo_type != CT_FOOD) {
 				SET_DPARAM16(5, STR_2029);
-				i = &_industries[s->to];
+				i = DEREF_INDUSTRY(s->to);
 				tile2 = i->xy;
 				SET_DPARAM16(8, i->type + STR_4802_COAL_MINE);
 				SET_DPARAM16(6, i->city->citynametype);
@@ -729,11 +732,11 @@ static void FindSubsidyPassengerRoute(FoundRoute *fr)
 
 	fr->distance = (uint)-1;
 
-	fr->from = from = DEREF_CITY(RandomRange(lengthof(_cities)));
+	fr->from = from = DEREF_CITY(RandomRange(_total_cities));
 	if (from->xy == 0 || from->population < 400)
 		return;
 
-	fr->to = to = DEREF_CITY(RandomRange(lengthof(_cities)));
+	fr->to = to = DEREF_CITY(RandomRange(_total_cities));
 	if (from==to || to->xy == 0 || to->population < 400 || to->pct_pass_transported > 42)
 		return;
 
@@ -748,7 +751,7 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 
 	fr->distance = (uint)-1;
 
-	fr->from = i = &_industries[RandomRange(lengthof(_industries))];
+	fr->from = i = DEREF_INDUSTRY(RandomRange(_total_industries));
 	if (i->xy == 0)
 		return;
 
@@ -773,7 +776,7 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 
 	if (cargo == CT_GOODS || cargo == CT_FOOD) {
 		// The destination is a city
-		City *c = DEREF_CITY(RandomRange(lengthof(_cities)));
+		City *c = DEREF_CITY(RandomRange(_total_cities));
 		
 		// Only want big cities
 		if (c->xy == 0 || c->population < 900)
@@ -782,7 +785,7 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 		fr->to = c;
 	} else {
 		// The destination is an industry
-		Industry *i2 = &_industries[RandomRange(lengthof(_industries))];
+		Industry *i2 = DEREF_INDUSTRY(RandomRange(_total_industries));
 		
 		// The industry must accept the cargo
 		if (i == i2 || i2->xy == 0 || 
@@ -990,7 +993,7 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 			if (cargo_type == CT_PASSENGERS || cargo_type == CT_MAIL) {
 				xy = DEREF_CITY(s->from)->xy;
 			} else {
-				xy = (&_industries[s->from])->xy;
+				xy = (DEREF_INDUSTRY(s->from))->xy;
 			}
 			if (GetTileDist1D(xy, from->xy) > 9)
 				continue;
@@ -999,7 +1002,7 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 			if (cargo_type == CT_PASSENGERS || cargo_type == CT_MAIL || cargo_type == CT_GOODS || cargo_type == CT_FOOD) {
 				xy = DEREF_CITY(s->to)->xy;
 			} else {
-				xy = (&_industries[s->to])->xy;
+				xy = (DEREF_INDUSTRY(s->to))->xy;
 			}
 
 			if (GetTileDist1D(xy, to->xy) > 9)
@@ -1261,7 +1264,7 @@ static void DoAcquireCompany(Player *p)
 		}
 	}
 
-	p->name_1 = 0;
+	p->is_active = false;
 
 	DeletePlayerWindows(pi);
 }
