@@ -198,14 +198,14 @@ static void GraphLegendWndProc(Window *w, WindowEvent *e)
 	switch(e->event) {
 	case WE_PAINT:
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 == 0)
+			if (!p->is_active)
 				SETBIT(_legend_showbits, p->index);
 		}
 		w->click_state = ((~_legend_showbits) << 3);
 		DrawWindowWidgets(w);
 
 		FOR_ALL_PLAYERS(p) {
-			if (!p->name_1)
+			if (!p->is_active)
 				continue;
 
 			DrawPlayerIcon(p->index, 4, 18+p->index*12);
@@ -271,14 +271,14 @@ static void SetupGraphDrawerForPlayers(GraphDrawer *gd)
 
 	// Exclude the players which aren't valid	
 	FOR_ALL_PLAYERS(p) {
-		if (p->name_1 == 0) CLRBIT(showbits,p->index);
+		if (!p->is_active) CLRBIT(showbits,p->index);
 	}	
 	gd->sel = showbits;
 	gd->num_vert_lines = 24;
 
 	nums = 0;
 	FOR_ALL_PLAYERS(p) {
-		if (p->name_1 != 0) nums = max(nums,p->num_valid_stat_ent);
+		if (p->is_active) nums = max(nums,p->num_valid_stat_ent);
 	}
 	gd->num_on_x_axis = min(nums,24);
 
@@ -317,7 +317,7 @@ static void OperatingProfitWndProc(Window *w, WindowEvent *e)
 
 		numd = 0;
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 == 0)
+			if (!p->is_active)
 				continue;
 			gd.colors[numd] = _color_list[p->player_color].window_color_bgb;
 			for(j=gd.num_on_x_axis,i=0; --j >= 0;) {
@@ -392,7 +392,7 @@ static void IncomeGraphWndProc(Window *w, WindowEvent *e)
 
 		numd = 0;
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 == 0)
+			if (!p->is_active)
 				continue;
 			gd.colors[numd] = _color_list[p->player_color].window_color_bgb;
 			for(j=gd.num_on_x_axis,i=0; --j >= 0;) {
@@ -466,7 +466,7 @@ static void DeliveredCargoGraphWndProc(Window *w, WindowEvent *e)
 
 		numd = 0;
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 == 0)
+			if (!p->is_active)
 				continue;
 			gd.colors[numd] = _color_list[p->player_color].window_color_bgb;
 			for(j=gd.num_on_x_axis,i=0; --j >= 0;) {
@@ -540,7 +540,7 @@ static void PerformanceHistoryWndProc(Window *w, WindowEvent *e)
 
 		numd = 0;
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 == 0)
+			if (!p->is_active)
 				continue;
 			gd.colors[numd] = _color_list[p->player_color].window_color_bgb;
 			for(j=gd.num_on_x_axis,i=0; --j >= 0;) {
@@ -613,7 +613,7 @@ static void CompanyValueGraphWndProc(Window *w, WindowEvent *e)
 
 		numd = 0;
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 == 0)
+			if (!p->is_active)
 				continue;
 			gd.colors[numd] = _color_list[p->player_color].window_color_bgb;
 			for(j=gd.num_on_x_axis,i=0; --j >= 0;) {
@@ -802,14 +802,14 @@ static void CompanyLeagueWndProc(Window *w, WindowEvent *e)
 	switch(e->event) {
 	case WE_PAINT: {
 		Player *p;
-		Player *plist[8];
+		Player *plist[MAX_PLAYERS];
 		size_t pl_num, i;
 
 		DrawWindowWidgets(w);
 		
 		pl_num=0;
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1)
+			if (p->is_active)
 				plist[pl_num++] = p;
 		}
 		assert(pl_num > 0);
