@@ -25,9 +25,10 @@ struct Industry {
 	byte was_cargo_delivered;
 };
 
+VARDEF int _total_industries; // For the AI: the amount of industries active
 VARDEF Industry _industries[90];
 #define DEREF_INDUSTRY(i) (&_industries[i])
-
+VARDEF bool _industry_sort_dirty;
 void DeleteIndustry(Industry *is);
 
 enum {
