@@ -76,14 +76,14 @@ typedef void ToolbarButtonProc(Window *w);
 
 static void ToolbarPauseClick(Window *w)
 {
-	if (DoCommandP(0, 0, 0, NULL, CMD_PAUSE))
+	if (DoCommandP(0, _pause?0:1, 0, NULL, CMD_PAUSE))
 		SndPlayFx(0x13);
 }
 
 static void ToolbarFastForwardClick(Window *w)
 {
-	if (DoCommandP(0, 0, 0, NULL, CMD_FAST_FORWARD))
-		SndPlayFx(0x13);
+	_fast_forward ^= true;
+	SndPlayFx(0x13);
 }
 
 
@@ -307,7 +307,8 @@ void MenuClickNewspaper(int index)
 	switch(index) {
 	case 0: ShowLastNewsMessage(); break;
 	case 1: ShowMessageOptions(); break;
-	case 2: ; /* XXX: chat not done */
+	case 2: ShowMessageHistory(); break;
+	case 3: ; /* XXX: chat not done */
 	}
 }
 
@@ -442,7 +443,7 @@ static int GetPlayerIndexFromMenu(int index)
 
 	if (index >= 0) {
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 != 0) {
+			if (p->is_active) {
 				if (--index < 0)
 					return p->index;
 			}
@@ -457,7 +458,7 @@ static void UpdatePlayerMenuHeight(Window *w)
 	Player *p;
 
 	FOR_ALL_PLAYERS(p) {
-		if (p->name_1)
+		if (p->is_active)
 			num++;
 	}
 	
@@ -489,7 +490,7 @@ static void PlayerMenuWndProc(Window *w, WindowEvent *e)
 		chk = WP(w,menu_d).checked_items; // let this mean gray items.
 		
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 != 0) {
+			if (p->is_active) {
 				if (p->index == sel) {
 					GfxFillRect(x, y, x + 0xEE, y + 9, 0);
 				}
@@ -554,6 +555,8 @@ static Window *PopupMainToolbMenu(Window *w, int x, int main_button, StringID ba
 {
 	int h;
 
+	x += w->left;
+
 	SETBIT(w->click_state, (byte)main_button);
 	InvalidateWidget(w, (byte)main_button);
 
@@ -579,6 +582,8 @@ static Window *PopupMainToolbMenu(Window *w, int x, int main_button, StringID ba
 
 static Window *PopupMainPlayerToolbMenu(Window *w, int x, int main_button, int gray)
 {
+	x += w->left;
+	
 	SETBIT(w->click_state, main_button);
 	InvalidateWidget(w, main_button);
 
@@ -789,7 +794,7 @@ static void ToolbarMusicClick(Window *w)
 
 static void ToolbarNewspaperClick(Window *w)
 {
-	PopupMainToolbMenu(w, 0x1E0, 25, STR_0200_LAST_MESSAGE_NEWS_REPORT, _newspaper_flag != 2 ? 2 : 3);
+	PopupMainToolbMenu(w, 0x1E0, 25, STR_0200_LAST_MESSAGE_NEWS_REPORT, _newspaper_flag != 2 ? 3 : 4);
 }
 
 static void ToolbarHelpClick(Window *w)
@@ -1316,36 +1321,37 @@ static const Widget _scenedit_industry_normal_widgets[] = {
 {    WWT_TEXTBTN,     7,     0,    10,     0,    13, STR_00C5},
 {    WWT_CAPTION,     7,    11,   169,     0,    13, STR_023F_INDUSTRY_GENERATION},
 {     WWT_IMGBTN,     7,     0,   169,    14,   198, 0x0},
-{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0240_COAL_MINE},
-{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0241_POWER_STATION},
-{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_0242_SAWMILL},
-{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_0243_FOREST},
-{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_0244_OIL_REFINERY},
-{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_0245_OIL_RIG},
-{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_0246_FACTORY},
-{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_0247_STEEL_MILL},
-{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_0248_FARM},
-{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_0249_IRON_ORE_MINE},
-{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_024A_OIL_WELLS},
-{    WWT_TEXTBTN,    14,     2,   167,   159,   170, STR_024B_BANK},
+{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0240_COAL_MINE, STR_0262_CONSTRUCT_COAL_MINE},
+{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0241_POWER_STATION, STR_0263_CONSTRUCT_POWER_STATION},
+{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_0242_SAWMILL, STR_0264_CONSTRUCT_SAWMILL},
+{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_0243_FOREST, STR_0265_PLANT_FOREST},
+{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_0244_OIL_REFINERY, STR_0266_CONSTRUCT_OIL_REFINERY},
+{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_0245_OIL_RIG, STR_0267_CONSTRUCT_OIL_RIG_CAN_ONLY},
+{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_0246_FACTORY, STR_0268_CONSTRUCT_FACTORY},
+{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_0247_STEEL_MILL, STR_0269_CONSTRUCT_STEEL_MILL},
+{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_0248_FARM, STR_026A_CONSTRUCT_FARM},
+{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_0249_IRON_ORE_MINE, STR_026B_CONSTRUCT_IRON_ORE_MINE},
+{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_024A_OIL_WELLS, STR_026C_CONSTRUCT_OIL_WELLS},
+{    WWT_TEXTBTN,    14,     2,   167,   159,   170, STR_024B_BANK, STR_026D_CONSTRUCT_BANK_CAN_ONLY},
 {      WWT_LAST},
 };
 
+
 static const Widget _scenedit_industry_hilly_widgets[] = {
 {    WWT_TEXTBTN,     7,     0,    10,     0,    13, STR_00C5},
 {    WWT_CAPTION,     7,    11,   169,     0,    13, STR_023F_INDUSTRY_GENERATION},
 {     WWT_IMGBTN,     7,     0,   169,    14,   198, 0x0},
-{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0240_COAL_MINE},
-{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0241_POWER_STATION},
-{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_024C_PAPER_MILL},
-{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_0243_FOREST},
-{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_0244_OIL_REFINERY},
-{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_024D_FOOD_PROCESSING_PLANT},
-{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_024E_PRINTING_WORKS},
-{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_024F_GOLD_MINE},
-{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_0248_FARM},
-{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_024B_BANK},
-{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_024A_OIL_WELLS},
+{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0240_COAL_MINE, STR_0262_CONSTRUCT_COAL_MINE},
+{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0241_POWER_STATION, STR_0263_CONSTRUCT_POWER_STATION},
+{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_024C_PAPER_MILL, STR_026E_CONSTRUCT_PAPER_MILL},
+{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_0243_FOREST, STR_0265_PLANT_FOREST},
+{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_0244_OIL_REFINERY, STR_0266_CONSTRUCT_OIL_REFINERY},
+{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_024D_FOOD_PROCESSING_PLANT, STR_026F_CONSTRUCT_FOOD_PROCESSING},
+{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_024E_PRINTING_WORKS, STR_0270_CONSTRUCT_PRINTING_WORKS},
+{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_024F_GOLD_MINE, STR_0271_CONSTRUCT_GOLD_MINE},
+{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_0248_FARM, STR_026A_CONSTRUCT_FARM},
+{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_024B_BANK, STR_0272_CONSTRUCT_BANK_CAN_ONLY},
+{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_024A_OIL_WELLS, STR_026C_CONSTRUCT_OIL_WELLS},
 {      WWT_LAST},
 };
 
@@ -1353,19 +1359,19 @@ static const Widget _scenedit_industry_desert_widgets[] = {
 {    WWT_TEXTBTN,     7,     0,    10,     0,    13, STR_00C5},
 {    WWT_CAPTION,     7,    11,   169,     0,    13, STR_023F_INDUSTRY_GENERATION},
 {     WWT_IMGBTN,     7,     0,   169,    14,   198, 0x0},
-{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0250_LUMBER_MILL},
-{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0251_FRUIT_PLANTATION},
-{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_0252_RUBBER_PLANTATION},
-{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_0244_OIL_REFINERY},
-{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_024D_FOOD_PROCESSING_PLANT},
-{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_0246_FACTORY},
-{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_0253_WATER_SUPPLY},
-{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_0248_FARM},
-{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_0254_WATER_TOWER},
-{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_024A_OIL_WELLS},
-{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_024B_BANK},
-{    WWT_TEXTBTN,    14,     2,   167,   159,   170, STR_0255_DIAMOND_MINE},
-{    WWT_TEXTBTN,    14,     2,   167,   172,   183, STR_0256_COPPER_ORE_MINE},
+{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0250_LUMBER_MILL, STR_0273_CONSTRUCT_LUMBER_MILL_TO},
+{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0251_FRUIT_PLANTATION, STR_0274_PLANT_FRUIT_PLANTATION},
+{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_0252_RUBBER_PLANTATION, STR_0275_PLANT_RUBBER_PLANTATION},
+{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_0244_OIL_REFINERY, STR_0266_CONSTRUCT_OIL_REFINERY},
+{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_024D_FOOD_PROCESSING_PLANT, STR_026F_CONSTRUCT_FOOD_PROCESSING},
+{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_0246_FACTORY, STR_0268_CONSTRUCT_FACTORY},
+{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_0253_WATER_SUPPLY, STR_0276_CONSTRUCT_WATER_SUPPLY},
+{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_0248_FARM, STR_026A_CONSTRUCT_FARM},
+{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_0254_WATER_TOWER, STR_0277_CONSTRUCT_WATER_TOWER_CAN},
+{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_024A_OIL_WELLS, STR_026C_CONSTRUCT_OIL_WELLS},
+{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_024B_BANK, STR_0272_CONSTRUCT_BANK_CAN_ONLY},
+{    WWT_TEXTBTN,    14,     2,   167,   159,   170, STR_0255_DIAMOND_MINE, STR_0278_CONSTRUCT_DIAMOND_MINE},
+{    WWT_TEXTBTN,    14,     2,   167,   172,   183, STR_0256_COPPER_ORE_MINE, STR_0279_CONSTRUCT_COPPER_ORE_MINE},
 {      WWT_LAST},
 };
 
@@ -1373,17 +1379,17 @@ static const Widget _scenedit_industry_candy_widgets[] = {
 {    WWT_TEXTBTN,     7,     0,    10,     0,    13, STR_00C5},
 {    WWT_CAPTION,     7,    11,   169,     0,    13, STR_023F_INDUSTRY_GENERATION},
 {     WWT_IMGBTN,     7,     0,   169,    14,   198, 0x0},
-{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0257_COTTON_CANDY_FOREST},
-{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0258_CANDY_FACTORY},
-{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_0259_BATTERY_FARM},
-{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_025A_COLA_WELLS},
-{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_025B_TOY_SHOP},
-{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_025C_TOY_FACTORY},
-{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_025D_PLASTIC_FOUNTAINS},
-{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_025E_FIZZY_DRINK_FACTORY},
-{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_025F_BUBBLE_GENERATOR},
-{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_0260_TOFFEE_QUARRY},
-{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_0261_SUGAR_MINE},
+{    WWT_TEXTBTN,    14,     2,   167,    16,    27, STR_0257_COTTON_CANDY_FOREST, STR_027A_PLANT_COTTON_CANDY_FOREST},
+{    WWT_TEXTBTN,    14,     2,   167,    29,    40, STR_0258_CANDY_FACTORY, STR_027B_CONSTRUCT_CANDY_FACTORY},
+{    WWT_TEXTBTN,    14,     2,   167,    42,    53, STR_0259_BATTERY_FARM, STR_027C_CONSTRUCT_BATTERY_FARM},
+{    WWT_TEXTBTN,    14,     2,   167,    55,    66, STR_025A_COLA_WELLS, STR_027D_CONSTRUCT_COLA_WELLS},
+{    WWT_TEXTBTN,    14,     2,   167,    68,    79, STR_025B_TOY_SHOP, STR_027E_CONSTRUCT_TOY_SHOP},
+{    WWT_TEXTBTN,    14,     2,   167,    81,    92, STR_025C_TOY_FACTORY, STR_027F_CONSTRUCT_TOY_FACTORY},
+{    WWT_TEXTBTN,    14,     2,   167,    94,   105, STR_025D_PLASTIC_FOUNTAINS, STR_0280_CONSTRUCT_PLASTIC_FOUNTAINS},
+{    WWT_TEXTBTN,    14,     2,   167,   107,   118, STR_025E_FIZZY_DRINK_FACTORY, STR_0281_CONSTRUCT_FIZZY_DRINK_FACTORY},
+{    WWT_TEXTBTN,    14,     2,   167,   120,   131, STR_025F_BUBBLE_GENERATOR, STR_0282_CONSTRUCT_BUBBLE_GENERATOR},
+{    WWT_TEXTBTN,    14,     2,   167,   133,   144, STR_0260_TOFFEE_QUARRY, STR_0283_CONSTRUCT_TOFFEE_QUARRY},
+{    WWT_TEXTBTN,    14,     2,   167,   146,   157, STR_0261_SUGAR_MINE, STR_0284_CONSTRUCT_SUGAR_MINE},
 {      WWT_LAST},
 };
 
@@ -1631,6 +1637,7 @@ static void MainToolbarWndProc(Window *w, WindowEvent *e)
 		case WKC_SHIFT | WKC_F12: ShowMusicWindow(); break;
 		case WKC_CTRL  | 'S': _make_screenshot = 1; break;
 		case WKC_CTRL  | 'G': _make_screenshot = 2; break;
+		case WKC_CTRL | WKC_ALT | 'C': if(!_networking) ShowCheatWindow(); break;
 		}
 	} break;
 	
@@ -1645,8 +1652,21 @@ static void MainToolbarWndProc(Window *w, WindowEvent *e)
 
 	case WE_ON_EDIT_TEXT: HandleOnEditText(e); break;
 
+	case WE_MOUSELOOP:
+		
+		if (((w->click_state) & 1) != (uint)!!_pause) {
+			w->click_state ^= (1 << 0);
+			SetWindowDirty(w);
+		}
+		
+		if (((w->click_state >> 1) & 1) != (uint)!!_fast_forward) {
+			w->click_state ^= (1 << 1);
+			SetWindowDirty(w);
+		}
+		break;
+
 	case WE_TIMEOUT:
-		UnclickSomeWindowButtons(w, ~1);
+		UnclickSomeWindowButtons(w, ~(1<<0 | 1<<1));
 		break;
 	}
 }
@@ -1810,6 +1830,8 @@ static void ScenEditToolbarWndProc(Window *w, WindowEvent *e)
 		case WKC_F9: ToolbarScenPlaceSign(w); break;
 		case WKC_F10: ShowMusicWindow(); break;
 		case WKC_F11: PlaceLandBlockInfo(); break;
+		case WKC_CTRL  | 'S': _make_screenshot = 1; break;
+		case WKC_CTRL  | 'G': _make_screenshot = 2; break;
 		}
 		break;
 	}
@@ -1824,6 +1846,14 @@ static void ScenEditToolbarWndProc(Window *w, WindowEvent *e)
 	} break;
 
 	case WE_ON_EDIT_TEXT: HandleOnEditText(e); break;
+
+	case WE_MOUSELOOP:
+		if (((w->click_state >> 1) & 1) != (uint)!!_fast_forward) {
+			w->click_state ^= (1 << 1);
+			SetWindowDirty(w);
+		}
+		break;
+
 	}
 }
 
@@ -1891,7 +1921,7 @@ void StatusBarWndProc(Window *w, WindowEvent *e)
 	case WE_PAINT:
 		DrawWindowWidgets(w);
 		SET_DPARAM16(0, _date);
-		DrawStringCentered(70, 1, ((_game_paused||_patches.status_long_date)?STR_00AF:STR_00AE), 0);
+		DrawStringCentered(70, 1, ((_pause||_patches.status_long_date)?STR_00AF:STR_00AE), 0);
 		
 		p = _local_player == 0xff ? NULL : DEREF_PLAYER(_local_player);
 
@@ -1904,7 +1934,7 @@ void StatusBarWndProc(Window *w, WindowEvent *e)
 		// Draw status bar
 		if (_do_autosave) {
 			DrawStringCentered(320, 1,	STR_032F_AUTOSAVE, 0);
-		} else if (_game_paused) {
+		} else if (_pause) {
 			DrawStringCentered(320, 1,	STR_0319_PAUSED, 0);
 		} else if (WP(w,def_d).data_1 > -1280 && FindWindowById(WC_NEWS_WINDOW,0) == NULL && _statusbar_news_item.string_id != 0) {
 			// Draw the scrolling news text
@@ -1931,7 +1961,7 @@ void StatusBarWndProc(Window *w, WindowEvent *e)
 		break;
 
 	case WE_TICK: {
-		if (_game_paused || WP(w,def_d).data_1 <= -1280)
+		if (_pause || WP(w,def_d).data_1 <= -1280)
 			return;
 		WP(w,def_d).data_1 -= 2;
 		InvalidateWidget(w, 1);
@@ -2005,6 +2035,35 @@ static void MainWindowWndProc(Window *w, WindowEvent *e) {
 	case WE_KEYPRESS:
 		if (_game_mode == GM_MENU)
 			break;
+
+		// this is a hack, but this needs to be called at a constant interval and i found
+		// no other window event that was suited for that purpose.
+		{
+			static const int8 scrollamt[16][2] = {
+				{0,0},
+				{-1,0},   // 1:left
+				{0,-1},   // 2:up
+				{-1,-1}, // 3:left + up
+				{1,0},    // 4:right
+				{0,0},     // 5:left + right
+				{1,-1},  // 6:right + up
+				{0,0},     // 7:impossible
+				{0,1},    // 8:down
+				{-1,1},  // 9:down+left
+				{0,0},     // 10:impossible
+				{0,0},     // 11:impossible
+				{1,1},   // 12:down+right
+				{0,0},     // 13:impossible
+				{0,0},     // 14:impossible
+				{0,0},     // 15:impossible
+			};
+
+			if (_dirkeys) {
+				int factor = _shift_pressed ? 50 : 10;
+				ScrollMainViewport(scrollamt[_dirkeys][0] * factor, scrollamt[_dirkeys][1] * factor);
+			}
+		}
+
 		switch(e->keypress.keycode) {
 		case 'C':
 		case 'Z': {
@@ -2021,13 +2080,16 @@ static void MainWindowWndProc(Window *w, WindowEvent *e) {
 		case WKC_ESC: ResetObjectToPlace(); break;
 		case WKC_DELETE: DeleteNonVitalWindows(); break;
 		case 'Q' | WKC_CTRL: AskExitGame(); break;
+		case 'Q' | WKC_META: AskExitGame(); break; // this enables command + Q on mac
 		case 'R' | WKC_CTRL: MarkWholeScreenDirty(); break;
 		case '0' | WKC_ALT:
 		case '1' | WKC_ALT:
 		case '2' | WKC_ALT:
 		case '3' | WKC_ALT:
 		case '4' | WKC_ALT:
+#if defined(_DEBUG)
 			DebugProc(e->keypress.keycode - ('0' | WKC_ALT));
+#endif
 			break;
 		case 'A' | WKC_CTRL:
 			disable_computer^=1;
@@ -2038,16 +2100,6 @@ static void MainWindowWndProc(Window *w, WindowEvent *e) {
 			ShowInfoF("Play as computer: %s", _local_player?"ON":"OFF");
 			break;
 
-		case WKC_UP: ScrollMainViewport(0, -10); break;
-		case WKC_DOWN: ScrollMainViewport(0, 10); break;
-		case WKC_LEFT: ScrollMainViewport(-10, 0); break;
-		case WKC_RIGHT: ScrollMainViewport(10, 0); break;
-
-		case WKC_UP | WKC_SHIFT: ScrollMainViewport(0, -50); break;
-		case WKC_DOWN | WKC_SHIFT: ScrollMainViewport(0, 50); break;
-		case WKC_LEFT | WKC_SHIFT: ScrollMainViewport(-50, 0); break;
-		case WKC_RIGHT | WKC_SHIFT: ScrollMainViewport(50, 0); break;
-
 		// invoke autorail
 		case 'A':
 			ShowBuildRailToolbar(_last_built_railtype, 0);
@@ -2063,6 +2115,7 @@ static void MainWindowWndProc(Window *w, WindowEvent *e) {
 		}
 		e->keypress.cont = false;
 		break;
+				
 	}
 }
 
