@@ -69,6 +69,7 @@ void InitializeTrees();
 void InitializeStations();
 void InitializeNameMgr();
 void InitializePlayers();
+void InitializeCheats();
 
 void GenerateLandscape();
 void GenerateClearTile();
@@ -88,9 +89,8 @@ void InitializeGame()
 {
 	SetObjectToPlace(1, 0, 0, 0);
 
-	_game_paused = 0;
+	_pause = 0;
 	_fast_forward = 0;
-	_is_paused = false;
 	_tick_counter = 0;
 	_date_fract = 0;
 	_cur_tileloop_tile = 0;
@@ -124,6 +124,7 @@ void InitializeGame()
 	InitializeTrains();
 
 	InitializePlayers();
+	InitializeCheats();
 
 	InitTextEffects();
 	InitializeAnimatedTiles();
@@ -206,6 +207,13 @@ byte *GetName(int id, byte *buff)
 	return buff - 1;
 }
 
+
+void InitializeCheats()
+{
+	memset(&_cheats, 0, sizeof(Cheats)); 
+}
+
+
 void InitializeNameMgr()
 {
 	memset(_name_array, 0, sizeof(_name_array));
@@ -245,7 +253,7 @@ StringID AllocateName(const byte *name, byte skip)
 	return free_item | 0x7800 | (skip << 8);
 }
 
-const int16 _tileoffs_by_dir[4] = {
+const TileIndexDiff _tileoffs_by_dir[4] = {
 	TILE_XY(-1, 0),
 	TILE_XY(0, 1),
 	TILE_XY(1, 0),
@@ -415,12 +423,14 @@ void InitializeLandscapeVariables(bool only_constants)
 	}
 }
 
+// distance in Manhattan metric
 uint GetTileDist(TileIndex xy1, TileIndex xy2)
 {
 	return myabs(GET_TILE_X(xy1) - GET_TILE_X(xy2)) + 
 				myabs(GET_TILE_Y(xy1) - GET_TILE_Y(xy2));
 }
 
+// maximum distance in x _or_ y
 uint GetTileDist1D(TileIndex xy1, TileIndex xy2)
 {
 	return max(myabs(GET_TILE_X(xy1) - GET_TILE_X(xy2)),
@@ -447,8 +457,8 @@ uint GetTileDistAdv(TileIndex xy1, TileIndex xy2)
 
 bool CheckDistanceFromEdge(TileIndex tile, uint distance)
 {
-	return IS_INT_INSIDE(GET_TILE_X(tile), distance, TILES_X - distance) &&
-			IS_INT_INSIDE(GET_TILE_Y(tile), distance, TILES_Y - distance);
+	return IS_INT_INSIDE(GET_TILE_X(tile), distance, TILE_X_MAX + 1 - distance) &&
+			IS_INT_INSIDE(GET_TILE_Y(tile), distance, TILE_Y_MAX + 1 - distance);
 }
 
 void OnNewDay_Train(Vehicle *v);
@@ -504,9 +514,19 @@ void IncreaseDate()
 		return;
 	}
 
-	/* if the day changed, call the vehicle event (but max 12 at a time) */
+	/*if the day changed, call the vehicle event but only update a part of the vehicles
+		old max was i!= 12. But with that and a bigger number of vehicles (2560), per day only
+		a part of it could be done, namely: function called max_size date_fract (uint16) / 885 x 12 ==>
+		65536 / 885 = 74; 74x12 = 888. So max 888. Any vehicles above that were not _on_new_vehicle_day_proc'd
+		eg. aged.
+		So new code updates it for max vehicles.
+		(NUM_VEHICLES / maximum number of times ctr is incremented before reset ) + 1 (to get last vehicles too)
+		max size of _date_fract / 885 (added each tick) is number of times before ctr is reset.
+		Calculation might look complicated, but compiler just replaces it with 35, so that's ok
+	*/
+
 	ctr = _vehicle_id_ctr_day;
-	for(i=0; i!=12 && ctr != lengthof(_vehicles); i++) {
+	for(i=0; i!=(NUM_VEHICLES / ((1<<sizeof(_date_fract)*8) / 885)) + 1 && ctr != lengthof(_vehicles); i++) {
 		Vehicle *v = &_vehicles[ctr++];
 		if ((t=v->type) != 0)
 			_on_new_vehicle_day_proc[t - 0x10](v);
@@ -632,7 +652,9 @@ static void Load_NAME()
 }
 
 static const byte _game_opt_desc[] = {
-	SLE_ARR(GameOptions,diff,						SLE_FILE_I16 | SLE_VAR_I32, 17),
+	// added a new difficulty option (town attitude) in version 4
+	SLE_CONDARR(GameOptions,diff,						SLE_FILE_I16 | SLE_VAR_I32, 17, 0, 3),
+	SLE_CONDARR(GameOptions,diff,						SLE_FILE_I16 | SLE_VAR_I32, 18, 4, 255),
 	SLE_VAR(GameOptions,diff_level,			SLE_UINT8),
 	SLE_VAR(GameOptions,currency,				SLE_UINT8),
 	SLE_VAR(GameOptions,kilometers,			SLE_UINT8),
@@ -640,7 +662,7 @@ static const byte _game_opt_desc[] = {
 	SLE_VAR(GameOptions,landscape,			SLE_UINT8),
 	SLE_VAR(GameOptions,snow_line,			SLE_UINT8),
 	SLE_VAR(GameOptions,autosave,				SLE_UINT8),
-	SLE_VAR(GameOptions,road_side,				SLE_UINT8),
+	SLE_VAR(GameOptions,road_side,			SLE_UINT8),
 	SLE_END()
 };
 
@@ -652,21 +674,22 @@ static void SaveLoad_OPTS()
 
 
 static const SaveLoadGlobVarList _date_desc[] = {
-	{&_date, SLE_UINT16},
-	{&_date_fract, SLE_UINT16},
-	{&_tick_counter, SLE_UINT16},
-	{&_vehicle_id_ctr_day, SLE_UINT16},
-	{&_age_cargo_skip_counter, SLE_UINT8},
-	{&_avail_aircraft, SLE_UINT8},
-	{&_cur_tileloop_tile, SLE_UINT16},
-	{&_disaster_delay, SLE_UINT16},
-	{&_station_tick_ctr, SLE_UINT16},
-	{&_random_seed_1, SLE_UINT32},
-	{&_random_seed_2, SLE_UINT32},
-	{&_cur_town_ctr, SLE_UINT8},
-	{&_cur_player_tick_index, SLE_FILE_U8 | SLE_VAR_UINT},
-	{&_next_competitor_start, SLE_FILE_U16 | SLE_VAR_UINT},
-	{&_trees_tick_ctr, SLE_UINT8},
+	{&_date, 										SLE_UINT16, 0, 255},
+	{&_date_fract, 							SLE_UINT16, 0, 255},
+	{&_tick_counter, 						SLE_UINT16, 0, 255},
+	{&_vehicle_id_ctr_day, 			SLE_UINT16, 0, 255},
+	{&_age_cargo_skip_counter, 	SLE_UINT8,	0, 255},
+	{&_avail_aircraft, 					SLE_UINT8,	0, 255},
+	{&_cur_tileloop_tile, 			SLE_UINT16, 0, 255},
+	{&_disaster_delay, 					SLE_UINT16, 0, 255},
+	{&_station_tick_ctr, 				SLE_UINT16, 0, 255},
+	{&_random_seed_1, 					SLE_UINT32, 0, 255},
+	{&_random_seed_2, 					SLE_UINT32, 0, 255},
+	{&_cur_town_ctr, 						SLE_UINT8,	0, 255},
+	{&_cur_player_tick_index, 	SLE_FILE_U8 | SLE_VAR_UINT, 0, 255},
+	{&_next_competitor_start, 	SLE_FILE_U16 | SLE_VAR_UINT, 0, 255},
+	{&_trees_tick_ctr, 					SLE_UINT8,	0, 255},
+	{&_pause, 									SLE_UINT8,	4, 255},
 	{NULL,0}
 };
 
@@ -679,9 +702,9 @@ static void SaveLoad_DATE()
 
 
 static const SaveLoadGlobVarList _view_desc[] = {
-	{&_saved_scrollpos_x, SLE_FILE_I16 | SLE_VAR_INT},
-	{&_saved_scrollpos_y, SLE_FILE_I16 | SLE_VAR_INT},
-	{&_saved_scrollpos_zoom, SLE_UINT8},
+	{&_saved_scrollpos_x, SLE_FILE_I16 | SLE_VAR_INT, 0, 255},
+	{&_saved_scrollpos_y, SLE_FILE_I16 | SLE_VAR_INT, 0, 255},
+	{&_saved_scrollpos_zoom, SLE_UINT8, 0, 255},
 	{NULL,0}
 };
 
@@ -718,6 +741,33 @@ static void SaveLoad_MAPE() {
   SlArray(_map_extra_bits, lengthof(_map_extra_bits), SLE_UINT8);
 }
 
+
+static void Save_CHTS()
+{
+	byte count = sizeof(_cheats)/sizeof(Cheat);
+	Cheat* cht = (Cheat*) &_cheats;
+	Cheat* cht_last = &cht[count];
+
+	SlSetLength(count*2);
+	for(; cht != cht_last; cht++) {
+		SlWriteByte(cht->been_used);
+		SlWriteByte(cht->value);
+	}
+}
+
+static void Load_CHTS()
+{
+	Cheat* cht = (Cheat*) &_cheats;
+
+	uint count = SlGetFieldLength()/2;
+	for(; count; count--, cht++)
+	{
+		cht->been_used = (byte)SlReadByte();
+		cht->value = (byte)SlReadByte();
+	}
+}
+
+
 const ChunkHandler _misc_chunk_handlers[] = {
 	{ 'MAPT', SaveLoad_MAPT, SaveLoad_MAPT, CH_RIFF },
 	{ 'MAP2', SaveLoad_MAP2, SaveLoad_MAP2, CH_RIFF },
@@ -730,6 +780,6 @@ const ChunkHandler _misc_chunk_handlers[] = {
 	{ 'NAME', Save_NAME, Load_NAME, CH_ARRAY},
 	{ 'DATE', SaveLoad_DATE, SaveLoad_DATE, CH_RIFF},
 	{ 'VIEW', SaveLoad_VIEW, SaveLoad_VIEW, CH_RIFF},
-	{ 'OPTS', SaveLoad_OPTS, SaveLoad_OPTS, CH_RIFF | CH_LAST}
+	{ 'OPTS', SaveLoad_OPTS, SaveLoad_OPTS, CH_RIFF},
+	{ 'CHTS', Save_CHTS, Load_CHTS, CH_RIFF | CH_LAST}
 };
-
