@@ -132,7 +132,7 @@ static void Place_LandInfo(uint tile)
 	WP(w,void_d).data = &lid;
 	
 	lid.tile = tile;
-	lid.city = ClosestCityFromTile(tile, 20);
+	lid.city = ClosestCityFromTile(tile, _patches.dist_local_authority);
 
 	if (_local_player == 255) {
 		lid.costclear = 0;
@@ -147,6 +147,19 @@ static void Place_LandInfo(uint tile)
 
 	GetAcceptedCargo(tile, &lid.ac);
 	GetTileDesc(tile, &lid.td);
+	
+#if defined(_DEBUG)
+	{
+		printf("TILE: %#x (%i,%i)\n", tile, GET_TILE_X(tile), GET_TILE_Y(tile));
+		printf("_map_type_and_height=%#x\n", _map_type_and_height[tile]);
+		printf("_map2=%#x\n", _map2[tile]);
+		printf("_map3_lo=%#x\n", _map3_lo[tile]);
+		printf("_map3_hi=%#x\n", _map3_hi[tile]);
+		printf("_map5=%#x\n", _map5[tile]);
+		printf("_map_owner=%#x\n", _map_owner[tile]);
+		printf("\n");
+	}
+#endif
 }
 
 void PlaceLandBlockInfo()
@@ -836,7 +849,7 @@ void ShowQueryString(StringID str, StringID caption, int maxlen, int maxwidth, b
 
 #define _orig_edit_str_buf (_edit_str_buf+MAX_QUERYSTR_LEN)
 
-	if (_game_paused != 0 && !_patches.build_in_pause)
+	if (_pause != 0 && !_patches.build_in_pause)
 		return;
 
 	DeleteWindowById(WC_QUERY_STRING, 0);
@@ -921,7 +934,10 @@ static int _saveload_mode;
 static void BuildFileList()
 {
 	FiosFreeSavegameList();
-	_fios_list = FiosGetSavegameList(&_fios_num, _saveload_mode);
+	if(_saveload_mode==SLD_NEW_GAME || _saveload_mode==SLD_LOAD_SCENARIO || _saveload_mode==SLD_SAVE_SCENARIO)
+		_fios_list = FiosGetScenarioList(&_fios_num, _saveload_mode);
+	else
+		_fios_list = FiosGetSavegameList(&_fios_num, _saveload_mode);
 }
 
 static void DrawFiosTexts()
@@ -935,7 +951,7 @@ static void DrawFiosTexts()
 	DoDrawString(path, 2, 15, 16);
 }
 
-static const byte _fios_colors[] = {13, 9, 9, 6, 5};
+static const byte _fios_colors[] = {13, 9, 9, 6, 5, 5};
 
 static void SaveLoadDlgWndProc(Window *w, WindowEvent *e)
 {
@@ -1024,7 +1040,7 @@ static void SaveLoadDlgWndProc(Window *w, WindowEvent *e)
 		}
 		break;
 	case WE_DESTROY:
-		UnpauseGame();
+		DoCommandP(0, 0, 0, NULL, CMD_PAUSE);
 		_query_string_active = false;
 		FiosFreeSavegameList();
 		break;
@@ -1098,7 +1114,7 @@ void ShowSaveLoadDialog(int mode)
 		strcpy(_edit_str_buf, "UNNAMED");
 	}
 
-	PauseGame();
+	DoCommandP(0, 1, 0, NULL, CMD_PAUSE);
 
 	BuildFileList();
 
@@ -1110,3 +1126,350 @@ void RedrawAutosave()
 	SetWindowDirty(FindWindowById(WC_STATUS_BAR, 0));
 }
 
+
+
+
+
+static const Widget _select_scenario_widgets[] = {
+{    WWT_TEXTBTN,     7,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,     7,    11,   256,     0,    13, STR_400E_SELECT_NEW_GAME_TYPE, STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,     7,     0,   245,    14,   319, 0x0},
+{          WWT_6,     7,     2,   243,    16,   317, 0x0, STR_400F_SELECT_SCENARIO_GREEN_PRE},
+{  WWT_SCROLLBAR,     7,   246,   256,    14,   319, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{      WWT_LAST},
+};
+
+
+static void SelectScenarioWndProc(Window *w, WindowEvent *e) {
+	switch(e->event) {
+	case WE_PAINT:
+	{
+		int y,pos;
+		const FiosItem *item;
+		
+		SetVScrollCount(w, _fios_num);
+
+		DrawWindowWidgets(w);
+		DrawString(4, 20, STR_4010_GENERATE_RANDOM_NEW_GAME, 9);
+
+		y = 33;
+		pos = w->vscroll.pos;
+		while (pos < _fios_num) {
+			item = _fios_list + pos;
+			DoDrawString(item->title[0] ? item->title : item->name, 4, y, _fios_colors[item->type] );
+			pos++;
+			y+=10;
+			if (y >= 28*10+33)
+				break;
+		}
+	}
+		break;
+
+	case WE_CLICK:
+		switch(e->click.widget) {
+		case 0: // Close button
+			
+			// Close the choose game dialog.
+			DoCommandP(0, 1, 0, NULL, CMD_START_NEW_GAME);
+			break;
+		
+		case 1: // Drag the window
+			StartWindowDrag(w);
+			break;
+
+		case 3: {// Click the listbox
+			if(e->click.pt.y<33)
+			{
+				DoCommandP(0, Random(), InteractiveRandom(), NULL, CMD_GEN_RANDOM_NEW_GAME);
+			}
+			else
+			{
+				char *name;
+				int y = (e->click.pt.y - 33) / 10;
+				if(y<=_fios_num)	{
+				if ((name = FiosBrowseTo(_fios_list + y)) != NULL) {
+							_switch_mode = SM_LOAD;
+							DeleteWindow(w);
+							strcpy(_file_to_saveload, name);
+							DoCommandP(0, Random(), InteractiveRandom(), NULL, CMD_START_SCENARIO);
+							}
+				}
+			}
+			break;
+		}			
+		}
+	case WE_DESTROY:
+		break;
+	}
+}
+
+static const WindowDesc _select_scenario_desc = {
+	WDP_CENTER, WDP_CENTER, 257, 320,
+	WC_SAVELOAD,0,
+	WDF_STD_TOOLTIPS | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
+	_select_scenario_widgets,
+	SelectScenarioWndProc
+};
+
+void AskForNewGameToStart()
+{
+	Window *w;
+
+	DeleteWindowById(WC_QUERY_STRING, 0);
+	DeleteWindowById(WC_SAVELOAD, 0);
+
+	_saveload_mode = SLD_NEW_GAME;
+	BuildFileList();
+
+	w = AllocateWindowDesc(&_select_scenario_desc);
+	w->vscroll.cap = 28;
+
+}
+
+
+
+
+int32 ClickMoneyCheat(int32 p1, int32 p2)
+{
+		DoCommandP(0, -10000000, 0, NULL, CMD_MONEY_CHEAT);
+		return true;
+}
+
+// p1 player to set to, p2 is -1 or +1 (down/up)
+int32 ClickChangePlayerCheat(int32 p1, int32 p2)
+{
+	while(p1 >= 0 && p1 < MAX_PLAYERS)
+	{
+		if (_players[p1].is_active)
+		{
+			_local_player = p1;
+			return _local_player;
+		}
+		p1 += p2;
+	}
+return _local_player;
+}
+
+typedef int32 CheckButtonClick(int32, int32);
+static CheckButtonClick * const _cheat_button_proc[] = {
+	ClickMoneyCheat,
+	ClickChangePlayerCheat,
+};
+
+
+typedef struct CheatEntry {
+	byte type;    // type of selector
+	byte flags;		// selector flags
+	StringID str; // string with descriptive text
+	void *variable; // pointer to the variable
+	bool *been_used; // has this cheat been used before?
+	CheckButtonClick *click_proc; // procedure
+	int16 min,max; // range for spinbox setting
+	uint16 step;   // step for spinbox
+} CheatEntry;
+
+enum {
+	CE_BOOL = 0,
+	CE_UINT8 = 1,
+	CE_INT16 = 2,
+	CE_UINT16 = 3,
+	CE_INT32 = 4,
+	CE_BYTE = 5,
+	CE_CLICK = 6,
+
+	CF_0ISDIS = 1,
+	CF_NOCOMMA = 2,
+};
+
+static int32 ReadCE(const CheatEntry*ce)
+{
+	switch(ce->type) {
+	case CE_BOOL:   return *(bool*)ce->variable;
+	case CE_UINT8:  return *(uint8*)ce->variable;
+	case CE_INT16:  return *(int16*)ce->variable;
+	case CE_UINT16: return *(uint16*)ce->variable;
+	case CE_INT32:  return *(int32*)ce->variable;
+	case CE_BYTE:   return *(byte*)ce->variable;
+	case CE_CLICK:  return 0;
+	default:
+		NOT_REACHED();
+	}
+  
+	/* useless, but avoids compiler warning this way */
+	return 0;
+}
+
+static void WriteCE(const CheatEntry *ce, int32 val)
+{
+	switch(ce->type) {
+	case CE_BOOL: *(bool*)ce->variable = (bool)val; break;
+	case CE_BYTE: *(byte*)ce->variable = (byte)val; break;
+	case CE_UINT8: *(uint8*)ce->variable = (uint8)val; break;
+	case CE_INT16: *(int16*)ce->variable = (int16)val; break;
+	case CE_UINT16: *(uint16*)ce->variable = (uint16)val; break;
+	case CE_INT32: *(int32*)ce->variable = val; break;
+	case CE_CLICK: break;
+	default:
+		NOT_REACHED();
+	}
+}
+
+
+static const CheatEntry _cheats_ui[] = {
+	{CE_CLICK, 0, STR_CHEAT_MONEY, 					&_cheats.money.value, 					&_cheats.money.been_used, 				&ClickMoneyCheat},
+	{CE_UINT8, 0, STR_CHEAT_CHANGE_PLAYER, 	&_local_player, 								&_cheats.switch_player.been_used,	&ClickChangePlayerCheat, 0, 11, 1},
+	{CE_BOOL, 0, STR_CHEAT_EXTRA_DYNAMITE,	&_cheats.magic_bulldozer.value,	&_cheats.magic_bulldozer.been_used},
+	{CE_BOOL, 0, STR_CHEAT_CROSSINGTUNNELS,	&_cheats.crossing_tunnels.value,&_cheats.crossing_tunnels.been_used},
+};
+
+
+static const Widget _cheat_widgets[] = {
+{   WWT_CLOSEBOX,    14,     0,    10,     0,    13, STR_00C5,			STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   399,     0,    13, STR_CHEATS,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{      WWT_PANEL,    14,     0,   399,    14,   147, 0x0},
+
+{     WWT_IMGBTN,    14,     0,   399,    14,   147, 0x0, STR_CHEATS_TIP},
+{      WWT_LAST},
+};
+
+extern void DrawPlayerIcon(int p, int x, int y);
+
+static void CheatsWndProc(Window *w, WindowEvent *e)
+{
+	switch(e->event) {
+	case WE_PAINT: {
+		int clk = WP(w,def_d).data_1;
+		const CheatEntry *ce = &_cheats_ui[0];
+		int32 val;
+		int x, y;
+		int i;
+
+		DrawWindowWidgets(w);
+
+		DrawStringMultiCenter(200, 25, STR_CHEATS_WARNING, 350);
+
+		x=0;
+		y=45;
+
+		for(i=0; i!=lengthof(_cheats_ui); i++,ce++) {
+
+			DrawSprite((SPR_OPENTTD_BASE + ((*ce->been_used)?67:66)), x+5, y+2);
+
+			if (ce->type == CE_BOOL) {
+				DrawFrameRect(x+20, y+1, x+30+9, y+9, (*(bool*)ce->variable)?6:4, (*(bool*)ce->variable)?0x20:0);
+				SET_DPARAM16(0, *(bool*)ce->variable ? STR_CONFIG_PATCHES_ON : STR_CONFIG_PATCHES_OFF);
+
+			}	else if (ce->type == CE_CLICK) {
+				DrawFrameRect(x+20, y+1, x+30+9, y+9, 0, (WP(w,def_d).data_1==i*2+1)?0x20:0x00);
+				if(i==0)
+					SET_DPARAM64(0, (int64) 10000000);
+				else
+					SET_DPARAM16(0, false);
+
+			} else {
+				DrawFrameRect(x+20, y+1, x+20+9, y+9, 3, clk == i*2+1 ? 0x20 : 0);
+				DrawFrameRect(x+30, y+1, x+30+9, y+9, 3, clk == i*2+2 ? 0x20 : 0);
+				DrawStringCentered(x+25, y+1, STR_6819, 0);
+				DrawStringCentered(x+35, y+1, STR_681A, 0);
+				
+				val = ReadCE(ce);
+				SET_DPARAM16(0, val);
+				
+				// draw colored flag for change player cheat
+				if(ce->str==STR_CHEAT_CHANGE_PLAYER)
+					DrawPlayerIcon(_current_player, 156, y+2);
+
+				
+			}
+
+		DrawString(50, y+1, ce->str, 0);
+
+		y+=12;
+		}
+		break;
+	}
+	case WE_CLICK: {
+			const CheatEntry *ce;
+			uint btn = (e->click.pt.y - 46) / 12;
+			int32 val, oval;
+			uint x = e->click.pt.x;
+
+			// not clicking a button?
+			if(!IS_INT_INSIDE(x, 20, 40) || btn>=lengthof(_cheats_ui))
+				break;
+
+			ce = &_cheats_ui[btn];
+			oval = val = ReadCE(ce);
+
+			*ce->been_used = true;
+
+			switch(ce->type) {
+				case CE_BOOL:	{
+					val ^= 1;
+					if (ce->click_proc != NULL)
+						ce->click_proc(val, 0);
+					break;
+				}
+
+				case CE_CLICK:	{
+					ce->click_proc(val, 0);
+					WP(w,def_d).data_1 = btn * 2 + 1;
+					break;
+				}
+
+				default:	{
+					if (x >= 30) {
+						//increase
+						val += ce->step;
+						if (val > ce->max) val = ce->max;
+					} else {
+						// decrease
+						val -= ce->step;
+						if (val < ce->min) val = ce->min;
+					}
+
+					// take whatever the function returns
+					val = ce->click_proc(val, (x>=30) ? 1 : -1);
+
+					if (val != oval)
+						WP(w,def_d).data_1 = btn * 2 + 1 + ((x>=30) ? 1 : 0);
+
+					break;
+				}
+			}
+
+			if (val != oval) {
+				WriteCE(ce, val);
+				SetWindowDirty(w);
+			}
+
+			w->flags4 |= 5 << WF_TIMEOUT_SHL;
+
+			SetWindowDirty(w);
+		}
+		break;
+	case WE_TIMEOUT:
+		WP(w,def_d).data_1 = 0;
+		SetWindowDirty(w);
+		break;
+	}	
+}
+static const WindowDesc _cheats_desc = {
+	240, 22, 400, 148,
+	WC_CHEATS,0,
+	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
+	_cheat_widgets,
+	CheatsWndProc
+};
+
+
+void ShowCheatWindow()
+{
+	Window *w;
+	
+	DeleteWindowById(WC_CHEATS, 0);
+	w = AllocateWindowDesc(&_cheats_desc);
+
+	if (w)
+		SetWindowDirty(w);
+}
