@@ -148,59 +148,415 @@ static byte MakeEnglishCityName(byte *buf, uint32 seed)
 	if (start[0]=='C' && (start[1] == 'e' || start[1] == 'i'))
 		start[0] = 'K'; 
 
-	/* FIXME: skip the banned words thing for now */
+	/* FIXME: skip the banned words thing for now 
+	only replacing "Cunt" with "Cult"   */
+	if (start[0]=='C' && start[1] == 'u' && start[2] == 'n' && start[3] == 't')
+		start[2] = 'l';
+
 	return result;
 }
 
-#define NUM_GERMAN_1 23
-static const char german_1[] = 
-	MK("Bruns")
-	MK("Lim")
-	MK("Han")
-	MK("Brun")
-	MK("Ober")
-	MK("Frank")
-	MK("Rott")
-	MK("Frei")
-	MK("Ravens")
-	MK("Schwein")
-	MK("Osna")
-	MK("Dsseldorf")
-	MK("Wester")
-	MK("Flens")
-	MK("Mhl")
-	MK("Heidel")
-	MK("Inns")
-	MK("Cloppen")
-	MK("Pfung")
-	MK("Michel")
-	MK("Wert")
-	MK("Wildes")
-	MK("Freuden")
+#define NUM_AUSTRIAN_A1 6
+static const char austrian_a1[] = 
+	MK("Bad ")
+	MK("Deutsch ")
+	MK("Gross ")
+	MK("Klein ")
+	MK("Markt ")
+	MK("Maria ")
 ;
 
-#define NUM_GERMAN_2 14
+#define NUM_AUSTRIAN_A2 42
+static const char austrian_a2[] = 
+	MK("Aus")
+	MK("Alten")
+	MK("Braun")
+	MK("Vsl")
+	MK("Mittern")
+	MK("Nuss")
+	MK("Neu")
+	MK("Walters")
+	MK("Breiten")
+	MK("Eisen")
+	MK("Feld")
+	MK("Mittern")
+	MK("Gall")
+	MK("Obern")
+	MK("Grat")
+	MK("Heiligen")
+	MK("Hof")
+	MK("Holla")
+	MK("Stein")
+	MK("Eber")
+	MK("Eggen")
+	MK("Enzers")
+	MK("Frauen")
+	MK("Herren")
+	MK("Hof")
+	MK("Htt")
+	MK("Kaisers")
+	MK("Knigs")
+	MK("Knittel")
+	MK("Lang")
+	MK("Ober")
+	MK("Ollers")
+	MK("Pfaffen")
+	MK("Potten")
+	MK("Salz")
+	MK("Schwarz")
+	MK("Stocker")
+	MK("Unter")
+	MK("Utten")
+	MK("Vsen")
+	MK("Vill")
+	MK("Weissen")
+;
+
+#define NUM_AUSTRIAN_A3 16
+static const char austrian_a3[] = 
+	MK("see")
+	MK("bach")
+	MK("dorf")
+	MK("ach")
+	MK("stein")
+	MK("hofen")
+	MK("au")
+	MK("ach")
+	MK("kirch")
+	MK("kirchen")
+	MK("kreuz")
+	MK("brunn")
+	MK("siedl")
+	MK("markt")
+	MK("wang")
+	MK("haag")
+;
+
+#define NUM_AUSTRIAN_A4 12
+static const char austrian_a4[] =
+	MK("Bruck")
+	MK("Brunn")
+	MK("Gams")
+	MK("Grein")
+	MK("Ried")
+	MK("Faak")
+	MK("Zell")
+	MK("Spital")
+	MK("Kirchberg")
+	MK("Saal")
+	MK("Taferl")
+	MK("Wald")
+;
+
+#define NUM_AUSTRIAN_A5 2
+static const char austrian_a5[] =
+	MK("St. ")
+	MK("Sankt ")
+;
+
+#define NUM_AUSTRIAN_A6 20
+static const char austrian_a6[] =
+	MK("Aegyd")
+	MK("Andr")
+	MK("Georgen")
+	MK("Jakob")
+	MK("Johann")
+	MK("Leonhard")
+	MK("Marein")
+	MK("Lorenzen")
+	MK("Margarethen")
+	MK("Martin")
+	MK("Michael")
+	MK("Nikolai")
+	MK("Oswald")
+	MK("Peter")
+	MK("Plten")
+	MK("Stefan")
+	MK("Stephan")
+	MK("Thomas")
+	MK("Veit")
+	MK("Wolfgang")
+;
+
+#define NUM_AUSTRIAN_F1 2
+static const char austrian_f1[] =
+	MK(" an der ")
+	MK(" ob der ")
+;
+
+#define NUM_AUSTRIAN_F2 13
+static const char austrian_f2[] =
+	MK("Donau")
+	MK("Steyr")
+	MK("Lafnitz")
+	MK("Leitha")
+	MK("Thaya")
+	MK("Gail")
+	MK("Drau")
+	MK("Salzach")
+	MK("Ybbs")
+	MK("Traisen")
+	MK("Enns")
+	MK("Mur")
+	MK("Ill")
+;
+
+#define NUM_AUSTRIAN_B1 1
+static const char austrian_b1[] =
+	MK(" am ")
+;
+
+#define NUM_AUSTRIAN_B2 10
+static const char austrian_b2[] =
+	MK("Brenner")
+	MK("Dachstein")
+	MK("Gebirge")
+	MK("Grossglockner")
+	MK("Hausruck")
+	MK("Semmering")
+	MK("Wagram")
+	MK("Wechsel")
+	MK("Wilden Kaiser")
+	MK("Ziller")
+;
+
+static byte MakeAustrianCityName(byte *buf, uint32 seed)
+{
+	int i, j=0;
+
+	// Bad, Maria, Gross, ...
+	i = GETNUM(0, NUM_AUSTRIAN_A1 + 15) - 15;
+	if (i >= 0) AppendPart(&buf, i, austrian_a1);
+
+	i = GETNUM(4, 6);
+	if(i >= 4) {
+	  // Kaisers-kirchen
+	  AppendPart(&buf, GETNUM( 7, NUM_AUSTRIAN_A2), austrian_a2);
+	  AppendPart(&buf, GETNUM(13, NUM_AUSTRIAN_A3), austrian_a3);
+	}
+	else if(i >= 2) {
+	  // St. Johann
+	  AppendPart(&buf, GETNUM( 7, NUM_AUSTRIAN_A5), austrian_a5);
+	  AppendPart(&buf, GETNUM( 9, NUM_AUSTRIAN_A6), austrian_a6);
+	  j=1; // More likely to have a " an der " or " am "
+	}
+        else {
+	  // Zell
+	  AppendPart(&buf, GETNUM( 7, NUM_AUSTRIAN_A4), austrian_a4);
+	}
+
+	i = GETNUM(1, 6);
+	if(i >= 4-j) {
+	  // an der Donau (rivers)
+	  AppendPart(&buf, GETNUM(4, NUM_AUSTRIAN_F1), austrian_f1);
+	  AppendPart(&buf, GETNUM(5, NUM_AUSTRIAN_F2), austrian_f2);
+	}
+	else if(i >= 2-j) {
+	  // am Dachstein (mountains)
+	  AppendPart(&buf, GETNUM(4, NUM_AUSTRIAN_B1), austrian_b1);
+	  AppendPart(&buf, GETNUM(5, NUM_AUSTRIAN_B2), austrian_b2);
+	}
+
+	return 0;
+}
+
+#define NUM_GERMAN_1 114
+#define NUM_GERMAN_1_HARDCODED 21 
+static const char german_1[] = 
+	/* hardcoded names */
+	MK("Berlin")
+	MK("Bonn")
+	MK("Bremen")
+	MK("Cottbus")
+	MK("Chemnitz")
+	MK("Dortmund")
+	MK("Dresden")
+	MK("Erfurt")
+	MK("Erlangen")
+	MK("Essen")
+	MK("Fulda")
+	MK("Gera")
+	MK("Kassel")
+	MK("Kiel")
+	MK("Kln")
+	MK("Lbeck")
+	MK("Magdeburg")
+	MK("Mnchen")
+	MK("Potsdam")
+	MK("Stuttgart")
+	MK("Wiesbaden")
+	/* randomizer names */
+	MK("Alb")
+	MK("Als")
+	MK("Ander")
+	MK("Arns")
+	MK("Bruns")
+	MK("Bam")
+	MK("Biele")
+	MK("Cloppen")
+	MK("Co")
+	MK("Duis")
+	MK("Dssel")
+	MK("Dannen")
+	MK("Elb")
+	MK("Els")
+	MK("Elster")
+	MK("Eichen")
+	MK("Ems")
+	MK("Fahr")
+	MK("Falken")
+	MK("Flens")
+	MK("Frank")
+	MK("Frei")
+	MK("Freuden")
+	MK("Fried")
+	MK("Frsten")
+	MK("Hahn")
+	MK("Ham")
+	MK("Harz")
+	MK("Heidel")
+	MK("Hers")
+	MK("Herz")
+	MK("Holz")
+	MK("Hildes")
+	MK("Inns")
+	MK("Ilsen")
+	MK("Ingols")
+	MK("Kel")
+	MK("Kies")
+	MK("Korn")
+	MK("Kor")
+	MK("Kreuz")
+	MK("Kulm")
+	MK("Langen")
+	MK("Lim")
+	MK("Lohr")
+	MK("Lne")
+	MK("Mel")
+	MK("Michels")
+	MK("Mhl")
+	MK("Naum")
+	MK("Nest")
+	MK("Nord")
+	MK("Nort")
+	MK("Nien")
+	MK("Nidda")
+	MK("Nieder")
+	MK("Nrn")
+	MK("Ober")
+	MK("Offen")
+	MK("Osna")
+	MK("Olden")
+	MK("Ols")
+	MK("Oranien")
+	MK("Pader")
+	MK("Quedlin")
+	MK("Quer")
+	MK("Ravens")
+	MK("Regens")
+	MK("Rott")
+	MK("Ros")
+	MK("Rssels")
+	MK("Saal")
+	MK("Saar")
+	MK("Salz")
+	MK("Schne")
+	MK("Schwein")
+	MK("Sonder")
+	MK("Sonnen")
+	MK("Stein")
+	MK("Strals")
+	MK("Straus")
+	MK("Sd")
+	MK("Ton")
+	MK("Unter")
+	MK("Ur")
+	MK("Vor")
+	MK("Wald")
+	MK("War")
+	MK("Wert")
+	MK("Wester")
+	MK("Witten")
+	MK("Wolfs")
+	MK("Wrz")
+;
+
+#define NUM_GERMAN_2 16
 static const char german_2[] =
-	MK("gen")
-	MK("stadt")
-	MK("heim")
+	MK("bach")
+	MK("berg")
+	MK("brck")
+	MK("brcken")
 	MK("burg")
-	MK("haven")
-	MK("ford")
+	MK("dorf")
 	MK("feld")
+	MK("furt")
+	MK("hausen")
+	MK("haven")
+	MK("heim")
+	MK("horst")
 	MK("mund")
 	MK("mnster")
-	MK("hausen")
-	MK("furt")
-	MK("brck")
-	MK("over")
+	MK("stadt")
 	MK("wald")
 ;
 
+#define NUM_GERMAN_3 5
+static const char german_3[] =
+	MK(" an der ")
+	MK(" am ")
+	MK("Bad ")
+	MK("Klein ")
+	MK("Neu ")
+;
+
+#define NUM_GERMAN_4 6
+#define NUM_GERMAN_4_PRECHANGE 5
+static const char german_4[] =
+	/* use "an der" */
+	MK("Oder")
+	MK("Spree")
+	MK("Donau")
+	MK("Saale")
+	MK("Elbe")
+	/* use "am" */
+	MK("Main")
+	
+;
+
 static byte MakeGermanCityName(byte *buf, uint32 seed)
 {
-	AppendPart(&buf, GETNUM(0, NUM_GERMAN_1), german_1);
-	AppendPart(&buf, GETNUM(5, NUM_GERMAN_2), german_2);
+	int i;
+	int ext;
+	
+	ext=GETNUM(7, 28); /* Extension - Prefix / Suffix */
+
+	if ((ext==12) || (ext==19)) {
+		i=GETNUM(2,NUM_GERMAN_3-2);
+		AppendPart(&buf, 2+i, german_3);
+		}
+
+
+	i=GETNUM(3,NUM_GERMAN_1);
+
+	AppendPart(&buf, i, german_1);
+
+	if (i>NUM_GERMAN_1_HARDCODED-1) {
+		AppendPart(&buf, GETNUM(5, NUM_GERMAN_2), german_2);
+		}
+
+	if (ext==24) {
+		i=GETNUM(9,NUM_GERMAN_4);
+
+		if (i<=NUM_GERMAN_4_PRECHANGE-1) {
+			AppendPart(&buf, 0, german_3);
+			AppendPart(&buf, i, german_4);
+			} else {
+			AppendPart(&buf, 1, german_3);
+			AppendPart(&buf, i, german_4);
+			}
+		}
+
 	return 0;
 }
 
@@ -1510,13 +1866,12 @@ static const char hungarian_2[] =
 	MK("Vsros")
 ;
 
-#define NUM_HUNGARIAN_3 17
+#define NUM_HUNGARIAN_3 16
 static const char hungarian_3[] = 
 	MK("apti")
 	MK("bba")
 	MK("bikk")
 	MK("dob")
-	MK("egy")
 	MK("fa")
 	MK("fld")
 	MK("hegyes")
@@ -1531,12 +1886,13 @@ static const char hungarian_3[] =
 	MK("vlgy")
 ;
 
-#define NUM_HUNGARIAN_4 3
+#define NUM_HUNGARIAN_4 5
 static const char hungarian_4[] = 
-	MK("vr")
 	MK("alja")
+	MK("egyhza")
 	MK("hza")
 	MK("r")
+	MK("vr")
 ;
 
 #define NUM_HUNGARIAN_REAL 35
@@ -1613,5 +1969,6 @@ CityNameGenerator * const _city_name_generators[] = {
 	MakeFinnishCityName,
 	MakePolishCityName,
 	MakeSlovakishCityName,
-	MakeHungarianCityName
+	MakeHungarianCityName,
+	MakeAustrianCityName
 };
