@@ -258,15 +258,13 @@ void TPFMode1(TrackPathFinder *tpf, uint tile, int direction)
 	RememberData rd;
 	uint tile_org = tile;
 
-	if (IS_TILETYPE(tile, MP_TUNNELBRIDGE) &&
-			(_map5[tile] & 0xF0)==0 && 
-			(_map5[tile] & 3) == direction &&
-			((_map5[tile]>>1)&6) == tpf->tracktype) {
+	if (IS_TILETYPE(tile, MP_TUNNELBRIDGE) && (_map5[tile] & 0xF0)==0) {
+		if ((_map5[tile] & 3) != direction || ((_map5[tile]>>1)&6) != tpf->tracktype)
+			return;
 		tile = SkipToEndOfTunnel(tpf, tile, direction);		
-	} else {
-		tile += _tileoffs_by_dir[direction];
-		tpf->rd.cur_length++;
 	}
+	tile += _tileoffs_by_dir[direction];
+	tpf->rd.cur_length++;
 
 	bits = GetTileTrackStatus(tile, tpf->tracktype);
 
@@ -592,14 +590,14 @@ void NTPEnum(NewTrackPathFinder *tpf, uint tile, uint direction)
 	si.state = 0;
 
 restart:
-	if (IS_TILETYPE(tile, MP_TUNNELBRIDGE) &&
-			(_map5[tile] & 0xF0)==0 && 
-			(uint)(_map5[tile] & 3) == direction &&
-			((_map5[tile]>>1)&6) == tpf->tracktype) {
+	if (IS_TILETYPE(tile, MP_TUNNELBRIDGE) && (_map5[tile] & 0xF0)==0) {
+		if ( (uint)(_map5[tile] & 3) != direction || ((_map5[tile]>>1)&6) != tpf->tracktype)
+				goto popnext;
 		flotr = FindLengthOfTunnel(tile, direction, tpf->tracktype);
 		si.cur_length += flotr.length;
 		tile = flotr.tile;
 	}
+
 	// remember the start tile so we know if we're in an inf loop.
 	tile_org = tile;
 
