@@ -63,7 +63,7 @@ static bool CheckAllowRemoveRoad(uint tile, uint br)
 		return true;
 
 	// Only do the special processing for actual players.
-	if (_current_player >= 8)
+	if (_current_player >= MAX_PLAYERS)
 		return true;
 
 	// A railway crossing has the road owner in the map3_lo byte.
@@ -72,7 +72,6 @@ static bool CheckAllowRemoveRoad(uint tile, uint br)
 	} else {
 		owner = _map_owner[tile];
 	}
-	
 	// Only do the special processing if the road is owned
 	// by a town
 	if (owner != OWNER_TOWN) {
@@ -81,6 +80,8 @@ static bool CheckAllowRemoveRoad(uint tile, uint br)
 
 	if (_patches.extra_dynamite)
 		return true;
+	if (_cheats.magic_bulldozer.value)
+		return true;
 
 	// Get a bitmask of which neighbouring roads has a tile
 	n = 0;
@@ -113,23 +114,24 @@ uint GetRoadBitsByTile(TileIndex tile)
 	return (byte)(r | (r >> 8));
 }
 
-
 /* Delete a piece of road
  * p1 = piece type
  */
-
 int32 CmdRemoveRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	TileInfo ti;
 	int32 cost;
 	uint tile;
-
+	City *c;
+	
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	FindLandscapeHeight(&ti, x, y);
 	tile = ti.tile;
-
-	if (!EnsureNoVehicle(tile))
+	c = ClosestCityFromTile(tile, (uint)-1); // needed for city rating penalty
+	
+	// allow deleting road under bridge
+	if (ti.type != MP_TUNNELBRIDGE && !EnsureNoVehicle(tile))
 		return CMD_ERROR;
 
 	{
@@ -142,6 +144,9 @@ int32 CmdRemoveRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	}
 
 	if (ti.type == MP_TUNNELBRIDGE) {
+		if (!EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)))
+			return CMD_ERROR;
+
 		if ((ti.map5 & 0xE9) == 0xE8) {
 			if (p1 & 10) goto return_error;
 		} else if ((ti.map5 & 0xE9) == 0xE9) {
@@ -153,7 +158,7 @@ int32 CmdRemoveRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 		if (flags & DC_EXEC) {
 			_map5[tile] = ti.map5 & 0xC7;
-			_map_owner[tile] = 0x10;
+			_map_owner[tile] = OWNER_NONE;
 			MarkTileDirtyByTile(tile);
 		}
 		return cost;
@@ -175,13 +180,28 @@ int32 CmdRemoveRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			do {
 				if (t2&1) cost += _price.remove_road;			
 			} while(t2>>=1);
-
+			
+			// check if you're allowed to remove the street owned by a town
+			// no removal if rating is lower than "Poor" (15) <- this should depend on difficulty setting
+			if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) {
+				if (c->ratings[_current_player] < 15 && !(flags & DC_NO_CITY_RATING) && !_cheats.magic_bulldozer.value) {
+					SET_DPARAM16(0, c->index);
+					return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
+				}
+			}
+			
 			if (flags & DC_EXEC) {
+				// checks if the owner is town than decrease city rating by 50 until you have 
+				// a "Poor" city rating
+				if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR)
+					ChangeCityRating(c, -50, -100); 
+
 				_map5[tile] ^= t;
 				if ((_map5[tile]&0xF) == 0)
 					DoClearSquare(tile);
 				else
 					MarkTileDirtyByTile(tile);
+
 			}
 			return cost;			
 		} else if (!(ti.map5 & 0xE0)) {
@@ -300,7 +320,8 @@ int32 CmdBuildRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	FindLandscapeHeight(&ti, x, y);
 	tile = ti.tile;
 
-	if (!EnsureNoVehicle(tile))
+	// allow building road under bridge
+	if (ti.type != MP_TUNNELBRIDGE && !EnsureNoVehicle(tile))
 		return CMD_ERROR;
 
 	if (ti.type == MP_STREET) {
@@ -339,6 +360,9 @@ int32 CmdBuildRoad(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		return _price.build_road * 2;
 	} else if (ti.type == MP_TUNNELBRIDGE) {
 		byte t;
+
+		if (!EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)))
+			return CMD_ERROR;
 		
 		t = ti.map5 & 0xF9;
 
@@ -748,7 +772,7 @@ static void DrawTile_Road(TileInfo *ti)
 
 		ormod = 0x315;
 		player = _map_owner[ti->tile];
-		if (player < 8)
+		if (player < MAX_PLAYERS)
 			ormod = PLAYER_SPRITE_COLOR(player);
 
 		s = _road_display_datas[ti->map5 & 0xF];
@@ -795,8 +819,6 @@ void DrawRoadDepotSprite(int x, int y, int image)
 	}
 }
 
-extern const byte _inclined_tileh[];
-
 uint GetSlopeZ_Road(TileInfo *ti)
 {
 	uint z = ti->z;
@@ -1032,7 +1054,7 @@ static void ChangeTileOwner_Road(uint tile, byte old_player, byte new_player)
 
 	// road/rail crossing where the road is owned by the current player?
 	if (old_player == _map3_lo[tile] && (_map5[tile]&0xF0) == 0x10) {
-		_map3_lo[tile] = new_player == 0xFF ? 0x10 : new_player;
+		_map3_lo[tile] = (new_player == 0xFF) ? OWNER_NONE : new_player;
 	}
 
 	if (_map_owner[tile] != old_player)
