@@ -8,8 +8,8 @@
 #include <setjmp.h>
 
 enum {
-	SAVEGAME_MAJOR_VERSION = 2,
-	SAVEGAME_MINOR_VERSION = 1,
+	SAVEGAME_MAJOR_VERSION = 4,
+	SAVEGAME_MINOR_VERSION = 0,
 
 	SAVEGAME_LOADABLE_VERSION = (SAVEGAME_MAJOR_VERSION << 8) + SAVEGAME_MINOR_VERSION
 };
@@ -299,7 +299,7 @@ uint SlGetFieldLength()
 
 static void SlSaveLoadConv(void *ptr, uint conv)
 {
-	int64 x;
+	int64 x = 0;
 	
 	if (_sl.save) {
 		// Read a value from the struct. These ARE endian safe.
@@ -554,7 +554,8 @@ void SlGlobList(const SaveLoadGlobVarList *desc)
 		void *ptr = desc->address;
 		if (ptr == NULL)
 			break;
-		SlSaveLoadConv(ptr, desc->conv);
+		if(_sl.version >= desc->from_version && _sl.version <= desc->to_version)
+			SlSaveLoadConv(ptr, desc->conv);
 		desc++;
 	}
 }
@@ -1017,7 +1018,7 @@ int SaveOrLoad(const char *filename, int mode)
 {
 	uint32 hdr[2];
 	const SaveLoadFormat *fmt;
-	uint version;
+  uint version;
 		
 	// old style load
 	if (mode == 2) {
@@ -1058,6 +1059,10 @@ int SaveOrLoad(const char *filename, int mode)
 		}
 	}
 
+  // we first initialize here to avoid: "warning: variable `version' might 
+  // be clobbered by `longjmp' or `vfork'"
+	version = 0;
+
 	if (mode != 0) {
 		fmt = GetSavegameFormat(_savegame_format);
 		
