@@ -432,7 +432,7 @@ static bool load_intlist(const char *str, void *array, int nelems, int type)
 
 static void make_intlist(char *buf, void *array, int nelems, int type)
 {
-	int i, v;
+	int i, v = 0;
 	byte *p = (byte*)array;
 	for(i=0; i!=nelems; i++) {
 		switch(type) {
@@ -608,7 +608,7 @@ static void save_setting_desc(IniFile *ini, const SettingDesc *desc, void *grpna
 	IniGroup *group_def = NULL, *group;
 	IniItem *item;
 	void *p, *ptr;
-	int i;
+	int i = 0;
 	char buf[512]; // setting buffer
 	const char *s;
 
@@ -760,7 +760,7 @@ static const SettingDesc misc_settings[] = {
 };
 
 static const SettingDesc network_settings[] = {
-	{"port", SDT_UINT | SDT_NOSAVE, (void*)12345, &_network_port},
+	{"port", SDT_UINT | SDT_NOSAVE, (void*)3979, &_network_port},
 	{"sync_freq", SDT_UINT | SDT_NOSAVE, (void*)4, &_network_sync_freq},
 	{"ahead_frames", SDT_UINT | SDT_NOSAVE, (void*)5, &_network_ahead_frames},
 	{NULL}
@@ -802,7 +802,6 @@ static const SettingDesc patch_settings[] = {
 	{"longbridges", SDT_BOOL, (void*)false, (void*)offsetof(Patches, longbridges)},
 	{"gotodepot", SDT_BOOL, (void*)true, (void*)offsetof(Patches, gotodepot)},
 
-	{"crossing_tunnels", SDT_BOOL, (void*)false, (void*)offsetof(Patches, crossing_tunnels)},
 	{"build_rawmaterial_ind", SDT_BOOL, (void*)false, (void*)offsetof(Patches, build_rawmaterial_ind)},
 	{"multiple_industry_per_town", SDT_BOOL, (void*)false, (void*)offsetof(Patches, multiple_industry_per_town)},
 	{"same_industry_close", SDT_BOOL, (void*)false, (void*)offsetof(Patches, same_industry_close)},
@@ -863,6 +862,13 @@ static const SettingDesc patch_settings[] = {
 
 	{"auto_euro",  SDT_BOOL, (void*)true, (void*)offsetof(Patches, auto_euro)},
 
+	{"serviceathelipad",  SDT_BOOL, (void*)true, (void*)offsetof(Patches, serviceathelipad)},
+	{"smooth_economy", SDT_BOOL, (void*)false, (void*)offsetof(Patches, smooth_economy)},
+	{"dist_local_authority", SDT_UINT8, (void*)20, (void*)offsetof(Patches, dist_local_authority)},
+
+	{"wait_oneway_signal", SDT_UINT8, (void*)15, (void*)offsetof(Patches, wait_oneway_signal)},
+	{"wait_twoway_signal", SDT_UINT8, (void*)41, (void*)offsetof(Patches, wait_twoway_signal)},
+
 	{NULL}
 };
 
@@ -880,10 +886,30 @@ static void HandleSettingDescs(IniFile *ini, SettingDescProc *proc)
 	proc(ini, debug_settings, "debug", NULL);
 }
 
+void LoadGrfSettings(IniFile *ini)
+{
+	IniGroup *group = ini_getgroup(ini, "newgrf", -1);
+	IniItem *item;
+	int i;
+
+	if (!group)
+		return;
+
+	for(i=0; i!=lengthof(_newgrf_files); i++) {
+		char buf[3];
+		sprintf(buf, "%d", i);
+		item = ini_getitem(group, buf, false);
+		if (!item)
+			break;
+		_newgrf_files[i] = strdup(item->value);
+	}
+}
+
 void LoadFromConfig()
 {
 	IniFile *ini = ini_load(_config_file);
 	HandleSettingDescs(ini, load_setting_desc);
+	LoadGrfSettings(ini);
 	ini_free(ini);
 }
 
