@@ -453,7 +453,7 @@ static const GameSettingData _game_setting_info[] = {
 	{0,3,1,STR_6826_X1_5},
 	{0,2,1,STR_6820_LOW},
 	{0,3,1,STR_682A_VERY_FLAT},
-	{0,2,1,STR_6820_LOW},
+	{0,3,1,STR_VERY_LOW},
 	{0,1,1,STR_682E_STEADY},
 	{0,1,1,STR_6834_AT_END_OF_LINE_AND_AT_STATIONS},
 	{0,1,1,STR_6836_OFF},
@@ -638,8 +638,8 @@ static const PatchEntry _patches_ui[] = {
 
 static const PatchEntry _patches_construction[] = {
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_BUILDONSLOPES, &_patches.build_on_slopes},
+	{PE_BOOL, 0, STR_CONFIG_PATCHES_EXTRADYNAMITE, &_patches.extra_dynamite},
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_LONGBRIDGES, &_patches.longbridges},
-	{PE_BOOL, 0, STR_CONFIG_PATCHES_CROSSINGTUNNELS, &_patches.crossing_tunnels},
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_SIGNALSIDE, &_patches.signal_side},
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_BUILD_IN_PAUSE, &_patches.build_in_pause},
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_SMALL_AIRPORTS, &_patches.always_small_airport},
@@ -658,6 +658,7 @@ static const PatchEntry _patches_vehicles[] = {
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_WARN_INCOME_LESS, &_patches.train_income_warn},
 
 	{PE_UINT16, PF_0ISDIS, STR_CONFIG_PATCHES_LOST_TRAIN_DAYS, &_patches.lost_train_days, 180, 720, 60},
+	{PE_BOOL, 0, STR_CONFIG_AUTORENEW_VEHICLE, &_patches.autorenew},
 
 	{PE_UINT8, 0, STR_CONFIG_PATCHES_MAX_TRAINS, &_patches.max_trains, 0, 240, 10},
 	{PE_UINT8, 0, STR_CONFIG_PATCHES_MAX_ROADVEH, &_patches.max_roadveh, 0, 240, 10},
@@ -682,6 +683,7 @@ static const PatchEntry _patches_stations[] = {
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_NEW_NONSTOP, &_patches.new_nonstop},
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_NONUNIFORM_STATIONS, &_patches.nonuniform_stations},
 	{PE_UINT8, 0, STR_CONFIG_PATCHES_STATION_SPREAD, &_patches.station_spread, 4, 64, 1},
+	{PE_BOOL, 0, STR_CONFIG_PATCHES_SERVICEATHELIPAD, &_patches.serviceathelipad},
 };
 
 static const PatchEntry _patches_economy[] = {
@@ -694,6 +696,8 @@ static const PatchEntry _patches_economy[] = {
 	
 	{PE_INT32, PF_NOCOMMA, STR_CONFIG_PATCHES_COLORED_NEWS_DATE, &_patches.colored_news_date, 1900, 2200, 5},
 	{PE_INT32, PF_NOCOMMA, STR_CONFIG_PATCHES_STARTING_DATE, &_patches.starting_date, 1920, 2100, 1},
+
+	{PE_BOOL, 0, STR_CONFIG_PATCHES_SMOOTH_ECONOMY, &_patches.smooth_economy},
 };
 
 typedef struct PatchPage {
@@ -712,15 +716,18 @@ static const PatchPage _patches_page[] = {
 static int32 ReadPE(const PatchEntry*pe)
 {
 	switch(pe->type) {
-	case PE_BOOL: return *(bool*)pe->variable;
-	case PE_UINT8: return *(uint8*)pe->variable;
-	case PE_INT16: return *(int16*)pe->variable;
+	case PE_BOOL:   return *(bool*)pe->variable;
+	case PE_UINT8:  return *(uint8*)pe->variable;
+	case PE_INT16:  return *(int16*)pe->variable;
 	case PE_UINT16: return *(uint16*)pe->variable;
-	case PE_INT32: return *(int32*)pe->variable;
-	case PE_BYTE: return *(byte*)pe->variable;
+	case PE_INT32:  return *(int32*)pe->variable;
+	case PE_BYTE:   return *(byte*)pe->variable;
 	default:
 		NOT_REACHED();
 	}
+  
+	/* useless, but avoids compiler warning this way */
+	return 0;
 }
 
 static void WritePE(const PatchEntry *pe, int32 val)
@@ -897,7 +904,7 @@ static const Widget _patches_selection_widgets[] = {
 {   WWT_CLOSEBOX,    10,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,    10,    11,   369,     0,    13, STR_CONFIG_PATCHES_CAPTION, STR_018C_WINDOW_TITLE_DRAG_THIS},
 {      WWT_PANEL,    10,     0,   369,    14,    41, 0x0, 0},
-{      WWT_PANEL,    10,     0,   369,    42,   280, 0x0, 0},
+{      WWT_PANEL,    10,     0,   369,    42,   290, 0x0, 0},
 
 {   WWT_CLOSEBOX,     3,    10,    96,    16,    27, STR_CONFIG_PATCHES_GUI, 0},
 {   WWT_CLOSEBOX,     3,    97,   183,    16,    27, STR_CONFIG_PATCHES_CONSTRUCTION, 0},
@@ -908,7 +915,7 @@ static const Widget _patches_selection_widgets[] = {
 };
 
 static const WindowDesc _patches_selection_desc = {
-	WDP_CENTER, WDP_CENTER, 370, 281, //251,
+	WDP_CENTER, WDP_CENTER, 370, 291,
 	WC_GAME_OPTIONS,0,
 	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
 	_patches_selection_widgets,
