@@ -13,21 +13,14 @@
 #include "engine.h"
 
 
-extern const byte _ship_cost_table[];
-extern const byte _ship_speed_table[];
-extern const byte _ship_cargo_types[];
-extern const uint16 _ship_cargo_amount[];
-extern const byte _ship_running_cost[];
-extern const byte _ship_refittable[];
-
 static void DrawShipImage(Vehicle *v, int x, int y, VehicleID selection);
 
 
 const byte _ship_refit_types[4][16] = {
-	{2, 1, 4, 5, 6, 7, 8, 9, 10, 255},
-	{2, 1, 4, 5, 6, 7, 9, 11, 10, 255},
-	{2, 4, 5, 8, 6, 7, 9, 10, 255},
-	{2, 1, 3, 5, 7, 8, 9, 6, 4, 10, 11, 255},
+	{CT_MAIL, CT_COAL, CT_LIVESTOCK, CT_GOODS, CT_GRAIN, CT_WOOD, CT_IRON_ORE, CT_STEEL, CT_VALUABLES, 255},
+	{CT_MAIL, CT_COAL, CT_LIVESTOCK, CT_GOODS, CT_GRAIN, CT_WOOD, CT_PAPER, CT_FOOD, CT_VALUABLES, 255},
+	{CT_MAIL, CT_FRUIT, CT_GOODS, CT_COPPER_ORE, CT_GRAIN, CT_WOOD, CT_WATER, CT_VALUABLES, 255},
+	{CT_MAIL, CT_SUGAR, CT_TOYS, CT_CANDY, CT_COLA, CT_COTTON_CANDY, CT_BUBBLES, CT_TOFFEE, CT_BATTERIES, CT_PLASTIC, CT_FIZZY_DRINKS, 255},
 };
 
 static void ShipRefitWndProc(Window *w, WindowEvent *e)
@@ -51,17 +44,39 @@ static void ShipRefitWndProc(Window *w, WindowEvent *e)
 		x = 6;
 		y = 25;
 		sel = WP(w,refit_d).sel;
-		b = _ship_refit_types[_opt.landscape];
-		do {
-			color = 16;
-			if (sel == 0) {
-				cargo = *b;
-				color = 12;
+
+#define show_cargo(ctype) { \
+		color = 16; \
+		if (sel == 0) { \
+			cargo = ctype; \
+			color = 12; \
+		} \
+		sel--; \
+		DrawString(x, y, _cargoc.names_s[ctype], color); \
+		y += 10; \
+		}
+
+		if (_engine_refit_masks[v->engine_type]) {
+			uint32 mask = _engine_refit_masks[v->engine_type];
+			int cid = 0;
+
+			for (; mask; mask >>= 1, cid++) {
+				if (!(mask & 1)) // not this cid
+					continue;
+				if (!(_local_cargo_id_landscape[cid] & (1 << _opt.landscape))) // not in this landscape
+					continue;
+
+				show_cargo(_local_cargo_id_ctype[cid]);
 			}
-			sel--;
-			DrawString(x,y,_cargoc.names_s[*b], color);
-			y += 10;
-		} while (*++b != 255);
+
+		} else { // generic refit list
+			b = _ship_refit_types[_opt.landscape];
+			do {
+				show_cargo(*b);
+			} while (*++b != 255);
+		}
+
+#undef show_cargo
 
 		WP(w,refit_d).cargo = cargo;
 
@@ -158,7 +173,7 @@ static void ShipDetailsWndProc(Window *w, WindowEvent *e)
 			}
 			SET_DPARAM16(0, str);
 			SET_DPARAM16(2, v->max_age / 366);
-			SET_DPARAM32(3, _ship_running_cost[v->engine_type - SHIP_ENGINES_INDEX] * _price.ship_running >> 8);
+			SET_DPARAM32(3, ship_vehicle_info(v->engine_type).running_cost * _price.ship_running >> 8);
 			DrawString(2, 15, STR_9812_AGE_RUNNING_COST_YR, 0);
 		}
 
@@ -192,7 +207,7 @@ static void ShipDetailsWndProc(Window *w, WindowEvent *e)
 		DrawShipImage(v, 3, 57, INVALID_VEHICLE);
 
 		SET_DPARAM16(1, 1920 + v->build_year);
-		SET_DPARAM16(0, _engine_name_strings[v->engine_type]);
+		SET_DPARAM16(0, GetCustomEngineName(v->engine_type));
 		SET_DPARAM32(2, v->value);
 		DrawString(74, 57, STR_9816_BUILT_VALUE, 0);
 		
@@ -328,7 +343,7 @@ static void NewShipWndProc(Window *w, WindowEvent *e)
 				if (HASBIT(e->player_avail, _local_player)) {
 					if (sel==0) selected_id = engine_id;
 					if (IS_INT_INSIDE(--pos, -4, 0)) {
-						DrawString(x+75, y+7, _engine_name_strings[engine_id], sel==0 ? 0xC : 0x10);
+						DrawString(x+75, y+7, GetCustomEngineName(engine_id), sel==0 ? 0xC : 0x10);
 						DrawShipEngine(x+35, y+10, engine_id, SPRITE_PALETTE(PLAYER_SPRITE_COLOR(_local_player)));
 						y += 24;
 					}
@@ -341,12 +356,12 @@ static void NewShipWndProc(Window *w, WindowEvent *e)
 			if (selected_id != -1) {
 				Engine *e;
 
-				SET_DPARAM32(0, _ship_cost_table[selected_id - SHIP_ENGINES_INDEX] * (_price.ship_base>>3)>>5);
-				SET_DPARAM16(1, _ship_speed_table[selected_id - SHIP_ENGINES_INDEX] * 10 >> 5);
-				SET_DPARAM16(2, _cargoc.names_long_p[_ship_cargo_types[selected_id - SHIP_ENGINES_INDEX]]);
-				SET_DPARAM16(3, _ship_cargo_amount[selected_id - SHIP_ENGINES_INDEX]);
-				SET_DPARAM16(4, _ship_refittable[selected_id - SHIP_ENGINES_INDEX] ? STR_9842_REFITTABLE : STR_EMPTY);
-				SET_DPARAM32(5,_ship_running_cost[selected_id - SHIP_ENGINES_INDEX] * _price.ship_running >> 8);
+				SET_DPARAM32(0, ship_vehicle_info(selected_id).base_cost * (_price.ship_base>>3)>>5);
+				SET_DPARAM16(1, ship_vehicle_info(selected_id).max_speed * 10 >> 5);
+				SET_DPARAM16(2, _cargoc.names_long_p[ship_vehicle_info(selected_id).cargo_type]);
+				SET_DPARAM16(3, ship_vehicle_info(selected_id).capacity);
+				SET_DPARAM16(4, ship_vehicle_info(selected_id).refittable ? STR_9842_REFITTABLE : STR_EMPTY);
+				SET_DPARAM32(5, ship_vehicle_info(selected_id).running_cost * _price.ship_running >> 8);
 
 				e = &_engines[selected_id];	
 				SET_DPARAM16(7, e->lifelength);
@@ -377,7 +392,7 @@ static void NewShipWndProc(Window *w, WindowEvent *e)
 		case 6:	/* rename */
 			WP(w,buildtrain_d).rename_engine = WP(w,buildtrain_d).sel_engine;
 			ShowQueryString(
-				_engine_name_strings[WP(w,buildtrain_d).sel_engine],
+				GetCustomEngineName(WP(w,buildtrain_d).sel_engine),
 				STR_9838_RENAME_SHIP_TYPE,
 				31,
 				160,
@@ -452,7 +467,7 @@ static void ShipViewWndProc(Window *w, WindowEvent *e) {
 		StringID str;
 
 		// Possible to refit?
-		if (_ship_refittable[v->engine_type - SHIP_ENGINES_INDEX] &&
+		if (ship_vehicle_info(v->engine_type).refittable &&
 				v->vehstatus&VS_STOPPED &&
 				v->u.ship.state == 0x80 &&
 				IsShipDepotTile(v->tile))
