@@ -8,6 +8,24 @@
 #include "vehicle.h"
 #include "city.h"
 
+static const Widget _smallmap_megabig_widgets[] = {
+{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    13,    11,   592,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,    13,   593,   606,     0,    13, 0x2AA,STR_01CC_TOGGLE_LARGE_SMALL_MAP},
+{     WWT_IMGBTN,    13,     0,   606,    14,   407, 0x0, 0},
+{          WWT_6,    13,     2,   604,    16,   405, 0x0, 0},
+{     WWT_IMGBTN,    13,   541,   562,   408,   429, 0x2E2,STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
+{     WWT_IMGBTN,    13,   563,   584,   408,   429, 0x2E3,STR_0192_SHOW_VEHICLES_ON_MAP},
+{     WWT_IMGBTN,    13,   585,   606,   408,   429, 0x2E5,STR_0193_SHOW_INDUSTRIES_ON_MAP},
+{     WWT_IMGBTN,    13,   519,   540,   430,   451, 0x2E4,STR_0194_SHOW_TRANSPORT_ROUTES_ON},
+{     WWT_IMGBTN,    13,   541,   562,   430,   451, 0x2E6,STR_0195_SHOW_VEGETATION_ON_MAP},
+{     WWT_IMGBTN,    13,   563,   584,   430,   451, 0x2E7,STR_0196_SHOW_LAND_OWNERS_ON_MAP},
+{     WWT_IMGBTN,    13,   585,   606,   430,   451, 0xFED,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    13,     0,   518,   408,   451, 0x0,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    13,   519,   540,   408,   429, 0x0,0},
+{      WWT_LAST},
+};
+
 static const Widget _smallmap_big_widgets[] = {
 {    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,    13,    11,   431,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
@@ -21,7 +39,7 @@ static const Widget _smallmap_big_widgets[] = {
 {     WWT_IMGBTN,    13,   402,   423,   280,   301, 0x2E6,STR_0195_SHOW_VEGETATION_ON_MAP},
 {     WWT_IMGBTN,    13,   424,   445,   280,   301, 0x2E7,STR_0196_SHOW_LAND_OWNERS_ON_MAP},
 {     WWT_IMGBTN,    13,   358,   379,   280,   301, 0xFED,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,     0,   357,   258,   301, 0x0,0},
+{     WWT_IMGBTN,    13,     0,   357,   258,   301, 0x0,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
 {     WWT_IMGBTN,    13,   358,   379,   258,   279, 0x0,0},
 {      WWT_LAST},
 };
@@ -39,16 +57,16 @@ static const Widget _smallmap_small_widgets[] = {
 {     WWT_IMGBTN,    13,   226,   247,    80,   101, 0x2E4,STR_0194_SHOW_TRANSPORT_ROUTES_ON},
 {     WWT_IMGBTN,    13,   226,   247,   102,   123, 0x2E6,STR_0195_SHOW_VEGETATION_ON_MAP},
 {     WWT_IMGBTN,    13,   226,   247,   124,   145, 0x2E7,STR_0196_SHOW_LAND_OWNERS_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,   146,   167, 0xFED,0},
+{     WWT_IMGBTN,    13,   226,   247,   146,   167, 0xFED,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
 {     WWT_IMGBTN,    13,     0,   247,   168,   211, 0x0,0},
 {      WWT_LAST},
 };
 
 static int _smallmap_type;
-static bool _smallmap_size;
+static byte _smallmap_size;
 static bool _smallmap_show_towns = true;
 
-static void DoShowSmallMap(bool big);
+static void DoShowSmallMap(int big);
 
 #define MK(a,b) a,b
 #define MKEND() 0xffff
@@ -309,57 +327,60 @@ static const uint32 _smallmap_vegetation_andor[12][2] = {
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
 };
 
-void DrawSmallMapContours(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+static uint32 INLINE GetSmallMapCountoursPixels(uint tile)
 {
-	uint tile;
-	int t;
+	uint t;
 
-	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX) {
-			tile = TILE_XY(xc,yc);
-
-			t = GET_TILETYPE(tile);
-			if (t == MP_TUNNELBRIDGE) {
-				t = _map5[tile];
-				if ((t & 0x80) == 0) t>>=1;
-				if ((t & 6) == 0) {
-					t = MP_RAILWAY;
-				} else if ((t & 6) == 2) {
-					t = MP_STREET;
-				} else {
-					t = MP_WATER;
-				}
-			}
-
-			WRITE_PIXELS_OR(dst, ((_map_height_bits[_map_type_and_height[tile] & 0xF] & 
-								_smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0]) & mask);
+	t = GET_TILETYPE(tile);
+	if (t == MP_TUNNELBRIDGE) {
+		t = _map5[tile];
+		if ((t & 0x80) == 0) t>>=1;
+		if ((t & 6) == 0) {
+			t = MP_RAILWAY;
+		} else if ((t & 6) == 2) {
+			t = MP_STREET;
+		} else {
+			t = MP_WATER;
 		}
+	}
+
+	return (_map_height_bits[_map_type_and_height[tile] & 0xF] & _smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0];
+}
+
+static void DrawSmallMapContours(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+{
+	do {
+		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+			WRITE_PIXELS_OR( dst, GetSmallMapCountoursPixels(TILE_XY(xc,yc)) & mask );
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
-void DrawSmallMapVehicles(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
-{
-	uint tile;
-	int t;
 
-	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX) {
-			tile = TILE_XY(xc,yc);
-			
-			t = GET_TILETYPE(tile);
-			if (t == MP_TUNNELBRIDGE) {
-				t = _map5[tile];
-				if ((t & 0x80) == 0) t>>=1;
-				if ((t & 6) == 0) {
-					t = MP_RAILWAY;
-				} else if ((t & 6) == 2) {
-					t = MP_STREET;
-				} else {
-					t = MP_WATER;
-				}
-			}
-			WRITE_PIXELS_OR(dst, ((MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0]) & mask);
+static uint32 INLINE GetSmallMapVehiclesPixels(uint tile)
+{
+	uint t;
+
+	t = GET_TILETYPE(tile);
+	if (t == MP_TUNNELBRIDGE) {
+		t = _map5[tile];
+		if ((t & 0x80) == 0) t>>=1;
+		if ((t & 6) == 0) {
+			t = MP_RAILWAY;
+		} else if ((t & 6) == 2) {
+			t = MP_STREET;
+		} else {
+			t = MP_WATER;
 		}
+	}
+	return (MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0];
+}
+
+
+static void DrawSmallMapVehicles(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+{
+	do {
+		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+			WRITE_PIXELS_OR( dst, GetSmallMapVehiclesPixels(TILE_XY(xc,yc)) & mask );
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
@@ -388,76 +409,76 @@ static const byte _industry_smallmap_colors[175] = {
 	 15, 15, 15, 15, 15, 15, 15,
 };
 
-void DrawSmallMapIndustries(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+static uint32 INLINE GetSmallMapIndustriesPixels(uint tile)
 {
-	uint tile;
 	int t;
-	uint32 bits;
 
-	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX) {
-			tile = TILE_XY(xc,yc);
-			
-			t = GET_TILETYPE(tile);
-			if (t == MP_INDUSTRY) {
-				byte color = _industry_smallmap_colors[_map5[tile]];
-				bits = color + (color << 8) + (color << 16) + (color << 24);
+	t = GET_TILETYPE(tile);
+	if (t == MP_INDUSTRY) {
+		byte color = _industry_smallmap_colors[_map5[tile]];
+		return color + (color << 8) + (color << 16) + (color << 24);
+	} else {
+		if (t == MP_TUNNELBRIDGE) {
+			t = _map5[tile];
+			if ((t & 0x80) == 0) t>>=1;
+			if ((t & 6) == 0) {
+				t = MP_RAILWAY;
+			} else if ((t & 6) == 2) {
+				t = MP_STREET;
 			} else {
-				if (t == MP_TUNNELBRIDGE) {
-					t = _map5[tile];
-					if ((t & 0x80) == 0) t>>=1;
-					if ((t & 6) == 0) {
-						t = MP_RAILWAY;
-					} else if ((t & 6) == 2) {
-						t = MP_STREET;
-					} else {
-						t = MP_WATER;
-					}
-				}
-				bits = ((MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0]);
+				t = MP_WATER;
 			}
-
-			WRITE_PIXELS_OR(dst, bits & mask);
 		}
+		return ((MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0]);
+	}
+}
+
+static void DrawSmallMapIndustries(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+{
+	do {
+		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+			WRITE_PIXELS_OR(dst, GetSmallMapIndustriesPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
-void DrawSmallMapRoutes(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+static uint32 INLINE GetSmallMapRoutesPixels(uint tile)
 {
-	uint tile;
 	int t;
 	uint32 bits;
 
-	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX) {
-			tile = TILE_XY(xc,yc);
-			
-			t = GET_TILETYPE(tile);
-			if (t == MP_STATION) {
-				byte m5 = _map5[tile];
-				(bits = MKCOLOR(0x56565656), m5 < 8) ||
-				(bits = MKCOLOR(0xB8B8B8B8), m5 < 0x43) ||
-				(bits = MKCOLOR(0xC2C2C2C2), m5 < 0x47) ||
-				(bits = MKCOLOR(0xBFBFBFBF), m5 < 0x4B) ||
-				(bits = MKCOLOR(0x98989898), m5 < 0x53) ||
-				(bits = MKCOLOR(0xFFFFFFFF), true);
+	t = GET_TILETYPE(tile);
+	if (t == MP_STATION) {
+		byte m5 = _map5[tile];
+		(bits = MKCOLOR(0x56565656), m5 < 8) ||			//   8 - railroad station (green)
+		(bits = MKCOLOR(0xB8B8B8B8), m5 < 0x43) ||	//  67 - airport (red)
+		(bits = MKCOLOR(0xC2C2C2C2), m5 < 0x47) ||	//  71 - truck loading bay (orange)
+		(bits = MKCOLOR(0xBFBFBFBF), m5 < 0x4B) ||	//  75 - bus station (yellow)
+		(bits = MKCOLOR(0x98989898), m5 < 0x53) ||	//  83 - docks (blue)
+		(bits = MKCOLOR(0xB8B8B8B8), m5 < 0x73) ||	// 115 - airport (red) (new airports)
+		(bits = MKCOLOR(0xFFFFFFFF), true);					// all others
+	} else {
+		if (t == MP_TUNNELBRIDGE) {
+			t = _map5[tile];
+			if ((t & 0x80) == 0) t>>=1;
+			if ((t & 6) == 0) {
+				t = MP_RAILWAY;
+			} else if ((t & 6) == 2) {
+				t = MP_STREET;
 			} else {
-				if (t == MP_TUNNELBRIDGE) {
-					t = _map5[tile];
-					if ((t & 0x80) == 0) t>>=1;
-					if ((t & 6) == 0) {
-						t = MP_RAILWAY;
-					} else if ((t & 6) == 2) {
-						t = MP_STREET;
-					} else {
-						t = MP_WATER;
-					}
-				}
-				bits = ((MKCOLOR(0x54545454) & _smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0]);
+				t = MP_WATER;
 			}
-
-			WRITE_PIXELS_OR(dst, bits & mask);
 		}
+		// ground color
+		bits = ((MKCOLOR(0x54545454) & _smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0]);
+	}
+	return bits;
+}
+
+static void DrawSmallMapRoutes(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+{
+	do {
+		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+			WRITE_PIXELS_OR(dst, GetSmallMapRoutesPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
@@ -476,75 +497,76 @@ static const uint32 _vegetation_clear_bits[4 + 7] = {
 	MKCOLOR(0x54545454),
 };
 
-void DrawSmallMapVegetation(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+static uint32 INLINE GetSmallMapVegetationPixels(uint tile)
 {
-	uint tile;
-	int t;
+	int i,t;
 	uint32 bits;
-	int i;
 
-	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX) {
-			tile = TILE_XY(xc,yc);
-			
-			t = GET_TILETYPE(tile);
-			if (t == MP_CLEAR) {
-				i = (_map5[tile] & 0x1F) - 4;
-				if (i >= 0) i = (i >> 2);
-				bits = _vegetation_clear_bits[i + 4];
-			} else if (t == MP_INDUSTRY) {
-				bits = IS_BYTE_INSIDE(_map5[tile], 0x10, 0x12) ? MKCOLOR(0xD0D0D0D0) : MKCOLOR(0xB5B5B5B5); 
-			} else if (t == MP_TREES) {
-				bits = MKCOLOR(0x54575754);
-				if ((_map2[tile] & 0x30) == 0x20)
-					bits = (_opt.landscape == LT_HILLY) ? MKCOLOR(0x98575798) : MKCOLOR(0xC25757C2);
+	t = GET_TILETYPE(tile);
+	if (t == MP_CLEAR) {
+		i = (_map5[tile] & 0x1F) - 4;
+		if (i >= 0) i = (i >> 2);
+		bits = _vegetation_clear_bits[i + 4];
+	} else if (t == MP_INDUSTRY) {
+		bits = IS_BYTE_INSIDE(_map5[tile], 0x10, 0x12) ? MKCOLOR(0xD0D0D0D0) : MKCOLOR(0xB5B5B5B5); 
+	} else if (t == MP_TREES) {
+		bits = MKCOLOR(0x54575754);
+		if ((_map2[tile] & 0x30) == 0x20)
+			bits = (_opt.landscape == LT_HILLY) ? MKCOLOR(0x98575798) : MKCOLOR(0xC25757C2);
+	} else {
+		if (t == MP_TUNNELBRIDGE) {
+			t = _map5[tile];
+			if ((t & 0x80) == 0) t>>=1;
+			if ((t & 6) == 0) {
+				t = MP_RAILWAY;
+			} else if ((t & 6) == 2) {
+				t = MP_STREET;
 			} else {
-				if (t == MP_TUNNELBRIDGE) {
-					t = _map5[tile];
-					if ((t & 0x80) == 0) t>>=1;
-					if ((t & 6) == 0) {
-						t = MP_RAILWAY;
-					} else if ((t & 6) == 2) {
-						t = MP_STREET;
-					} else {
-						t = MP_WATER;
-					}
-				}
-				bits = ((MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0]);
+				t = MP_WATER;
 			}
-
-			WRITE_PIXELS_OR(dst, bits & mask);
 		}
+		bits = ((MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0]);
+	}
+
+	return bits;
+}
+
+
+static void DrawSmallMapVegetation(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+{
+	do {
+		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+			WRITE_PIXELS_OR(dst, GetSmallMapVegetationPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
 
 static uint32 *_owner_colors;
 
-void DrawSmallMapOwners(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+static uint32 INLINE GetSmallMapOwnerPixels(uint tile)
 {
-	uint tile;
 	int t;
 
-	do {
-		if (xc < TILE_X_MAX && yc < TILE_Y_MAX) {
-			tile = TILE_XY(xc,yc);
-			
-			t = GET_TILETYPE(tile);
-			if (t == MP_HOUSE) {
-				t = 0x80;				
-			} else if (t == MP_INDUSTRY) {
-				t = 0xff;
-			} else {
-				t = _map_owner[tile];
-			}
+	t = GET_TILETYPE(tile);
+	if (t == MP_HOUSE || _map_owner[tile] == OWNER_TOWN) {
+		t = 0x80;				
+	} else if (t == MP_INDUSTRY) {
+		t = 0xff;
+	} else {
+		t = _map_owner[tile];
+	}
 
-			WRITE_PIXELS_OR(dst, _owner_colors[t] & mask);
-		}
-	} while (xc++,yc++,dst+=pitch,--reps != 0);
+	return _owner_colors[t];
 }
 
 
+static void DrawSmallMapOwners(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
+{
+	do {
+		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
+			WRITE_PIXELS_OR(dst, GetSmallMapOwnerPixels(TILE_XY(xc,yc)) & mask);
+	} while (xc++,yc++,dst+=pitch,--reps != 0);
+}
 
 
 static const uint32 _smallmap_mask_left[3] = {
@@ -560,7 +582,6 @@ static const uint32 _smallmap_mask_right[4] = {
 	MKCOLOR(0x00FFFFFF),
 };
 
-
 /* each tile has 4 x pixels and 1 y pixel */
 
 typedef void SmallmapDrawProc(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask);
@@ -630,7 +651,7 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 
 		/* now fill with the player colors */
 		FOR_ALL_PLAYERS(p) {
-			if (p->name_1 != 0) {
+			if (p->is_active) {
 				tbl[p->index] = dup_byte32(GetSpritePtr(0x307 + p->player_color)[0xCB]);
 			}
 		}	
@@ -686,6 +707,7 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 		/* number of lines */
 		reps = ((dpi->height - y + 1) >> 1);
 		if (reps > 0) {
+//			assert(ptr >= dpi->dst_ptr);
 			proc(ptr, tile_x, tile_y, dpi->pitch*2, reps, mask);
 		}
 
@@ -845,7 +867,7 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 			}
 		}
 
-		if (!FillDrawPixelInfo(&new_dpi, NULL, 3, 17, w->width - 28 + (w->widget == _smallmap_big_widgets ? 22 : 0), w->height - 64))
+		if (!FillDrawPixelInfo(&new_dpi, NULL, 3, 17, w->width - 28 + (w->widget == _smallmap_small_widgets ? 0 : 22), w->height - 64))
 			return;
 
 		DrawSmallMap(&new_dpi, w, _smallmap_type, _smallmap_show_towns);
@@ -854,10 +876,14 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 	case WE_CLICK:
 		switch(e->click.widget) {
 		case 2: {/* big/small size */
-			const Widget *wi = w->widget;
+			// const Widget *wi = w->widget; 
 			DeleteWindow(w);
 			SndPlayFx(0x13);
-			DoShowSmallMap(_smallmap_size ^= 1);
+			{
+				int i = _smallmap_size + 1;
+				if (i == 3) i = 0;
+				DoShowSmallMap(i);
+			}
 		} break;
 
 		case 4: {/* main wnd */
@@ -930,14 +956,29 @@ static const WindowDesc _smallmap_big_desc = {
 	SmallMapWindowProc
 };
 
+static const WindowDesc _smallmap_megabig_desc = {
+	-1,-1, 607, 452,
+	WC_SMALLMAP,0,
+	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
+	_smallmap_megabig_widgets,
+	SmallMapWindowProc
+};
 
-static void DoShowSmallMap(bool big)
+static const WindowDesc * const _smallmap_descs[3] = {
+	&_smallmap_small_desc,
+	&_smallmap_big_desc,
+	&_smallmap_megabig_desc
+};
+
+static void DoShowSmallMap(int big)
 {
 	Window *w;
 	ViewPort *vp;
 	int x,y;
 
-	w = AllocateWindowDescFront(big ? &_smallmap_big_desc : &_smallmap_small_desc, 0);
+	_smallmap_size = big;
+	
+	w = AllocateWindowDescFront(_smallmap_descs[big], 0);
 	if (w) {
 		w->click_state = (1<<5) << _smallmap_type | _smallmap_show_towns << 11;
 		vp = FindWindowById(WC_MAIN_WINDOW, 0)->viewport;
