@@ -11,11 +11,15 @@
 #include "saveload.h"
 #include "economy.h"
 #include "player.h"
+#include "airport.h"
 
-const byte _airport_size_x[3] = {4, 6, 1 };
-const byte _airport_size_y[3] = {3, 6, 1 };
+// FIXME -- need to be embedded into Airport variable. Is dynamically
+// deducteable from graphics-tile array, so will not be needed
+const byte _airport_size_x[5] = {4, 6, 1, 6, 7 };
+const byte _airport_size_y[5] = {3, 6, 1, 6, 7 };
 
 void ShowAircraftDepotWindow(uint tile);
+extern void UpdateAirplanesOnNewStation(Station *st);
 
 static void MarkStationDirty(Station *st)
 {
@@ -110,7 +114,7 @@ static Station *AllocateStation()
 
 
 static int CountMapSquareAround(uint tile, byte type, byte min, byte max) {
-	static const int16 _count_square_table[7*7+1] = {
+	static const TileIndexDiff _count_square_table[7*7+1] = {
 		TILE_XY(-3,-3), 1, 1, 1, 1, 1, 1,
 		TILE_XY(-6,1),  1, 1, 1, 1, 1, 1,
 		TILE_XY(-6,1),  1, 1, 1, 1, 1, 1,
@@ -121,7 +125,7 @@ static int CountMapSquareAround(uint tile, byte type, byte min, byte max) {
 		0,
 	};	
 	int j;
-	const int16 *p = _count_square_table;
+	const TileIndexDiff *p = _count_square_table;
 	int num = 0;
 
 	while ( (j=*p++) != 0 ) {
@@ -266,56 +270,6 @@ static Station *GetClosestStationFromTile(uint tile, uint threshold, byte owner)
 	return best_station;
 }
 
-static void RemoveStationFromAlphaList(Station *st)
-{
-	byte b;
-	Station *s;
-
-	b =  st->alpha_order;
-	FOR_ALL_STATIONS(s) {
-		if (s->xy != 0 && b < s->alpha_order)
-			s->alpha_order--;
-	}
-
-	st->alpha_order = 0xFF;
-}
-
-static void UpdateStationAlphaList(Station *myst)
-{
-	Station *st;
-	int n;
-	char buf1[64];
-	char buf2[64];
-
-	RemoveStationFromAlphaList(myst);
-
-	SET_DPARAM16(0, myst->city->citynametype);
-	SET_DPARAM32(1, myst->city->citynameparts);
-	GetString(buf1, myst->string_id);
-
-	n = 0;
-restart:;
-	FOR_ALL_STATIONS(st) {
-		if (st->xy != 0 && st->alpha_order == n) {
-			SET_DPARAM16(0, st->city->citynametype);
-			SET_DPARAM32(1, st->city->citynameparts);
-			GetString(buf2, st->string_id);
-			if (!str_is_below(buf1,buf2)) {
-				n++;
-				goto restart;
-			}
-			break;
-		}
-	}
-
-	FOR_ALL_STATIONS(st) {
-		if (st->xy != 0 && n <= st->alpha_order)
-			st->alpha_order++;
-	}
-
-	myst->alpha_order = n;
-}
-
 static void StationInitialize(Station *st, TileIndex tile)
 {
 	int i;
@@ -340,9 +294,7 @@ static void StationInitialize(Station *st, TileIndex tile)
 		ge->last_speed = 0;
 		ge->last_age = 0xFF;
 	}
-
-	st->alpha_order = 0xFF;
-	UpdateStationAlphaList(st);
+	_station_sort_dirty = true;
 }
 
 // Update the virtual coords needed to draw the station sign.
@@ -351,7 +303,7 @@ static void UpdateStationVirtCoord(Station *st)
 {
 	Point pt = RemapCoords2(GET_TILE_X(st->xy) * 16, GET_TILE_Y(st->xy) * 16);
 	pt.y -= 32;
-	if (st->facilities&FACIL_AIRPORT && st->airport_type==AT_OILFIELD) pt.y -= 16;
+	if (st->facilities&FACIL_AIRPORT && st->airport_type==AT_OILRIG) pt.y -= 16;
 
 	SET_DPARAM16(0, st->index);
 	SET_DPARAM8(1, st->facilities);
@@ -807,7 +759,7 @@ int32 CmdBuildRailroadStation(int x_org, int y_org, uint32 flags, uint32 p1, uin
 
 	if (st != NULL) {
 		// Reuse an existing station.
-		if (st->owner != 0x10 && st->owner != _current_player)
+		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
 
 		if (st->train_tile != 0) {
@@ -826,7 +778,7 @@ int32 CmdBuildRailroadStation(int x_org, int y_org, uint32 flags, uint32 p1, uin
 			return CMD_ERROR;
 		
 		st->city = ClosestCityFromTile(tile_org, (uint)-1);
-		if (_current_player < 8 && flags&DC_EXEC)
+		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
 			SETBIT(st->city->have_ratings, _current_player);
 
 		if (!GenerateStationName(st, tile_org, 0))
@@ -1078,7 +1030,7 @@ int32 CmdBuildBusStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	}
 
 	if (st != NULL) {
-		if (st->owner != 0x10 && st->owner != _current_player)
+		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
 		
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
@@ -1095,7 +1047,7 @@ int32 CmdBuildBusStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
 
-		if (_current_player < 8 && flags&DC_EXEC)
+		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
 			SETBIT(cit->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
@@ -1188,7 +1140,7 @@ int32 CmdBuildTruckStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	}
 
 	if (st != NULL) {
-		if (st->owner != 0x10 && st->owner != _current_player)
+		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
 		
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
@@ -1205,7 +1157,7 @@ int32 CmdBuildTruckStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
 
-		if (_current_player < 8 && flags&DC_EXEC)
+		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
 			SETBIT(cit->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
@@ -1266,19 +1218,16 @@ static int32 RemoveTruckStation(Station *st, uint32 flags)
 	return _price.remove_truck_station;	
 }
 
-static const uint16 _initial_airport_flags[3] = {
-	4,
-	0x100,
-	0x40,
-};
-
-static const byte _airport_map5_tiles_small[] = {
+// FIXME -- need to move to its corresponding Airport variable
+// Country Airfield (small)
+static const byte _airport_map5_tiles_country[] = {
 	54, 53, 52, 65,
 	58, 57, 56, 55,
 	64, 63, 63, 62
 };
 
-static const byte _airport_map5_tiles_big[] = {
+// City Airport (large)
+static const byte _airport_map5_tiles_city[] = {
 	31, 9, 33, 9, 9, 32,
 	27, 36, 29, 34, 8, 10,
 	30, 11, 35, 13, 20, 21,
@@ -1287,19 +1236,38 @@ static const byte _airport_map5_tiles_big[] = {
 	26, 22, 23, 24, 25, 26
 };
 
+// Metropolitain Airport (large) - 2 runways
+static const byte _airport_map5_tiles_metropolitan[] = {
+	31, 9, 33, 9, 9, 32,
+	27, 36, 29, 34, 8, 10,
+	30, 11, 35, 13, 20, 21,
+ 102,  8,  8,  8,  8, 28,
+	83, 84, 84, 84, 84, 83,
+	26, 23, 23, 23, 23, 26
+};
+
+// International Airport (large) - 2 runways
+static const byte _airport_map5_tiles_international[] = {
+  88, 89, 89, 89, 89, 89, 88,
+ 	51,  8,  8,  8,  8,  8, 32,
+	30,  8, 11, 27, 11,  8, 10,
+	32,  8, 11, 27, 11,  8, 114,
+	87,  8, 11, 85, 11,  8, 114,
+	87,  8,  8,  8,  8,  8, 90,
+	26, 23, 23, 23, 23, 23, 26
+};
+
+// Heliport
 static const byte _airport_map5_tiles_heliport[] = {
 	66,
 };
 
 static const byte * const _airport_map5_tiles[] = {
-	_airport_map5_tiles_small,
-	_airport_map5_tiles_big,
-	_airport_map5_tiles_heliport,
-};
-
-static const int8 _airport_depot_offs[] = {
-	TILE_XY(3,0),
-	TILE_XY(5,0),
+	_airport_map5_tiles_country,				// Country Airfield (small)
+	_airport_map5_tiles_city,						// City Airport (large)
+	_airport_map5_tiles_heliport,				// Heliport
+	_airport_map5_tiles_metropolitan,   // Metropolitain Airport (large) 
+	_airport_map5_tiles_international,	// International Airport (xlarge)
 };
 
 /* Place an Airport
@@ -1313,6 +1281,7 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	Station *st;
 	int32 cost;
 	int w,h;
+	bool airport_upgrade = true;	
 
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
@@ -1327,7 +1296,7 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	{
 		uint num = 0;
 		FOR_ALL_STATIONS(st) {
-			if (st->xy != 0 && st->city == city && st->facilities&FACIL_AIRPORT && st->airport_type != AT_OILFIELD)
+			if (st->xy != 0 && st->city == city && st->facilities&FACIL_AIRPORT && st->airport_type != AT_OILRIG)
 				num++;
 		}
 		if (num >= 2) {
@@ -1354,7 +1323,7 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	}
 
 	if (st != NULL) {
-		if (st->owner != 0x10 && st->owner != _current_player)
+		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
 		
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
@@ -1365,18 +1334,22 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	} else {
 		City *cit;
 
+		airport_upgrade = false;
+
 		st = AllocateStation();
 		if (st == NULL)
 			return CMD_ERROR;
 
 		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
 
-		if (_current_player < 8 && flags&DC_EXEC)
+		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
 			SETBIT(cit->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
 
-		if (!GenerateStationName(st, tile, p1 == 2 ? 5 : 1))
+		// if airport type equals Heliport then generate
+		// type 5 name, which is heliport, otherwise airport names (1)
+		if (!GenerateStationName(st, tile, p1 == AT_HELIPORT ? 5 : 1))
 			return CMD_ERROR;
 
 		if (flags & DC_EXEC)
@@ -1387,15 +1360,23 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	if (flags & DC_EXEC) {
 		st->owner = _current_player;
-		if (_current_player == _local_player && p1 != 2) {
-			_last_built_aircraft_depot_tile = tile + _airport_depot_offs[p1];
-		}
+		if (_current_player == _local_player && p1 <= AT_INTERNATIONAL) {
+      _last_built_aircraft_depot_tile = tile + GetAirport(p1)->airport_depots[0];
+    }
 
 		st->airport_tile = tile;
 		if (!st->facilities) st->xy = tile;
 		st->facilities |= FACIL_AIRPORT;
 		st->airport_type = (byte)p1;
-		st->airport_flags = _initial_airport_flags[p1];
+		st->airport_flags = 0;
+
+		/*	if airport was demolished while planes were en-route to it, the positions can no longer
+				be the same (v->u.air.pos), since different airports have different indexes. So update
+				all planes en-route to this airport. Only update if
+				1. airport is upgraded
+				2. airport is added to existing station (unfortunately unavoideable)
+		*/
+		if (airport_upgrade) {UpdateAirplanesOnNewStation(st);}
 	
 		{
 			const byte *b = _airport_map5_tiles[p1];
@@ -1444,9 +1425,8 @@ END_TILE_LOOP(tile_cur, w,h,tile)
 	}
 
 	if (flags & DC_EXEC) {
-		if (st->airport_type != 2)
-			DeleteWindowById(WC_VEHICLE_DEPOT, tile + _airport_depot_offs[st->airport_type]);
-
+		if (st->airport_type <= AT_INTERNATIONAL)
+      DeleteWindowById(WC_VEHICLE_DEPOT, tile + GetAirport(st->airport_type)->airport_depots[0]);
 		st->airport_tile = 0;
 		st->facilities &= ~FACIL_AIRPORT;
 
@@ -1488,14 +1468,14 @@ int32 CmdBuildBuoy(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		st->dock_tile = ti.tile;
 		st->facilities |= FACIL_DOCK;
 		st->had_vehicle_of_type |= HVOT_BUOY;
-		st->owner = 0x10;
+		st->owner = OWNER_NONE;
 
 		ModifyTile(ti.tile,
 			MP_SETTYPE(MP_STATION) |
 			MP_MAP2 | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR | MP_MAPOWNER | MP_MAP5,
-			st->index,		 /* map2 */
-			0x10,						/* map_owner */
-			0x52						/* map5 */
+			st->index,		/* map2 */
+			OWNER_NONE,		/* map_owner */
+			0x52					/* map5 */
 		);
 
 		UpdateStationVirtCoordDirty(st);
@@ -1511,7 +1491,7 @@ static int32 RemoveBuoy(Station *st, uint32 flags)
 {
 	uint tile;
 
-	if (_current_player >= 8) {
+	if (_current_player >= MAX_PLAYERS) {
 		/* XXX: strange stuff */
 		return_cmd_error(INVALID_STRING_ID);
 	}
@@ -1540,7 +1520,7 @@ static int32 RemoveBuoy(Station *st, uint32 flags)
 	return _price.remove_truck_station;
 }
 
-static const int16 _dock_tileoffs_chkaround[4] = {
+static const TileIndexDiff _dock_tileoffs_chkaround[4] = {
 	TILE_XY(-1,0),
 	0,0,
 	TILE_XY(0,-1),
@@ -1604,7 +1584,7 @@ int32 CmdBuildDock(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	}
 
 	if (st != NULL) {
-		if (st->owner != 0x10 && st->owner != _current_player)
+		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
 		
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
@@ -1621,7 +1601,7 @@ int32 CmdBuildDock(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
 
-		if (_current_player < 8 && flags&DC_EXEC)
+		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
 			SETBIT(cit->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
@@ -1712,17 +1692,19 @@ static void DrawTile_Station(TileInfo *ti)
 	const DrawTileSeqStruct *dtss;
 	const byte *t;
 
+	// station_land array has been increased from 82 elements to 114
+	// but this is something else. If AI builds station with 114 it looks all weird
 	base_img = (_map3_lo[ti->tile] & 0xF) * 82;
 
 	{
 		uint owner = _map_owner[ti->tile];
 		image_or_modificator = 0x315 << 16; /* NOTE: possible bug in ttd here? */
-		if (owner < 8)
+		if (owner < MAX_PLAYERS)
 			image_or_modificator = PLAYER_SPRITE_COLOR(owner);
 	}
 
-	// don't show foundation for docks
-	if (ti->tileh != 0 && ti->map5 < 0x4C)
+	// don't show foundation for docks (docks are between 76 (0x4C) and 81 (0x51))
+	if (ti->tileh != 0 && (ti->map5 < 0x4C || ti->map5 > 0x51))
 		DrawFoundation(ti, ti->tileh);
 
 	t = _station_display_datas[ti->map5];
@@ -1803,8 +1785,8 @@ static void GetTileDesc_Station(uint tile, TileDesc *td)
 
 	m5 = _map5[tile];
 	(str=STR_305E_RAILROAD_STATION, m5 < 8) ||
-	(str=STR_305F_AIRCRAFT_HANGAR, m5==0x20 || m5==0x41) ||
-	(str=STR_3060_AIRPORT, m5 < 0x43) ||
+	(str=STR_305F_AIRCRAFT_HANGAR, m5==32 || m5==45) || // hangars
+	(str=STR_3060_AIRPORT, m5 < 0x43 || (m5 >= 83 && m5 <= 114)) ||
 	(str=STR_3061_TRUCK_LOADING_AREA, m5 < 0x47) ||
 	(str=STR_3062_BUS_STATION, m5 < 0x4B) ||
 	(str=STR_4807_OIL_RIG, m5 == 0x4B) ||
@@ -1835,7 +1817,13 @@ static uint32 GetTileTrackStatus_Station(uint tile, int mode) {
 
 static void TileLoop_Station(uint tile)
 {
-	if (_map5[tile] == 0x27 || _map5[tile] == 0x3A)
+  //FIXME -- GetTileTrackStatus_Station -> animated stationtiles
+  // hardcoded.....not good
+  // 0x27 - large big airport (39)
+  // 0x66 - radar metropolitan airport (102)
+  // 0x5A - radar international airport (90)
+  // 0x3A - flag small airport (58)
+	if (_map5[tile] == 39 || _map5[tile] == 58 || _map5[tile] == 90 || _map5[tile] == 102)
 		AddAnimatedTile(tile);
 	
 	// treat a bouy tile as water.
@@ -1847,17 +1835,33 @@ static void TileLoop_Station(uint tile)
 static void AnimateTile_Station(uint tile)
 {
 	byte m5 = _map5[tile];
+	//FIXME -- AnimateTile_Station -> not nice code, lots of things double
+  // again hardcoded...was a quick hack
 
-	if (m5 >= 0x27 && m5 <= 0x32) {
+  // turning radar / windsack on airport
+	if (m5 >= 39 && m5 <= 50) { // turning radar (39 - 50)
 		if (_tick_counter & 3)
 			return;
 		
-		if (++m5 == 0x32+1)
-			m5 = 0x27;
+		if (++m5 == 50+1)
+			m5 = 39;
 
 		_map5[tile] = m5;
 		MarkTileDirtyByTile(tile);
-	} else if (m5 >= 0x3A && m5 <= 0x3D) {
+  //added - begin
+	} else if (m5 >= 90 && m5 <= 113) { // turning radar with ground under it (different fences) (90 - 101 | 102 - 113)
+		if (_tick_counter & 3)
+			return;
+		
+		m5++;
+		
+		if (m5 == 101+1) {m5 = 90;}  // radar with fences in south
+		else if (m5 == 113+1) {m5 = 102;} // radar with fences in north
+
+		_map5[tile] = m5;
+		MarkTileDirtyByTile(tile);		
+	//added - end
+	} else if (m5 >= 0x3A && m5 <= 0x3D) {  // windsack (58 - 61)
 		if (_tick_counter & 1)
 			return;
 
@@ -1871,7 +1875,9 @@ static void AnimateTile_Station(uint tile)
 
 static void ClickTile_Station(uint tile)
 {
-	if (_map5[tile] == 0x20 || _map5[tile] == 0x41) {
+  // 0x20 - hangar large airport (32)
+  // 0x41 - hangar small airport (65)
+	if (_map5[tile] == 32 || _map5[tile] == 65) {
 		ShowAircraftDepotWindow(tile);
 	} else {
 		ShowStationViewWindow(_map2[tile]);
@@ -1900,9 +1906,7 @@ static uint32 VehicleEnter_Station(Vehicle *v, uint tile, int x, int y)
 			if ((!(v->next_order & OF_NON_STOP) && !_patches.new_nonstop) || 
 					 (((v->next_order & OT_MASK) == OT_GOTO_STATION && v->next_order_param == station_id))) {
 
-				if (_patches.new_nonstop && (v->next_order & OF_NON_STOP)) {
-					v->cur_order_index++;
-				} else if (v->next_order != OT_LEAVESTATION && v->last_station_visited != station_id) {
+				if (!(_patches.new_nonstop && (v->next_order & OF_NON_STOP)) && v->next_order != OT_LEAVESTATION && v->last_station_visited != station_id) {
 					x &= 0xF;
 					y &= 0xF;
 					
@@ -1934,6 +1938,9 @@ static uint32 VehicleEnter_Station(Vehicle *v, uint tile, int x, int y)
 
 				bb = *b;
 
+				/* bb bits 1..0 describe current the two parking spots.
+				0 means occupied, 1 means free. */
+
 				// Station busy?
 				if (bb & 0x80 || (bb&3) == 0)
 					return 8;
@@ -1957,12 +1964,11 @@ static uint32 VehicleEnter_Station(Vehicle *v, uint tile, int x, int y)
 static void DeleteStation(Station *st)
 {
 	int index;
-
 	st->xy = 0;
 
 	DeleteName(st->string_id);
 	MarkStationDirty(st);
-	RemoveStationFromAlphaList(st);
+	_station_sort_dirty = true;
 
 	index = st->index;
 	DeleteWindowById(WC_STATION_VIEW, index);
@@ -1975,7 +1981,7 @@ void DeleteAllPlayerStations()
 	Station *st;
 
 	FOR_ALL_STATIONS(st) {
-		if (st->xy && st->owner < 8)
+		if (st->xy && st->owner < MAX_PLAYERS)
 			DeleteStation(st);		
 	}
 }
@@ -2028,7 +2034,7 @@ static void UpdateStationRating(Station *st)
 			}
 
 			{
-				if (!IS_HUMAN_PLAYER(st->owner) && st->owner != 0x10)
+				if (!IS_HUMAN_PLAYER(st->owner) && st->owner != OWNER_NONE)
 							rating += _rating_boost[_opt.diff.competitor_intelligence];
 			}
 
@@ -2189,7 +2195,7 @@ int32 CmdRenameStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		st->string_id = str;
 		UpdateStationVirtCoord(st);
 		DeleteName(old_str);
-		UpdateStationAlphaList(st);
+		_station_sort_dirty = true;
 		MarkWholeScreenDirty();
 	} else {
 		DeleteName(str);
@@ -2314,8 +2320,8 @@ void BuildOilRig(uint tile)
 			_map2[tile] = st->index;
 
 			st->owner = 0x10;
-			st->airport_flags = 0x40;
-			st->airport_type = AT_OILFIELD;
+      st->airport_flags = 0;
+			st->airport_type = AT_OILRIG;
 			st->xy = tile;
 			st->bus_tile = 0;
 			st->lorry_tile = 0;
@@ -2367,6 +2373,7 @@ static void ChangeTileOwner_Station(uint tile, byte old_player, byte new_player)
 		Station *st = DEREF_STATION(_map2[tile]);
 		_map_owner[tile] = new_player;
 		st->owner = new_player;
+		_station_sort_dirty = true;
 	} else {
 		DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 	}
@@ -2378,7 +2385,7 @@ static int32 ClearTile_Station(uint tile, byte flags) {
 
 	if (flags & DC_AUTO) {
 		if (m5 < 8) return_cmd_error(STR_300B_MUST_DEMOLISH_RAILROAD);
-		if (m5 < 0x43) return_cmd_error(STR_300E_MUST_DEMOLISH_AIRPORT_FIRST);
+		if (m5 < 0x43 || (m5 >= 83 && m5 <= 114)) return_cmd_error(STR_300E_MUST_DEMOLISH_AIRPORT_FIRST);
 		if (m5 < 0x47) return_cmd_error(STR_3047_MUST_DEMOLISH_TRUCK_STATION);
 		if (m5 < 0x4B) return_cmd_error(STR_3046_MUST_DEMOLISH_BUS_STATION);
 		if (m5 == 0x52) return_cmd_error(STR_306A_BUOY_IN_THE_WAY);
@@ -2392,7 +2399,8 @@ static int32 ClearTile_Station(uint tile, byte flags) {
 	if (m5 < 8)
 		return RemoveRailroadStation(st, flags);
 
-	if (m5 < 0x43)
+	// original airports < 67, new airports between 83 - 114
+	if (m5 < 0x43 || ( m5 >= 83 && m5 <= 114) )
 		return RemoveAirport(st, flags);
 		
 	if (m5 < 0x47)
@@ -2418,6 +2426,7 @@ void InitializeStations()
 	for(i=0;i!=lengthof(_stations); i++)
 		_stations[i].index=i;
 	_station_tick_ctr = 0;
+	_station_sort_dirty = true;	// set stations to be sorted on first load
 }
 
 
@@ -2449,9 +2458,10 @@ static const byte _station_desc[] = {
 	SLE_VAR(Station,trainst_w,			SLE_UINT8),
 	SLE_CONDVAR(Station,trainst_h,	SLE_UINT8, 2, 255),
 
-	SLE_VAR(Station,alpha_order,		SLE_UINT8),
-	SLE_VAR(Station,string_id,			SLE_STRINGID),
+	// alpha_order was stored here in savegame format 0 - 3
+	SLE_CONDARR(NullStruct,null,SLE_FILE_U8 | SLE_VAR_NULL, 1, 0, 3),
 
+	SLE_VAR(Station,string_id,			SLE_STRINGID),
 	SLE_VAR(Station,had_vehicle_of_type,SLE_UINT16),
 
 	SLE_VAR(Station,time_since_load,		SLE_UINT8),
@@ -2464,11 +2474,17 @@ static const byte _station_desc[] = {
 	SLE_VAR(Station,bus_stop_status,		SLE_UINT8),
 	SLE_VAR(Station,blocked_months,			SLE_UINT8),
 
-	SLE_VAR(Station,airport_flags,			SLE_UINT16),
+	SLE_CONDVAR(Station,airport_flags,			SLE_VAR_U32 | SLE_FILE_U16, 0, 2),
+	SLE_CONDVAR(Station,airport_flags,			SLE_UINT32, 3, 255),
+
 	SLE_VAR(Station,last_vehicle,				SLE_UINT16),
 
+	SLE_CONDVAR(Station,class_id,				SLE_UINT8, 3, 255),
+	SLE_CONDVAR(Station,stat_id,				SLE_UINT8, 3, 255),
+	SLE_CONDVAR(Station,build_date,			SLE_UINT16, 3, 255),
+
 	// reserve extra space in savegame here. (currently 32 bytes)
-	SLE_CONDARR(NullStruct,null,SLE_FILE_U64 | SLE_VAR_NULL, 4, 2, 255),
+	SLE_CONDARR(NullStruct,null,SLE_FILE_U8 | SLE_VAR_NULL, 32, 2, 255),
 
 	SLE_END()
 };
