@@ -100,6 +100,13 @@ const uint16 _currency_string_list[] = {
 	INVALID_STRING_ID
 };
 
+static const uint16 _cargo_string_list[NUM_LANDSCAPE][NUM_CARGO] = {
+	/* LT_NORMAL */ {STR_PASSENGERS, STR_TONS, STR_BAGS, STR_LITERS, STR_ITEMS, STR_CRATES, STR_TONS, STR_TONS, STR_TONS, STR_TONS, STR_BAGS, STR_RES_OTHER},
+	/* LT_HILLY */  {STR_PASSENGERS, STR_TONS, STR_BAGS, STR_LITERS, STR_ITEMS, STR_CRATES, STR_TONS, STR_TONS, STR_RES_OTHER, STR_TONS, STR_BAGS, STR_TONS},
+	/* LT_DESERT */ {STR_PASSENGERS, STR_LITERS, STR_BAGS, STR_LITERS, STR_TONS, STR_CRATES, STR_TONS, STR_TONS, STR_TONS, STR_LITERS, STR_BAGS, STR_TONS},
+	/* LT_CANDY */  {STR_PASSENGERS, STR_TONS, STR_BAGS, STR_NOTHING, STR_NOTHING, STR_TONS, STR_TONS, STR_LITERS, STR_TONS, STR_NOTHING, STR_LITERS, STR_NOTHING}
+};
+
 // get a mask of the allowed currencies depending on the year
 uint GetMaskOfAllowedCurrencies()
 {
@@ -430,6 +437,21 @@ static byte *DecodeString(byte *buff, const byte *str)
 				buff = str_cat(buff, (const byte*)_openttd_revision);
 #endif
 				break;
+			case 3: { // {SHORTCARGO}
+				// Layout:
+				// 8-bit = cargo type
+				// 16-bit = cargo count
+				char *s;
+				uint16 cargo_str = _cargo_string_list[_opt.landscape][(byte)GetParamInt8()];
+				uint16 multiplier = (cargo_str == STR_LITERS) ? 1000 : 1;
+				// liquid type of cargo is multiplied by 100 to get correct amount
+				buff = FormatCommaNumber(buff, GetParamInt16() * multiplier );
+				s = GetStringPtr(cargo_str);
+				
+				memcpy(buff++, " ", 1);
+				while (*s) *buff++ = *s++;
+				break;
+			}
 
 			default:
 				error("!invalid escape sequence in string");
@@ -442,7 +464,7 @@ static byte *DecodeString(byte *buff, const byte *str)
 			break;
 		case 0x87: { // {VOLUME}
 			char *s;
-			buff = FormatCommaNumber(buff, GetParamInt16() * 100);
+			buff = FormatCommaNumber(buff, GetParamInt16() * 1000);
 			memcpy(buff++, " ", 1);
 			s = GetStringPtr(STR_LITERS);
 			while (*s) *buff++ = *s++;
@@ -752,7 +774,11 @@ bool ReadLanguagePack(int lang_index) {
 	byte *s;
 
 #define HDR ((LanguagePackHeader*)lang_pack)
-	lang_pack = ReadFileToMem(_dynlang.ent[lang_index].file, &len, 100000);
+	{
+		char *lang = str_fmt("%s%s", _path.lang_dir, _dynlang.ent[lang_index].file);
+		lang_pack = ReadFileToMem(lang, &len, 100000);
+		free(lang);
+	}
 	if (lang_pack == NULL) return false;
 	if (len < sizeof(LanguagePackHeader) || 
 			HDR->ident != TO_LE32(LANGUAGE_PACK_IDENT) ||
@@ -817,7 +843,9 @@ void InitializeLanguagePacks()
 
 	// go through the language files and make sure that they are valid.
 	for(i=m=0; i!=n; i++) {
-		in = fopen(files[i], "rb");
+		char *s = str_fmt("%s%s", _path.lang_dir, files[i]);
+		in = fopen(s, "rb");
+		free(s);
 		if (!in || 
 				(j = fread(&hdr, sizeof(hdr), 1, in), fclose(in), j) != 1 ||
 				hdr.ident != TO_LE32(LANGUAGE_PACK_IDENT) ||
