@@ -10,6 +10,13 @@
 #include "engine.h"
 #include "player.h"
 
+
+#define is_firsthead_sprite(spritenum) \
+	(is_custom_sprite(spritenum) \
+		? is_custom_firsthead_sprite(spritenum) \
+		: _engine_sprite_add[spritenum] == 0)
+
+
 static const byte _vehicle_initial_x_fract[4] = {10,8,4,8};
 static const byte _vehicle_initial_y_fract[4] = {8,4,8,10};
 static const byte _state_dir_table[4] = { 0x20, 8, 0x10, 4 };
@@ -92,9 +99,9 @@ static int GetRealisticAcceleration(Vehicle *v)
 		uint mass = rvi->weight + ((_cargoc.weights[u->cargo_type] * u->cargo_count) >> 4);
 		if (rvi->power) emass += mass;
 
-		if (u->z_pos > u->u.rail.last_z) {
+		if (u->u.rail.flags & VRF_GOINGUP) {
 			f += (float)mass * ( -F_GRAV * F_THETA);
-		} else if (u->z_pos < u->u.rail.last_z) {
+		} else if (u->u.rail.flags & VRF_GOINGDOWN) {
 			f += (float)mass * ( F_GRAV * F_THETA);
 		}
 
@@ -145,6 +152,12 @@ int GetTrainImage(Vehicle *v, byte direction)
 {
 	int img = v->spritenum;
 	int base;
+
+	if (is_custom_sprite(img)) {
+		base = GetCustomVehicleSprite(v, direction + 4 * is_custom_secondhead_sprite(img));
+		if (base) return base;
+		img = _engine_original_sprites[v->engine_type];
+	}
 	
 	base = _engine_sprite_base[img] + ((direction + _engine_sprite_add[img]) & _engine_sprite_and[img]);
 
@@ -158,14 +171,27 @@ void DrawTrainEngine(int x, int y, int engine, uint32 image_ormod)
 	const RailVehicleInfo *rvi = &_rail_vehicle_info[engine];
 	
 	int img = rvi->image_index;
-	uint32 image;
+	uint32 image = 0;
 
-	image = (6 & _engine_sprite_and[img]) + _engine_sprite_base[img];
+	if (is_custom_sprite(img)) {
+		image = GetCustomEngineSprite(engine, -1, CID_PURCHASE, 0, 0, 6);
+		if (!image) img = _engine_original_sprites[engine];
+	}
+	if (!image) {
+		image = (6 & _engine_sprite_and[img]) + _engine_sprite_base[img];
+	}
 
 	if (rvi->flags & RVI_MULTIHEAD) {
 		DrawSprite(image | image_ormod, x-14, y);
 		x += 15;
-		image = ((6 + _engine_sprite_add[img+1]) & _engine_sprite_and[img+1]) + _engine_sprite_base[img+1];	
+		image = 0;
+		if (is_custom_sprite(img)) {
+			image = GetCustomEngineSprite(engine, -1, CID_PURCHASE, 0, 0, 2);
+			if (!image) img = _engine_original_sprites[engine];
+		}
+		if (!image) {
+			image = ((6 + _engine_sprite_add[img+1]) & _engine_sprite_and[img+1]) + _engine_sprite_base[img+1];	
+		}
 	}
 	DrawSprite(image | image_ormod, x, y);
 }
@@ -181,7 +207,7 @@ void DrawTrainEngineInfo(int engine, int x, int y, int maxw)
 	SET_DPARAM16(3, rvi->power << multihead);
 	SET_DPARAM16(1, rvi->weight << multihead);
 
-	SET_DPARAM32(4, (rvi->running_cost_base * _price.running_rail[rvi->running_cost_class-1] >> 8) << multihead);
+	SET_DPARAM32(4, (rvi->running_cost_base * _price.running_rail[rvi->engclass] >> 8) << multihead);
 	
 	cap = rvi->capacity;
 	SET_DPARAM16(5, STR_8838_N_A);
@@ -221,8 +247,7 @@ int32 CmdBuildRailWagon(uint engine, uint tile, uint32 flags)
 			u = _vehicles;
 			for(;;) {
 				if (u->type == VEH_Train && u->tile == (TileIndex)tile &&
-						u->subtype == 4 && u->spritenum == img) {
-					
+				    u->subtype == 4 && u->engine_type == engine) {
 					u = GetLastVehicleInChain(u);
 					break;
 				}
@@ -245,7 +270,7 @@ int32 CmdBuildRailWagon(uint engine, uint tile, uint32 flags)
 
 			v->x_pos = x;
 			v->y_pos = y;
-			v->u.rail.last_z = v->z_pos = GetSlopeZ(x,y);
+			v->z_pos = GetSlopeZ(x,y);
 			v->owner = _current_player;
 			v->z_height = 6;
 			v->u.rail.track = 0x80;
@@ -255,6 +280,11 @@ int32 CmdBuildRailWagon(uint engine, uint tile, uint32 flags)
 			if (u != NULL) {
 				u->next = v;
 				v->subtype = 2;
+				v->u.rail.first_engine = u->u.rail.first_engine;
+				if (v->u.rail.first_engine == 0xffff && u->subtype == 0)
+					v->u.rail.first_engine = u->engine_type;
+			} else {
+				v->u.rail.first_engine = 0xffff;
 			}
 
 			v->cargo_type = rvi->cargo_type;
@@ -331,6 +361,11 @@ static const byte _railveh_score[] = {
 };
 
 
+static int32 EstimateTrainCost(const RailVehicleInfo *rvi)
+{
+	return (rvi->base_cost * (_price.build_railvehicle >> 3)) >> 5;
+}
+
 /* Build a railroad vehicle
  * p1 = vehicle type id
  */
@@ -355,7 +390,7 @@ int32 CmdBuildRailVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		return CmdBuildRailWagon(p1, tile, flags);
 	}
 
-	value = (rvi->base_cost * (_price.build_railvehicle >> 3)) >> 5;
+	value = EstimateTrainCost(rvi);
 
 	if (!(flags & DC_QUERY_COST)) {
 		v = AllocateVehicle();
@@ -376,9 +411,10 @@ int32 CmdBuildRailVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			v->owner = _current_player;
 			v->x_pos = (x |= _vehicle_initial_x_fract[dir]);
 			v->y_pos = (y |= _vehicle_initial_y_fract[dir]);
-			v->u.rail.last_z = v->z_pos = GetSlopeZ(x,y);
+			v->z_pos = GetSlopeZ(x,y);
 			v->z_height = 6;
 			v->u.rail.track = 0x80;
+			v->u.rail.first_engine = 0xffff;
 			v->vehstatus = VS_HIDDEN | VS_STOPPED | VS_DEFPAL;
 //			v->subtype = 0;
 			v->spritenum = rvi->image_index;
@@ -438,6 +474,7 @@ int32 CmdBuildRailVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 				u->z_pos = v->z_pos;
 				u->z_height = 6;
 				u->u.rail.track = 0x80;
+				v->u.rail.first_engine = 0xffff;
 				u->vehstatus = VS_HIDDEN | VS_DEFPAL;
 				u->subtype = 2;
 				u->spritenum = v->spritenum + 1;
@@ -488,7 +525,7 @@ int CheckStoppedInDepot(Vehicle *v)
 	TileIndex tile = v->tile;
 	
 	/* check if stopped in a depot */
-	if (!IsTrainDepotTile(tile)) {
+	if (!IsTrainDepotTile(tile) || v->cur_speed != 0) {
 errmsg:
 		_error_message = STR_881A_TRAINS_CAN_ONLY_BE_ALTERED;
 		return -1;
@@ -510,8 +547,11 @@ errmsg:
 static Vehicle *UnlinkWagon(Vehicle *v, Vehicle *first)
 {
 	// unlinking the first vehicle of the chain?
+	v->u.rail.first_engine = 0xffff;
 	if (v == first) {
+		Vehicle *u;
 		if ((v=v->next) == NULL) return NULL;
+		for (u=v; u; u=u->next) u->u.rail.first_engine = v->engine_type;
 		v->subtype = 4;
 		return v;
 	} else {
@@ -554,7 +594,8 @@ int32 CmdMoveRailVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	src = &_vehicles[p1 & 0xffff];
 	if (src->type != VEH_Train) return CMD_ERROR;
 
-	is_loco = !(_rail_vehicle_info[src->engine_type].flags & RVI_WAGON) && _engine_sprite_add[src->spritenum] == 0;
+	is_loco = !(_rail_vehicle_info[src->engine_type].flags & RVI_WAGON)
+	          && is_firsthead_sprite(src->spritenum);
 
 	// if nothing is selected as destination, try and find a matching vehicle to drag to.
 	if (((int32)p1 >> 16) == -1) {	
@@ -592,7 +633,8 @@ int32 CmdMoveRailVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			return_cmd_error(STR_8819_TRAIN_TOO_LONG);
 
 		// if it's a multiheaded vehicle we're dragging to, drag to the vehicle before..
-		while (_engine_sprite_add[dst->spritenum] != 0) {
+		while (is_custom_secondhead_sprite(dst->spritenum)
+		       || (!is_custom_sprite(dst->spritenum) && _engine_sprite_add[dst->spritenum] != 0)) {
 			Vehicle *v = GetPrevVehicleInChain(dst);
 			if (!v || src == v) break;
 			dst = v;
@@ -645,6 +687,7 @@ int32 CmdMoveRailVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			} else {
 				src->subtype = 4;
 			}
+			src->u.rail.first_engine = 0xffff;
 		} else {
 			if (src->subtype == 0) {
 				// the vehicle was previously a loco. need to free the schedule list and delete vehicle windows etc.
@@ -654,11 +697,19 @@ int32 CmdMoveRailVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			
 			src->subtype = 2;
 			src->unitnumber = 0; // doesn't occupy a unitnumber anymore.
-			
+
+			// setup first_engine
+			src->u.rail.first_engine = dst->u.rail.first_engine;
+			if (src->u.rail.first_engine == 0xffff && dst->subtype == 0)
+				src->u.rail.first_engine = dst->engine_type;
+
 			// link in the wagon(s) in the chain.
 			{
 				Vehicle *v = src;
-				while (v->next != NULL) v=v->next;
+				while (v->next != NULL) {
+					v->next->u.rail.first_engine = v->u.rail.first_engine;
+					v = v->next;
+				}
 				v->next = dst->next;
 			}
 			dst->next = src;
@@ -726,7 +777,8 @@ int32 CmdSellRailWagon(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		if (p2 != 1) {
 			// sell last part of multiheaded?
 			last = GetLastVehicleInChain(v);
-			if (_engine_sprite_add[last->spritenum] == 0) last = NULL;
+			if (is_firsthead_sprite(last->spritenum))
+				last = NULL;
 		} else {
 			last = NULL;
 		}
@@ -827,6 +879,15 @@ static void UpdateVarsAfterSwap(Vehicle *v)
 	EndVehicleMove(v);
 }
 
+static void SetLastSpeed(Vehicle *v, int spd) {	
+	int old = v->u.rail.last_speed;
+	if (spd != old) {
+		v->u.rail.last_speed = spd;
+		if (_patches.vehicle_speed || !old != !spd)
+			InvalidateWindowWidget(WC_VEHICLE_VIEW, v->index, 4);
+	}
+}
+
 static void ReverseTrainSwapVeh(Vehicle *v, int l, int r)
 {
 	Vehicle *a, *b;
@@ -871,6 +932,9 @@ static void ReverseTrainDirection(Vehicle *v)
 	int l = 0, r = -1;
 	Vehicle *u;
 
+	if (IsTrainDepotTile(v->tile))
+		InvalidateWindow(WC_VEHICLE_DEPOT, v->tile);
+
 	// count number of vehicles
 	u = v;
 	do r++; while ( (u = u->next) != NULL );
@@ -879,6 +943,11 @@ static void ReverseTrainDirection(Vehicle *v)
 	do {
 		ReverseTrainSwapVeh(v, l++, r--);
 	} while (l <= r);
+
+	if (IsTrainDepotTile(v->tile))
+		InvalidateWindow(WC_VEHICLE_DEPOT, v->tile);
+
+	v->u.rail.flags &= ~VRF_REVERSING;
 }
 
 /* p1 = vehicle */
@@ -893,15 +962,20 @@ int32 CmdReverseTrainDirection(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	_error_message = STR_EMPTY;
 
-	if (v->u.rail.track & 0x80 || IsTrainDepotTile(v->tile))
-		return CMD_ERROR;
+//	if (v->u.rail.track & 0x80 || IsTrainDepotTile(v->tile))
+//		return CMD_ERROR;
 
 	if (v->u.rail.crash_anim_pos != 0 || v->breakdown_ctr != 0)
 		return CMD_ERROR;
 
 	if (flags & DC_EXEC) {
-		v->cur_speed = 0;
-		ReverseTrainDirection(v);
+		if (_patches.realistic_acceleration && v->cur_speed != 0) {
+			v->u.rail.flags ^= VRF_REVERSING;
+		} else {
+			v->cur_speed = 0;
+			SetLastSpeed(v, 0);
+			ReverseTrainDirection(v);
+		}
 	}
 	return 0;
 }
@@ -1102,11 +1176,16 @@ static void HandleLocomotiveSmokeCloud(Vehicle *v)
 	u = v;
 
 	do {
+		int engtype = v->engine_type;
+
 		// no smoke?
-		if (v->spritenum >= 0x19 || (v->vehstatus&VS_HIDDEN) || (v->u.rail.track & 0xC0) )
+		if (_rail_vehicle_info[engtype].flags & 2
+		    || _engines[engtype].railtype > 0
+		    || (v->vehstatus&VS_HIDDEN) || (v->u.rail.track & 0xC0) )
 			continue;
 
-		if (v->spritenum < 4) {
+		switch (_rail_vehicle_info[engtype].engclass) {
+		case 0:
 			// steam smoke.
 			if ( (v->tick_counter&0xF) == 0 && !IsTrainDepotTile(v->tile) && !IsTunnelTile(v->tile)) {
 				CreateEffectVehicleRel(v, 
@@ -1115,17 +1194,21 @@ static void HandleLocomotiveSmokeCloud(Vehicle *v)
 					10,
 					EV_STEAM_SMOKE);
 			}
+			break;
 
-		} else if (v->spritenum < 0x14) {
+		case 1:
 			// diesel smoke
 			if (u->cur_speed <= 40 && !IsTrainDepotTile(v->tile) && !IsTunnelTile(v->tile) && (uint16)Random() <= 0x1E00) {
 				CreateEffectVehicleRel(v, 0,0,10, EV_SMOKE_3);
 			}
-		} else {
+			break;
+
+		case 2:
 			// blue spark
 			if ( (v->tick_counter&0x3) == 0 && !IsTrainDepotTile(v->tile) && !IsTunnelTile(v->tile) && (uint16)Random() <= 0x5B0) {
 				CreateEffectVehicleRel(v, 0,0,10, EV_SMOKE_2);
 			}
+			break;
 		}
 	} while ( (v = v->next) != NULL );
 
@@ -1133,21 +1216,20 @@ static void HandleLocomotiveSmokeCloud(Vehicle *v)
 
 static void TrainPlayLeaveStationSound(Vehicle *v)
 {
-	if (v->spritenum < 4) {
-		SndPlayVehicleFx(2, v);
-	} else if (v->spritenum < 0x19) {
-		SndPlayVehicleFx(8, v);
-	} else if (v->spritenum < 0x1C) {
-		SndPlayVehicleFx(0x47, v);
-	} else {
-		SndPlayVehicleFx(0x41, v);
+	static const byte sfx[3] = { 0x2, 0x8, 0x8 };
+	int engtype = v->engine_type;
+
+	switch (_engines[engtype].railtype) {
+		case 0: SndPlayVehicleFx(sfx[_rail_vehicle_info[engtype].engclass], v); break;
+		case 1: SndPlayVehicleFx(0x41, v); break;
+		case 2: SndPlayVehicleFx(0x47, v); break;
 	}
 }
 
 static bool CheckTrainStayInDepot(Vehicle *v)
 {
 	Vehicle *u;
-	if (v->u.rail.track != 0x80)
+	if (v->u.rail.track != 0x80)	// first wagon (eg engine) in depot
 		return false;
 
 	// make sure that all vehicles are in the depot
@@ -1155,6 +1237,11 @@ static bool CheckTrainStayInDepot(Vehicle *v)
 	if (u->u.rail.track != 0x80)
 		return false;
 
+	// fix hung train if both ends are in depots (when here first wagon and last wagon is in depot)
+	// both first and last should be in the same depot, eg on the same tile
+	if (v->tile != u->tile)
+		return false;
+
 	if (v->u.rail.force_proceed == 0) {
 		if (++v->load_unload_time_rem < 37)
 			return true;
@@ -1313,8 +1400,8 @@ static byte ChooseTrainTrack(Vehicle *v, uint tile, int direction, byte trackbit
 		}
 	} else {
 		int i, r;
-		uint best_bird_dist;
-		uint best_track_dist;
+		uint best_bird_dist  = 0;
+		uint best_track_dist = 0;
 		byte train_dir = v->direction & 3;
 
 
@@ -1382,8 +1469,8 @@ static bool CheckReverseTrain(Vehicle *v)
 	TrainTrackFollowerData fd;
 	int i, r;
 	int best_track;
-	uint best_bird_dist;
-	uint best_track_dist;
+	uint best_bird_dist  = 0;
+	uint best_track_dist = 0;
 	uint reverse, reverse_best;
 
 	if (_opt.diff.line_reverse_mode != 0 ||
@@ -1461,11 +1548,20 @@ static bool ProcessTrainOrder(Vehicle *v)
 			return false;
 	}
 
+	if ((v->next_order & (OT_MASK|OF_UNLOAD|OF_FULL_LOAD)) == (OT_GOTO_DEPOT|OF_UNLOAD|OF_FULL_LOAD) &&
+		v->date_of_last_service+v->service_interval > _date) {
+		v->cur_order_index++;
+	}
+
 	// check if we've reached the checkpoint?
 	if ((v->next_order & OT_MASK) == OT_GOTO_CHECKPOINT && v->tile == v->dest_tile) {
 		v->cur_order_index++;
 	}
 
+	// check if we've reached a non-stop station while TTDPatch nonstop is enabled.. 
+	if (_patches.new_nonstop && (v->next_order & OF_NON_STOP) && v->next_order_param == _map2[v->tile]) { 
+		v->cur_order_index++; 
+	}
 
 	// Get the current order
 	if (v->cur_order_index >= v->num_orders)
@@ -1569,7 +1665,7 @@ static int UpdateTrainSpeed(Vehicle *v)
 	uint spd;
 	uint accel;
 
-	if (v->vehstatus & VS_STOPPED) {
+	if (v->vehstatus & VS_STOPPED || v->u.rail.flags & VRF_REVERSING) {
 		accel = -v->acceleration * 2;
 	} else {
 		accel = v->acceleration;
@@ -1638,9 +1734,15 @@ static byte AfterSetTrainPos(Vehicle *v)
 	new_z = GetSlopeZ(v->x_pos, v->y_pos);
 	_get_z_hint = 0;
 
-	v->u.rail.last_z = old_z = v->z_pos;
+	old_z = v->z_pos;
 	v->z_pos = new_z;
 
+	v->u.rail.flags &= ~(VRF_GOINGUP | VRF_GOINGDOWN);
+
+	if (new_z != old_z) {
+		v->u.rail.flags |= (new_z > old_z) ? VRF_GOINGUP : VRF_GOINGDOWN;
+	}
+
 	VehiclePositionChanged(v);
 	EndVehicleMove(v);
 	return old_z;
@@ -1696,8 +1798,17 @@ static bool CheckCompatibleRail(Vehicle *v, uint tile)
 			IS_TILETYPE(tile, MP_STATION)) {
 
 	} else if (IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
-		if ((_map5[tile] & 0xC0) == 0xC0)
-			return true;
+		if ((_map5[tile] & 0xC0) == 0xC0) {// is bridge middle part?
+			TileInfo ti;
+			FindLandscapeHeightByTile(&ti, tile);
+
+			// correct Z position of a train going under a bridge on slopes
+			if (CORRECT_Z(ti.tileh))
+				ti.z += 8;
+			
+			if(v->z_pos != ti.z) // train is going over bridge
+				return true;
+		}
 	} else
 		return true;
 
@@ -2027,13 +2138,13 @@ red_light: {
 			v->cur_speed = 0;
 			v->subspeed = 0;
 			v->progress = 255-100;
-			if (++v->load_unload_time_rem < 254)
+			if (++v->load_unload_time_rem < _patches.wait_oneway_signal * 20)
 				return;
 		} else if (_map3_lo[gp.new_tile] & _signal_onedir[i]){
 			v->cur_speed = 0;
 			v->subspeed = 0;
 			v->progress = 255-10;
-			if (++v->load_unload_time_rem < 254) {
+			if (++v->load_unload_time_rem < _patches.wait_twoway_signal * 73) {
 				uint o_tile = gp.new_tile + _tileoffs_by_dir[dir>>1];
 				/* check if a train is waiting on the other side */
 				if (VehicleFromPos(o_tile, (void*)( (o_tile<<8) | (dir^4)), (VehicleFromPosProc*)CheckVehicleAtSignal) == NULL)
@@ -2274,10 +2385,15 @@ static void TrainLocoHandler(Vehicle *v, bool mode)
 		v->breakdown_ctr--;
 	}
 
+	if (v->u.rail.flags & VRF_REVERSING && v->cur_speed == 0) {
+		ReverseTrainDirection(v);
+	}
+
 	/* exit if train is stopped */
 	if (v->vehstatus & VS_STOPPED && v->cur_speed == 0)
 		return;
 
+
 	if (ProcessTrainOrder(v)) {
 		v->load_unload_time_rem = 0;
 		v->cur_speed = 0;
@@ -2311,11 +2427,7 @@ static void TrainLocoHandler(Vehicle *v, bool mode)
 		} while (--j != 0);
 	}
 
-	if (v->cur_speed != v->u.rail.last_speed) {
-		v->u.rail.last_speed = v->cur_speed;
-		if (_patches.vehicle_speed)
-			InvalidateWindowWidget(WC_VEHICLE_VIEW, v->index, 4);
-	}
+	SetLastSpeed(v, v->cur_speed);
 }
 
 
@@ -2355,6 +2467,8 @@ void TrainEnterDepot(Vehicle *v, uint tile)
 	v->load_unload_time_rem = 0;
 	v->cur_speed = 0;
 
+	MaybeRenewVehicle(v, EstimateTrainCost(&_rail_vehicle_info[v->engine_type]));
+
 	if ((v->next_order&OT_MASK) == OT_GOTO_DEPOT) {
 		InvalidateWindow(WC_VEHICLE_VIEW, v->index);
 	
@@ -2365,7 +2479,7 @@ void TrainEnterDepot(Vehicle *v, uint tile)
 		if (t & OF_UNLOAD) { v->u.rail.days_since_order_progr = 0; v->cur_order_index++; }
 
 		// User initiated?
-		if (t & OF_FULL_LOAD) {
+		else if (t & OF_FULL_LOAD) {
 			v->vehstatus |= VS_STOPPED;
 			if (v->owner == _local_player) {
 				SET_DPARAM16(0, v->unitnumber);
@@ -2392,6 +2506,9 @@ static void CheckIfTrainNeedsService(Vehicle *v)
 
 	if (v->vehstatus & VS_STOPPED)
 		return;
+
+	if (_patches.gotodepot && IS_HUMAN_PLAYER(v->owner) && ScheduleHasDepotOrders(v->schedule_ptr))
+		return;
 	
 	// Don't interfere with a depot visit scheduled by the user, or a
 	// depot visit by the order list.
@@ -2426,7 +2543,7 @@ int32 GetTrainRunningCost(Vehicle *v)
 	do {
 		const RailVehicleInfo *rvi = &_rail_vehicle_info[v->engine_type];
 		if (rvi->running_cost_base)
-			cost += rvi->running_cost_base * _price.running_rail[rvi->running_cost_class-1];
+			cost += rvi->running_cost_base * _price.running_rail[rvi->engclass];
 	} while ( (v=v->next) != NULL );
 
 	return cost;
@@ -2513,3 +2630,11 @@ void InitializeTrains()
 {
 	_age_cargo_skip_counter = 1;
 }
+
+int ScheduleHasDepotOrders(uint16 *schedule)
+{
+	for (;*schedule!=0;schedule++)
+		if ((*schedule&OT_MASK) == OT_GOTO_DEPOT)
+			return true;
+	return false;
+}
