@@ -108,7 +108,7 @@ static void NewRailVehicleWndProc(Window *w, WindowEvent *e)
 				if (HASBIT(e->player_avail, _local_player)) {
 					if (sel==0) selected_id = engine_id;
 					if (IS_INT_INSIDE(--pos, -8, 0)) {
-						DrawString(x+59, y+2, _engine_name_strings[engine_id], sel==0 ? 0xC : 0x10);
+						DrawString(x+59, y+2, GetCustomEngineName(engine_id), sel==0 ? 0xC : 0x10);
 						DrawTrainEngine(x+29, y+6, engine_id, SPRITE_PALETTE(PLAYER_SPRITE_COLOR(_local_player)));
 						y += 14;
 					}
@@ -131,7 +131,7 @@ static void NewRailVehicleWndProc(Window *w, WindowEvent *e)
 					SET_DPARAM16(2, rvi->max_speed * 10 >> 4);
 					SET_DPARAM16(3, rvi->power << multihead);
 					SET_DPARAM16(1, rvi->weight << multihead);
-					SET_DPARAM32(4, (rvi->running_cost_base * _price.running_rail[rvi->running_cost_class-1] >> 8) << multihead);
+					SET_DPARAM32(4, (rvi->running_cost_base * _price.running_rail[rvi->engclass] >> 8) << multihead);
 					
 					SET_DPARAM16(5, STR_8838_N_A);
 					if (rvi->capacity != 0) {
@@ -181,7 +181,7 @@ static void NewRailVehicleWndProc(Window *w, WindowEvent *e)
 		case 6:
 			WP(w,buildtrain_d).rename_engine = WP(w,buildtrain_d).sel_engine;
 			ShowQueryString(
-				_engine_name_strings[WP(w,buildtrain_d).sel_engine],
+				GetCustomEngineName(WP(w,buildtrain_d).sel_engine),
 				STR_886A_RENAME_TRAIN_VEHICLE_TYPE,
 				31,
 				160,
@@ -712,6 +712,7 @@ static Widget _train_view_widgets[] = {
 { WWT_PUSHIMGBTN,    14,   232,   249,    68,    85, 0x2CB, STR_884B_REVERSE_DIRECTION_OF_TRAIN},
 { WWT_PUSHIMGBTN,    14,   232,   249,    86,   103, 0x2B2, STR_8847_SHOW_TRAIN_S_ORDERS},
 { WWT_PUSHIMGBTN,    14,   232,   249,   104,   121, 0x2B3, STR_884C_SHOW_TRAIN_DETAILS},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    68,    85, 0x2B4, STR_RAIL_REFIT_VEHICLE_TO_CARRY},
 {      WWT_LAST},
 };
 
@@ -725,16 +726,8 @@ static void TrainViewWndProc(Window *w, WindowEvent *e)
 		v = &_vehicles[w->window_number];	
 	
 		w->disabled_state = (v->owner == _local_player) ? 0 : 0x1C0;
-
-		// adjust button to reflect either refit or reverse direction
-		if (CheckStoppedInDepot(v) >= 0) {
-			_train_view_widgets[8].unkA = 0x2B4;
-			_train_view_widgets[8].tooltips = STR_RAIL_REFIT_VEHICLE_TO_CARRY;
-		} else {
-			_train_view_widgets[8].unkA = 0x2CB;
-			_train_view_widgets[8].tooltips = STR_884B_REVERSE_DIRECTION_OF_TRAIN;
-		}
-	
+		
+		
 		/* draw widgets & caption */
 		SET_DPARAM16(0, v->string_id);
 		SET_DPARAM16(1, v->unitnumber);
@@ -813,11 +806,7 @@ static void TrainViewWndProc(Window *w, WindowEvent *e)
 			DoCommandP(v->tile, v->index, 0, NULL, CMD_FORCE_TRAIN_PROCEED | CMD_MSG(STR_8862_CAN_T_MAKE_TRAIN_PASS_SIGNAL)); 
 			break;
 		case 8: /* reverse direction */
-			if (CheckStoppedInDepot(v) >= 0) {
-				ShowRailVehicleRefitWindow(v);
-			} else {
-				DoCommandP(v->tile, v->index, 0, NULL, CMD_REVERSE_TRAIN_DIRECTION | CMD_MSG(STR_8869_CAN_T_REVERSE_DIRECTION)); 
-			}
+			DoCommandP(v->tile, v->index, 0, NULL, CMD_REVERSE_TRAIN_DIRECTION | CMD_MSG(STR_8869_CAN_T_REVERSE_DIRECTION)); 
 			break;
 		case 9: /* show train orders */
 			ShowOrdersWindow(v);
@@ -825,6 +814,9 @@ static void TrainViewWndProc(Window *w, WindowEvent *e)
 		case 10: /* show train details */
 			ShowTrainDetailsWindow(v);
 			break;
+		case 11:
+			ShowRailVehicleRefitWindow(v);
+			break;
 		}
 	} break;
 
@@ -832,6 +824,21 @@ static void TrainViewWndProc(Window *w, WindowEvent *e)
 		DeleteWindowById(WC_VEHICLE_ORDERS, w->window_number);
 		DeleteWindowById(WC_VEHICLE_DETAILS, w->window_number);
 		break;
+
+	case WE_MOUSELOOP: {
+		Vehicle *v;
+		uint32 h;
+
+		v = &_vehicles[w->window_number];		
+		assert(v->type == VEH_Train);
+		h = CheckStoppedInDepot(v) >= 0 ? (1 << 8) : (1 << 11);
+		if (h != w->hidden_state) {
+			w->hidden_state = h;
+			SetWindowDirty(w);
+		}
+		break;
+	}
+
 	}
 }
 
@@ -879,12 +886,12 @@ static void TrainDetailsInfoTab(Vehicle *v, int x, int y)
 	rvi = &_rail_vehicle_info[v->engine_type];
 
 	if (!(rvi->flags & RVI_WAGON)) {
-		SET_DPARAM16(0, _engine_name_strings[v->engine_type]);
+		SET_DPARAM16(0, GetCustomEngineName(v->engine_type));
 		SET_DPARAM16(1, v->build_year + 1920);
 		SET_DPARAM32(2, v->value);
 		DrawString(x, y, STR_882C_BUILT_VALUE, 0x10);
 	} else {
-		SET_DPARAM16(0, _engine_name_strings[v->engine_type]);
+		SET_DPARAM16(0, GetCustomEngineName(v->engine_type));
 		SET_DPARAM32(1, v->value);
 		DrawString(x, y, STR_882D_VALUE, 0x10);
 	}	
@@ -910,17 +917,42 @@ static TrainDetailsDrawerProc * const _train_details_drawer_proc[3] = {
 static void DrawTrainDetailsWindow(Window *w)
 {
 	Vehicle *v, *u;
-	int num,x,y,sel;
+	uint16 tot_cargo[NUM_CARGO][2];	// count total cargo ([0]-actual cargo, [1]-total cargo)
+	int i,num,x,y,sel;
 	StringID str;
+	byte det_tab = WP(w, traindetails_d).tab;
 
 	/* Count number of vehicles */
 	num = 0;
+
+	// det_tab == 3 <-- Total Cargo tab
+	if (det_tab == 3)	// reset tot_cargo array to 0 values
+		memset(tot_cargo, 0, sizeof(tot_cargo));
 	
 	u = v = &_vehicles[w->window_number];
-	do num++; while ( (u=u->next) != NULL);
+	do {
+		if (det_tab != 3)
+			num++;
+		else {
+			tot_cargo[u->cargo_type][0] += u->cargo_count;
+			tot_cargo[u->cargo_type][1] += u->cargo_cap;
+		}
+	} while ( (u = u->next) != NULL);
+
+	/*	set scroll-amount seperately from counting, as to not
+			compute num double for more carriages of the same type
+	*/
+	if (det_tab == 3) {
+		for (i = 0; i != NUM_CARGO; i++) {
+			if (tot_cargo[i][1] > 0)	// only count carriages that the train has
+				num++;
+		}
+		num++;	// needs one more because first line is description string
+	}
+
 	SetVScrollCount(w, num);
 
-	w->disabled_state = 1 << (WP(w,traindetails_d).tab + 9);
+	w->disabled_state = 1 << (det_tab + 9);
 	if (v->owner != _local_player)
 		w->disabled_state |= (1 << 2);
 
@@ -965,14 +997,32 @@ static void DrawTrainDetailsWindow(Window *w)
 	y = 57;
 	sel = w->vscroll.pos;
 	
-	for(;;) {
-		if (--sel < 0 && sel >= -6) {
-			DrawTrainImage(v, x, y, 1, 0, INVALID_VEHICLE);
-			_train_details_drawer_proc[WP(w,traindetails_d).tab](v, x + 30, y + 2);
-			y += 14;
+	// draw the first 3 details tabs
+	if (det_tab != 3) {
+		for(;;) {
+			if (--sel < 0 && sel >= -6) {
+				DrawTrainImage(v, x, y, 1, 0, INVALID_VEHICLE);
+				_train_details_drawer_proc[WP(w,traindetails_d).tab](v, x + 30, y + 2);
+				y += 14;
+			}
+			if ( (v=v->next) == NULL)
+				return;
 		}
-		if ( (v=v->next) == NULL)
-			return;
+	}
+	else {	// draw total cargo tab
+		i = 0;
+		DrawString(x, y + 2, STR_013F_TOTAL_CAPACITY_TEXT, 0);
+		do {
+			if (tot_cargo[i][1] > 0 && --sel < 0 && sel >= -5) {
+				y += 14;
+				// STR_013F_TOTAL_CAPACITY			:{LTBLUE}- {CARGO} ({SHORTCARGO})
+				SET_DPARAM8(0, i);								// {CARGO} #1
+				SET_DPARAM16(1, tot_cargo[i][0]);	// {CARGO} #2
+				SET_DPARAM8(2, i);								// {SHORTCARGO} #1
+				SET_DPARAM16(3, tot_cargo[i][1]);	// {SHORTCARGO} #2
+				DrawString(x, y, STR_013F_TOTAL_CAPACITY, 0);
+			}
+		} while (++i != NUM_CARGO);
 	}
 }
 
@@ -1005,12 +1055,15 @@ do_change_service_int:
 			
 			DoCommandP(v->tile, v->index, mod, NULL, CMD_CHANGE_TRAIN_SERVICE_INT | CMD_MSG(STR_018A_CAN_T_CHANGE_SERVICING));
 			break;
-		case 9:	/* details buttons*/
-		case 10:
-		case 11:
+		/* details buttons*/
+		case 9:		// Cargo
+		case 10:	// Information
+		case 11:	// Capacities
+		case 12:	// Total cargo
 			CLRBIT(w->disabled_state, 9);
 			CLRBIT(w->disabled_state, 10);
 			CLRBIT(w->disabled_state, 11);
+			CLRBIT(w->disabled_state, 12);
 			SETBIT(w->disabled_state, e->click.widget);
 			WP(w,traindetails_d).tab = e->click.widget - 9;
 			SetWindowDirty(w);
@@ -1043,9 +1096,10 @@ static const Widget _train_details_widgets[] = {
 { WWT_PUSHTXTBTN,    14,     0,    10,   140,   145, STR_0188, STR_884D_INCREASE_SERVICING_INTERVAL},
 { WWT_PUSHTXTBTN,    14,     0,    10,   146,   151, STR_0189, STR_884E_DECREASE_SERVICING_INTERVAL},
 {      WWT_PANEL,    14,    11,   369,   140,   151, 0x0, 0},
-{ WWT_PUSHTXTBTN,    14,     0,   122,   152,   163, STR_013C_CARGO, STR_884F_SHOW_DETAILS_OF_CARGO_CARRIED},
-{ WWT_PUSHTXTBTN,    14,   123,   245,   152,   163, STR_013D_INFORMATION, STR_8850_SHOW_DETAILS_OF_TRAIN_VEHICLES},
-{ WWT_PUSHTXTBTN,    14,   246,   369,   152,   163, STR_013E_CAPACITIES, STR_8851_SHOW_CAPACITIES_OF_EACH},
+{ WWT_PUSHTXTBTN,    14,     0,    92,   152,   163, STR_013C_CARGO, STR_884F_SHOW_DETAILS_OF_CARGO_CARRIED},
+{ WWT_PUSHTXTBTN,    14,    93,   184,   152,   163, STR_013D_INFORMATION, STR_8850_SHOW_DETAILS_OF_TRAIN_VEHICLES},
+{ WWT_PUSHTXTBTN,    14,   185,   277,   152,   163, STR_013E_CAPACITIES, STR_8851_SHOW_CAPACITIES_OF_EACH},
+{ WWT_PUSHTXTBTN,    14,   278,   369,   152,   163, STR_013E_TOTAL_CARGO, STR_8852_SHOW_TOTAL_CARGO},
 {      WWT_LAST},
 };
 
