@@ -18,16 +18,10 @@
 #include "economy.h"
 #include "fileio.h"
 #include "hal.h"
+#include "airport.h"
 
 #include <stdarg.h>
 
-#ifdef __MORPHOS__
-#include <exec/types.h>
-ULONG __stack = (1024*1024)*4;
-#warning release note: update versions tag
-const STRPTR morphos_versions_tag = "\0$VER: OpenTTD 0.3 (23.05.04) 2004 OpenTTD Team [build: 0.3.2-svn, PowerPC, MorphOS]";
-#endif __MORPHOS__
-
 void GameLoop();
 
 void IncreaseSpriteLRU();
@@ -38,6 +32,7 @@ void IncreaseDate();
 void RunOtherPlayersLoop();
 void DoPaletteAnimations();
 void MusicLoop();
+void ResetMusic();
 void InitializeStations();
 void DeleteAllPlayerStations();
 
@@ -91,6 +86,23 @@ void CDECL ShowInfoF(const char *str, ...)
 	ShowInfo(buf);
 }
 
+char * CDECL str_fmt(const char *str, ...)
+{
+	char buf[4096];
+	va_list va;
+	int len;
+	char *p;
+
+	va_start(va, str);
+	len = vsprintf(buf, str, va);
+	va_end(va);
+	p = malloc(len + 1);
+	if (p)
+		memcpy(p, buf, len + 1);
+	return p;
+}
+
+
 // NULL midi driver
 static char *NullMidiStart(char **parm) { return NULL; }
 static void NullMidiStop() {}
@@ -459,18 +471,18 @@ int ttd_main(int argc, char* argv[])
 	MyGetOptData mgo;
 	int i;
 	int network = 0;
-	char *network_conn;
-	char *language = 0;
+	char *network_conn = NULL;
+	char *language = NULL;
 	char musicdriver[32], sounddriver[32], videodriver[32];
 	int resolution[2] = {0,0};
 	uint startdate = -1;
-
+	_ignore_wrong_grf = false;
 	musicdriver[0] = sounddriver[0] = videodriver[0] = 0;
 
 	_game_mode = GM_MENU;
 	_switch_mode = SM_MENU;
 
-	MyGetOptInit(&mgo, argc-1, argv+1, "m:s:v:hn::l:et:d::r:g::G:cp:");
+	MyGetOptInit(&mgo, argc-1, argv+1, "m:s:v:hn::l:eit:d::r:g::G:cp:");
 	while ((i = MyGetOpt(&mgo)) != -1) {
 		switch(i) {
 		case 'm': ttd_strlcpy(musicdriver, mgo.opt, sizeof(musicdriver)); break;
@@ -492,6 +504,7 @@ int ttd_main(int argc, char* argv[])
 					SetDebugString(mgo.opt);
 			} break;
 		case 'e': _switch_mode = SM_EDITOR; break;
+		case 'i': _ignore_wrong_grf = true; break;
 		case 'g': 
 			if (mgo.opt) {
 				strcpy(_file_to_saveload, mgo.opt);
@@ -514,6 +527,7 @@ int ttd_main(int argc, char* argv[])
 		}
 	}
 
+	DeterminePaths();
 	LoadFromConfig();
 
 	// override config?
@@ -528,6 +542,9 @@ int ttd_main(int argc, char* argv[])
 
 	// initialize screenshot formats
 	InitializeScreenshotFormats();
+
+  // initialize airport state machines
+  InitializeAirports();
 	
 	// Sample catalogue
 	DEBUG(misc, 1) ("Loading sound effects...");
@@ -568,12 +585,19 @@ int ttd_main(int argc, char* argv[])
 
 	while (_video_driver->main_loop() == ML_SWITCHDRIVER) {}
 
+	if (network) {
+		NetworkShutdown();
+	}
+
 	_video_driver->stop();
 	_music_driver->stop();
 	_sound_driver->stop();
 
 	SaveToConfig();
 
+  // uninitialize airport state machines
+  UnInitializeAirports();
+
 	return 0;
 }
 
@@ -634,12 +658,42 @@ void MakeNewEditorWorld()
 	// Startup the game system
 	GenerateWorld(1);
 
-	_local_player = 0x10;
+	_local_player = OWNER_NONE;
 	MarkWholeScreenDirty();
 }
 
+void StartScenario()
+{
+	_game_mode = GM_NORMAL;
+
+	// Copy in game options
+	_opt_mod_ptr = &_opt;
+	memcpy(&_opt, &_new_opt, sizeof(_opt));
+
+	GfxLoadSprites();
+
+	// Reinitialize windows
+	InitWindowSystem();
+	LoadStringWidthTable();
+
+	SetupColorsAndInitialWindow();
+
+	// Load game
+	if (SaveOrLoad(_file_to_saveload, 0) != SL_OK)
+		GenerateWorld(0);
+
+	// Create a single player
+	DoStartupNewPlayer(false);
+	
+	_local_player = 0;
+
+	MarkWholeScreenDirty();
+}
+
+
 void LoadIntroGame()
 {
+	char filename[256];
 	_game_mode = GM_MENU;
 	_display_opt |= DO_TRANS_BUILDINGS; // don't make buildings transparent in intro
 
@@ -651,13 +705,18 @@ void LoadIntroGame()
 	SetupColorsAndInitialWindow();
 
 	// Generate a world.
-	if (SaveOrLoad(OTTD_DATADIR "opntitle.dat", 0) != SL_OK)
+	sprintf(filename, "%sopntitle.dat",  _path.data_dir);
+	if (SaveOrLoad(filename, 0) != SL_OK)
 		GenerateWorld(1); // if failed loading, make empty world.
 	
 	_opt.currency = _new_opt.currency;
 
+	_pause = 0;
 	_local_player = 0;
 	MarkWholeScreenDirty();
+
+	// Play main theme
+	if (_music_driver->is_song_playing()) ResetMusic();
 }
 
 bool SafeSaveOrLoad(const char *filename, int mode, int newgm)
@@ -708,6 +767,7 @@ normal_load:
 		} else {
 			_opt_mod_ptr = &_opt;
 			_local_player = 0;
+			DoCommandP(0, 0, 0, NULL, CMD_PAUSE); // decrease pause counter (was increased from opening load dialog)
 		}
 		break;
 	}
@@ -722,12 +782,12 @@ normal_load:
 		if (SafeSaveOrLoad(_file_to_saveload, s && strcmp(s, ".sv1") == 0 ? 2 : 0, GM_EDITOR)) {
 			_opt_mod_ptr = &_opt;
 
-			_local_player = 0x10;
+			_local_player = OWNER_NONE;
 			_generating_world = true;
 			// delete all players.
-			for(i=0; i!=8; i++) {
+			for(i=0; i != MAX_PLAYERS; i++) {
 				ChangeOwnershipOfPlayerItems(i, 0xff);
-				_players[i].name_1 = 0;
+				_players[i].is_active = false;
 			}
 			_generating_world = false;
 			// delete all stations owned by a player
@@ -753,7 +813,7 @@ normal_load:
 	case SM_GENRANDLAND:
 		GenerateWorld(2);
 		// XXX: set date
-		_local_player = 0x10;
+		_local_player = OWNER_NONE;
 		MarkWholeScreenDirty();
 		break;
 	}
@@ -811,17 +871,17 @@ static void DoAutosave()
 	if (_patches.keep_all_autosave && _local_player != 255) {
 		Player *p;
 		char *s;
-		strcpy(buf, "autosave" PATHSEP);
+		strcpy(buf, _path.autosave_dir);
 		p = DEREF_PLAYER(_local_player);
 		SET_DPARAM16(0, p->name_1);
 		SET_DPARAM32(1, p->name_2);
 		SET_DPARAM16(2, _date);
-		s= (char*)GetString(buf + sizeof("autosave" PATHSEP) - 1, STR_4004);
+		s= (char*)GetString(buf + strlen(_path.autosave_dir) - 1, STR_4004);
 		strcpy(s, ".sav");
 	} else {
 		int n = _autosave_ctr;
 		_autosave_ctr = (_autosave_ctr + 1) & 15;
-		sprintf(buf, "autosave" PATHSEP "autosave%d.sav", n);
+		sprintf(buf, "%sautosave%d.sav", _path.autosave_dir, n);
 	}
 
 	if (SaveOrLoad(buf, 1) != SL_OK)
@@ -848,7 +908,7 @@ void GameLoop()
 			ShowScreenshotResult(MakeScreenshot());
 			break;
 		case 2: // make large screenshot
-			ShowScreenshotResult(MakeWorldScreenshot(-(TILES_X-1)*32, 0, TILES_X*32 + (TILES_X-1)*32, TILES_Y * 32, 0));
+			ShowScreenshotResult(MakeWorldScreenshot(-(TILE_X_MAX)*32, 0, TILE_X_MAX*32 + (TILE_X_MAX)*32, TILES_Y * 32, 0));
 			break;
 		}
 	}
@@ -890,14 +950,14 @@ void GameLoop()
 			NetworkProcessCommands();
 		}
 	} else {
-		if (!_game_paused)
+		if (!_pause)
 			StateGameLoop();
 	}
 
-	if (!_game_paused && _display_opt&DO_FULL_ANIMATION)
+	if (!_pause && _display_opt&DO_FULL_ANIMATION)
 		DoPaletteAnimations();
 
-	if (!_game_paused || _patches.build_in_pause)
+	if (!_pause || _patches.build_in_pause)
 		MoveAllTextEffects();
 
 	MouseLoop();
@@ -921,24 +981,35 @@ void BeforeSaveGame()
 
 void ConvertCityOwner()
 {
-	{
-		uint tile;
+	uint tile;
 
-		for(tile=0; tile!=TILES_X * TILES_Y; tile++) {
-			if (IS_TILETYPE(tile, MP_STREET)) {
-				if ((_map5[tile] & 0xF0) == 0x10 && _map3_lo[tile] & 0x80)
-					_map3_lo[tile] = OWNER_TOWN;
+	for(tile=0; tile!=TILES_X * TILES_Y; tile++) {
+		if (IS_TILETYPE(tile, MP_STREET)) {
+			if ((_map5[tile] & 0xF0) == 0x10 && _map3_lo[tile] & 0x80)
+				_map3_lo[tile] = OWNER_TOWN;
 
-				if (_map_owner[tile] & 0x80)
-					_map_owner[tile] = OWNER_TOWN;
-			} else if (IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
-				if (_map_owner[tile] & 0x80)
-					_map_owner[tile] = OWNER_TOWN;
-			}
+			if (_map_owner[tile] & 0x80)
+				_map_owner[tile] = OWNER_TOWN;
+		} else if (IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
+			if (_map_owner[tile] & 0x80)
+				_map_owner[tile] = OWNER_TOWN;
 		}
 	}
 }
 
+// before savegame version 4, the name of the company determined if it existed
+void CheckIsPlayerActive()
+{
+	Player *p;
+	FOR_ALL_PLAYERS(p) {
+		if (p->name_1 != 0) {
+			p->is_active = true;
+		}
+	}
+}
+
+extern void UpdateOldAircraft();
+
 bool AfterLoadGame(uint version)
 {
 	Window *w;
@@ -963,6 +1034,11 @@ bool AfterLoadGame(uint version)
 	
 	// Update all vehicles
 	AfterLoadVehicles();
+	// in version 2.2 of the savegame, we have new airports
+	if (version <= 0x201) {
+		UpdateOldAircraft();
+	}
+
 	UpdateAllStationVirtCoord();
 
 	// Setup city coords
@@ -987,8 +1063,15 @@ bool AfterLoadGame(uint version)
 	vp->virtual_width = vp->width << vp->zoom;
 	vp->virtual_height = vp->height << vp->zoom;
 
+
+	// in version 4.0 of the savegame, is_active was introduced to determine
+	// if a player does exist, rather then checking name_1
+	if (version <= 0x400) {
+		CheckIsPlayerActive();
+	}
+
 	// in case the player doesn't exist, create one (for scenario)
-	if (_players[0].name_1 == 0)
+	if (!_players[0].is_active)
 		DoStartupNewPlayer(false);
 
 	DoZoomInOut(ZOOM_NONE); // update button status
@@ -1012,16 +1095,3 @@ void DebugProc(int i)
 		break;
 	}
 }
-
-void PauseGame()
-{
-	_game_paused++;
-	InvalidateWindow(WC_STATUS_BAR, 0);
-}
-
-void UnpauseGame()
-{
-	_game_paused--;
-	InvalidateWindow(WC_STATUS_BAR, 0);
-}
-
