@@ -4,51 +4,38 @@
 #include "viewport.h"
 #include "command.h"
 #include "player.h"
+#include "city.h"
 
 extern void DrawCanalWater(uint tile);
 
-static const byte _bridge_available_year[13] = {
+static const byte _bridge_available_year[MAX_BRIDGES] = {
 	0, 0, 10, 0, 10, 10, 10, 10, 10, 10, 75, 85, 90
 };
 
-static const byte _bridge_minlen[13] = {
+static const byte _bridge_minlen[MAX_BRIDGES] = {
 	0, 0, 0, 2, 3, 3, 3, 3, 3, 0, 2, 2, 2
 };
 
-static const byte _bridge_maxlen[13] = {
+static const byte _bridge_maxlen[MAX_BRIDGES] = {
 	16, 2, 5, 10, 16, 16, 7, 8, 9, 2, 16, 32, 32, 
 };
 
-static const uint16 _bridge_type_price_mod[13] = {
+const uint16 _bridge_type_price_mod[MAX_BRIDGES] = {
 	80, 112, 144, 168, 185, 192, 224, 232, 248, 240, 255, 380, 510,
 };
 
-const uint16 _bridge_speeds[13] = {
+const uint16 _bridge_speeds[MAX_BRIDGES] = {
 	0x20, 0x30, 0x40, 0x50, 0x60, 0x70, 0x0A0, 0x0D0, 0x0F0, 0x100, 0x140, 0x200, 0x260
 };
 
-const PalSpriteID _bridge_sprites[13] = {
+const PalSpriteID _bridge_sprites[MAX_BRIDGES] = {
 	0x0A24, 0x31E8A26, 0x0A25, 0x3208A22,
 	0x0A22, 0x3218A22, 0x0A23, 0x31C8A23,
 	0x31E8A23, 0x0A27, 0x0A28, 0x3218A28, 
 	0x3238A28, 
 };
 
-// calculate the price factor for building a long bridge.
-// basically the cost delta is 1,1, 1, 2,2, 3,3,3, 4,4,4,4, 5,5,5,5,5, 6,6,6,6,6,6,  7,7,7,7,7,7,7,  8,8,8,8,8,8,8,8,
-static int CalcBridgeLenCostFactor(int x)
-{
-	int n,r;
-	if (x < 2) return x;
-	x -= 2;
-	for(n=0,r=2;;n++) {
-		if (x <= n) return r + x * n;
-		r += n * n;
-		x -= n;
-	}
-}
-
-const StringID _bridge_material[13] = {
+const StringID _bridge_material[MAX_BRIDGES] = {
 	STR_5012_WOODEN,
 	STR_5013_CONCRETE,
 	STR_500F_GIRDER_STEEL,
@@ -64,6 +51,89 @@ const StringID _bridge_material[13] = {
 	STR_BRIDGE_TUBULAR_SILICON,
 };
 
+// calculate the price factor for building a long bridge.
+// basically the cost delta is 1,1, 1, 2,2, 3,3,3, 4,4,4,4, 5,5,5,5,5, 6,6,6,6,6,6,  7,7,7,7,7,7,7,  8,8,8,8,8,8,8,8,
+int CalcBridgeLenCostFactor(int x)
+{
+	int n,r;
+	if (x < 2) return x;
+	x -= 2;
+	for(n=0,r=2;;n++) {
+		if (x <= n) return r + x * n;
+		r += n * n;
+		x -= n;
+	}
+}
+
+enum {
+	// foundation, whole tile is leveled up (tileh's 7, 11, 13, 14)
+	BRIDGE_FULL_LEVELED_FOUNDATION = 1 << 7 | 1 << 11 | 1 << 13 | 1 << 14,
+	// foundation, tile is partly leveled up (tileh's 1, 2, 4, 8)
+	BRIDGE_PARTLY_LEVELED_FOUNDATION = 1 << 1 | 1 << 2 | 1 << 4 | 1 << 8,
+	// no foundations (X,Y direction) (tileh's 0, 3, 6, 9, 12)
+	BRIDGE_NO_FOUNDATION = 1 << 0 | 1 << 3 | 1 << 6 | 1 << 9 | 1 << 12
+};
+
+/*	check if bridge can be built on slope
+		start-end_tile = false	<-- end tile
+		start-end_tile = true		<-- start tile
+*/
+static uint32 CheckBridgeSlope(uint direction, uint tileh, bool start_end_tile)
+{
+	if (!(tileh & 0x10)) {	// disable building on very steep slopes
+		// slope foundations
+		if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << tileh) || BRIDGE_PARTLY_LEVELED_FOUNDATION & (1 << tileh))
+			return _price.terraform;
+
+		if (start_end_tile) {
+			/* check slope at start tile
+					- no extra cost
+					- direction X: tiles 0, 9
+					- direction Y: tiles 0,12
+			*/		
+			if ((direction?0x201:0x1001) & (1 << tileh))
+				return 0;
+		}
+		else {
+			/*	check slope at end tile
+					- no extra cost
+					- direction X: tiles 0, 6
+					- direction Y: tiles 0, 3
+			*/
+			if ((direction?0x41:0x9) & (1 << tileh))
+				return 0;
+		}
+	}
+	return CMD_ERROR;
+}
+
+uint32 GetBridgeLength(TileIndex begin, TileIndex end)
+{
+	int x1, y1, x2, y2;	// coordinates of starting and end tiles
+	x1 = GET_TILE_X(begin);
+	y1 = GET_TILE_Y(begin);
+	x2 = GET_TILE_X(end);
+	y2 = GET_TILE_Y(end);
+	
+	return abs((x2 + y2 - x1 - y1)) - 1;
+}
+
+bool CheckBridge_Stuff(byte bridge_type, int bridge_len)
+{
+	int max;	// max possible length of a bridge (with patch 100)
+
+	if (_bridge_available_year[bridge_type] > _cur_year)
+				return false;
+
+	max = _bridge_maxlen[bridge_type];
+	if (max >= 16 && _patches.longbridges)
+		max = 100;
+
+	if (bridge_len < _bridge_minlen[bridge_type] || bridge_len > max)
+		return false;
+
+	return true;
+}
 
 /* Build a Bridge 
  * x,y - end tile coord
@@ -80,7 +150,7 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	int bridge_len, bridge_odd_len;
 	uint direction;
 	int i;
-	int32 cost, ret;
+	int32 cost, terraformcost, ret;
 
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 	
@@ -126,40 +196,62 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			ti_start.type == MP_WATER) && ti_start.map5 == 0))
 		return_cmd_error(STR_02A0_ENDS_OF_BRIDGE_MUST_BOTH);
 
+	if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << ti_start.tileh)) {
+		ti_start.z += 8;
+		ti_start.tileh = 0;
+	}
+
+	if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << ti_end.tileh)) {
+		ti_end.z += 8;
+		ti_end.tileh = 0;
+	}
+
 	if (ti_start.z != ti_end.z)
 		return_cmd_error(STR_5009_LEVEL_LAND_OR_WATER_REQUIRED);
 
 
 	_error_message = STR_500C;
 
-	/* check various params */
-	{
-		int max;
+	/* try and clear the start landscape */
+	if ((ret=DoCommandByTile(ti_start.tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR)) == CMD_ERROR)
+		return CMD_ERROR;
+	cost = ret;
 
-		if (_bridge_available_year[bridge_type] > _cur_year)
-			return CMD_ERROR;
+	terraformcost = CheckBridgeSlope(direction, ti_start.tileh, true);	// true - bridge-start-tile, false - bridge-end-tile
 
-		bridge_len = ((sx + sy - x - y) >> 4) - 1;
+	// the AI and towns are not allowed to use bridges on slopes.
+	if (terraformcost == CMD_ERROR || 
+		 (terraformcost && (_is_ai_player || _current_player == OWNER_TOWN || !_patches.build_on_slopes)))
+		return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
 
-		max = _bridge_maxlen[bridge_type];
-		if (max >= 16 && _patches.longbridges) max = 100;
-		if (bridge_len < _bridge_minlen[bridge_type] || bridge_len > max)
-			return CMD_ERROR;
-
-		bridge_odd_len = bridge_len&1 ? (bridge_len>>1) : -1;
-	}
+	cost += terraformcost;
 
 	/* try and clear the end landscape */
 	if ((ret=DoCommandByTile(ti_end.tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR)) == CMD_ERROR)
 		return CMD_ERROR;
-	cost = ret;
+	cost += ret;
 	
-	/* check slope at end tile */
-	if (!((direction?0x41:0x9) & (1 << ti_end.tileh)))
+	terraformcost = CheckBridgeSlope(direction, ti_end.tileh, false);	// false - end tile slope check
+
+	// the AI and towns are not allowed to use bridges on slopes.
+	if (terraformcost == CMD_ERROR || 
+		 (terraformcost && (_is_ai_player || _current_player == OWNER_TOWN || !_patches.build_on_slopes)))
 		return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
 
+	cost += terraformcost;
+
 	/* do the drill? */
 	if (flags & DC_EXEC) {
+		/* build the start tile */
+		ModifyTile(ti_start.tile,
+			MP_SETTYPE(MP_TUNNELBRIDGE) |
+			MP_MAP2 | MP_MAP3LO | MP_MAPOWNER_CURRENT | MP_MAP5,
+			(bridge_type << 4), /* map2 */
+			railtype, /* map3_lo */
+			0x80 | direction | rail_or_road /* map5 */
+		);
+
+		/* build the end tile */
 		ModifyTile(ti_end.tile,
 			MP_SETTYPE(MP_TUNNELBRIDGE) |
 			MP_MAP2 | MP_MAP3LO | MP_MAPOWNER_CURRENT | MP_MAP5,
@@ -168,32 +260,16 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			0x80 | 0x20 | direction | rail_or_road /* map5 */
 		);
 	}
-	
-	/* try and clear the start landscape */
-	if ((ret=DoCommandByTile(ti_start.tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR)) == CMD_ERROR)
-		return CMD_ERROR;
-	cost += ret;
 
-	/* check slope at start tile */
-	if (!((direction?0x201:0x1001) & (1 << ti_start.tileh)))
-		return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
-
-	/* build a piece of start tile? */
-	if (flags & DC_EXEC) {
-		ModifyTile(ti_start.tile,
-			MP_SETTYPE(MP_TUNNELBRIDGE) |
-			MP_MAP2 | MP_MAP3LO | MP_MAPOWNER_CURRENT | MP_MAP5,
-			(bridge_type << 4), /* map2 */
-			railtype, /* map3_lo */
-			0x80 | direction | rail_or_road /* map5 */
-		);
-	}
+	/* set various params */
+	bridge_len = ((sx + sy - x - y) >> 4) - 1;
+	bridge_odd_len = (bridge_len & 1) ? (bridge_len>>1) : -1;
 
 	lookflag = 0;
-	if ((bridge_len&3) == 3)
+	if ((bridge_len & 3) == 3)
 		lookflag |= 2;
 
-	for(i=0; i!=bridge_len; i++) {
+	for(i = 0; i != bridge_len; i++) {
 		if (direction != 0)
 			y+=16;
 		else
@@ -231,8 +307,7 @@ not_valid_below:;
 			m5 = 0xC0;
 		}
 
-		/* doit */
-
+		/* do middle part of bridge */
 		if (flags & DC_EXEC) {
 			_map5[ti.tile] = (byte)(m5 | direction | rail_or_road);
 			_map_type_and_height[ti.tile] &= ~0xF0;
@@ -268,11 +343,18 @@ not_valid_below:;
 
 	SetSignalsOnBothDir(ti_start.tile, (direction&1) ? 1 : 0);
 
-	bridge_len += 2;		
+	/*	for human player that builds the bridge he gets a selection to choose from bridges (DC_QUERY_COST)
+			It's unnecessary to execute this command every time for every bridge. So it is done only
+			and cost is computed in "bridge_gui.c". For AI, Towns this has to be of course calculated
+	*/
+	if (!(flags & DC_QUERY_COST)) {
+		bridge_len += 2;	// begin and end tiles/ramps
 
-	if (IS_HUMAN_PLAYER(_current_player)) { bridge_len = CalcBridgeLenCostFactor(bridge_len); }
+		if (_current_player < MAX_PLAYERS && IS_HUMAN_PLAYER(_current_player))
+			bridge_len = CalcBridgeLenCostFactor(bridge_len);
 
-	cost += ((bridge_len * _price.build_bridge) * _bridge_type_price_mod[bridge_type]) >> 8;
+		cost += ((bridge_len * _price.build_bridge) * _bridge_type_price_mod[bridge_type]) >> 8;
+	}
 
 	return cost;
 }
@@ -316,7 +398,7 @@ static int32 DoBuildTunnel(int x, int y, int x2, int y2, uint32 flags, uint exc_
 	TileInfo ti;
 	uint z;
 
-	if ( (uint) x > 0xFEF ||  (uint) y > 0xFEF)
+	if ( (uint) x > TILE_X_MAX * 16 - 1 ||  (uint) y > TILE_X_MAX * 16 - 1)
 		return CMD_ERROR;
 
 	/* check if valid, and make sure that (x,y) is smaller than (x2,y2) */
@@ -365,7 +447,7 @@ static int32 DoBuildTunnel(int x, int y, int x2, int y2, uint32 flags, uint exc_
 		if (ti.z <= z)
 			return_cmd_error(STR_5002);
 
-		if (!_patches.crossing_tunnels && !CheckTunnelInWay(ti.tile, z))
+		if (!_cheats.crossing_tunnels.value && !CheckTunnelInWay(ti.tile, z))
 			return CMD_ERROR;
 
 		cost += _price.build_tunnel;
@@ -492,11 +574,13 @@ static int32 DoClearTunnel(uint tile, uint32 flags)
 {
 	uint endtile;
 	int length;
-
-	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
+	City *c = ClosestCityFromTile(tile, (uint)-1); //needed for city rating penalty
 	
-	if (_game_mode != GM_EDITOR) {
-		if (!CheckTileOwnership(tile))
+	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
+
+	// in scenario editor you can always destroy tunnels
+	if (_game_mode != GM_EDITOR && !CheckTileOwnership(tile)) {
+		if (!_patches.extra_dynamite || _map_owner[tile] != OWNER_TOWN)
 			return CMD_ERROR;
 	}
 
@@ -504,12 +588,22 @@ static int32 DoClearTunnel(uint tile, uint32 flags)
 	if (endtile == (uint)-1) return CMD_ERROR;
 
 	_build_tunnel_endtile = endtile;
+	// check if you're allowed to remove the tunnel owned by a town
+	// no removal if rating is lower than "Good" (250) <- this should depend on difficulty settings
+	if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR ) {
+		if (c->ratings[_current_player] < 250 && !(flags & DC_NO_CITY_RATING) && !_cheats.magic_bulldozer.value) {
+			SET_DPARAM16(0, c->index);
+			return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
+		}
+	}
 
 	if (flags & DC_EXEC) {
 		DoClearSquare(tile);
 		DoClearSquare(endtile);
 		UpdateSignalsOnSegment(tile, _updsignals_tunnel_dir[_map5[tile]&3]);
 		UpdateSignalsOnSegment(endtile, _updsignals_tunnel_dir[_map5[endtile]&3]);
+		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) 
+			ChangeCityRating(c, -250, 0);
 	}
 	return  _price.clear_tunnel * (length + 1);
 }
@@ -545,7 +639,9 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 	uint endtile;
 	Vehicle *v;
 	int direction;
-
+	
+	City *c = ClosestCityFromTile(tile, (uint)-1); //needed for city rating penalty
+	
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	direction = _map5[tile]&1;
@@ -555,7 +651,7 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 		int32 cost;
 
 		// check if we own the tile below the bridge..
-		if (_current_player != OWNER_WATER && !CheckTileOwnership(tile))
+		if (_current_player != OWNER_WATER && (!CheckTileOwnership(tile) || !EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)) ))
 			return CMD_ERROR;
 		
 		cost = (_map5[tile] & 8) ? _price.remove_road * 2 : _price.remove_rail;
@@ -570,23 +666,52 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 
 	tile = FindEdgesOfBridge(tile, &endtile);
 
-	if (_current_player != OWNER_WATER && _game_mode != GM_EDITOR && !CheckTileOwnership(tile))
+	// floods, scenario editor can always destroy bridges
+	if (_current_player != OWNER_WATER && _game_mode != GM_EDITOR && !CheckTileOwnership(tile)) {
+		if (!_patches.extra_dynamite || _map_owner[tile] != OWNER_TOWN)
+			return CMD_ERROR;
+	}
+
+	if (!EnsureNoVehicle(tile) || !EnsureNoVehicle(endtile))
 		return CMD_ERROR;
 
-	/* Make sure there's no vehicle on the bridge */
-	if ((v=FindVehicleBetween(tile, endtile, 0xff)) != NULL) {
+	/*	Make sure there's no vehicle on the bridge
+			Omit tile and endtile, since these are already checked, thus solving the problem
+			of bridges over water, or higher bridges, where z is not increased, eg level bridge
+	*/
+	tile		+= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
+	endtile	-= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
+	if ((v=FindVehicleBetween(tile, endtile, GET_TILEHEIGHT(tile) + 8)) != NULL) {
 		VehicleInTheWayErrMsg(v);		
 		return CMD_ERROR;
 	}
 
+	/* Put the tiles back to start/end position */
+	tile		-= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
+	endtile	+= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
+
+	// check if you're allowed to remove the bridge owned by a town. 
+	// no removal if rating is lower than "Good" (250) <- this should depend on difficult settings
+	if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) {
+		if (c->ratings[_current_player] < 250 && !(flags & DC_NO_CITY_RATING) && !_cheats.magic_bulldozer.value) {
+			SET_DPARAM16(0, c->index);
+			return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
+		}
+	}
+
 	if (flags & DC_EXEC) {
 		byte m5;
 		uint t = tile;
 		uint16 new_data;
 
+		//checks if the owner is town then decrease city rating by 250 until
+		// you have a "Poor" (0) city rating
+		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) 
+			ChangeCityRating(c, -250, 0); 
+
 		do {
 			m5 = _map5[t];
-		
+
 			if (m5 & 0x40) {
 				if (m5 & 0x20) {
 					static const uint16 _new_data_table[] = {0x1002, 0x1001, 0x2005, 0x200A, 0, 0, 0, 0};
@@ -602,6 +727,7 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 				_map2[t] = 0;
 
 				MarkTileDirtyByTile(t);
+
 			} else {
 clear_it:;
 				DoClearSquare(t);
@@ -611,6 +737,7 @@ clear_it:;
 
 		SetSignalsOnBothDir(tile, direction);
 		SetSignalsOnBothDir(endtile, direction);
+		
 	}
 
 	return ((((endtile - tile) >> (direction?8:0))&0xFF)+1) * _price.clear_bridge;
@@ -662,7 +789,10 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 		return (length + 1) * (_price.build_rail >> 1);
 	} else if ((_map5[tile] & 0xF8) == 0xE0) {
 		// bridge middle part with rail below
-		if (!CheckTileOwnership(tile) || !EnsureNoVehicle(tile)) return CMD_ERROR;
+		// only check for train under bridge
+		if (!CheckTileOwnership(tile) || !EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)))
+			return CMD_ERROR;
+
 		// tile is already of requested type?
 		if ( (uint)(_map3_lo[tile] & 0xF) == totype) return CMD_ERROR;
 		// change type.
@@ -707,19 +837,28 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 
 
 // fast routine for getting the height of a middle bridge tile. 'tile' MUST be a middle bridge tile.
-static uint GetBridgeHeight(uint tile)
+static uint GetBridgeHeight(const TileInfo *ti)
 {
 	uint delta;
+	TileInfo ti_end;
+	uint tile = ti->tile;
+	uint z_correction = 0;
 
 	// find the end tile of the bridge.
 	delta = (_map5[tile] & 1) ? TILE_XY(0,1) : TILE_XY(1,0);
 	do {
-		assert((_map5[tile] & 0xC0) == 0xC0);
+		assert((_map5[tile] & 0xC0) == 0xC0);	// bridge and middle part
 		tile += delta;
-	} while (_map5[tile] & 0x40);
+	} while (_map5[tile] & 0x40);	// while bridge middle parts
 
-	// return the height there.
-	return (_map_type_and_height[tile] & 0xF) * 8;
+	// if the end of the bridge is on a tileh 7, the z coordinate is 1 tile too low
+	// correct it.
+	FindLandscapeHeightByTile(&ti_end, tile);
+	if (HASBIT(1 << 7, ti_end.tileh))
+		z_correction += 8;
+	
+	// return the height there (the height of the NORTH CORNER)
+	return GET_TILEHEIGHT(tile) + z_correction;
 }
 
 static const byte _bridge_foundations[2][16] = {
@@ -730,7 +869,6 @@ static const byte _bridge_foundations[2][16] = {
 
 extern const byte _track_sloped_sprites[14];
 extern const byte _road_sloped_sprites[14];
-extern const byte _inclined_tileh[];
 
 #include "table/tunnel_land.h"
 
@@ -778,6 +916,20 @@ static void DrawBridgePillars(TileInfo *ti, int x, int y, int z)
 	}
 }
 
+static uint GetBridgeFoundation(uint tileh, byte direction) {
+	int i;
+	// normal level sloped building (7, 11, 13, 14)
+	if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << tileh))
+		return tileh;
+
+	// inclined sloped building
+	if (	((i=0, tileh == 1) || (i+=2, tileh == 2) || (i+=2, tileh == 4) || (i+=2, tileh == 8)) && 
+				( direction == 0 || (i++, direction == 1)) )
+		return i + 15;
+
+	return 0;
+}
+
 static void DrawTile_TunnelBridge(TileInfo *ti)
 {
 	uint32 image;
@@ -815,7 +967,16 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 		if (tmp != 0) tmp++;
 		tmp = (ti->map5&3) + (tmp*2);
 
-		if (!(ti->map5 & 0x40)) {
+		if (!(ti->map5 & 0x40)) {	// bridge ramps
+			
+			if (!(BRIDGE_NO_FOUNDATION & (1 << ti->tileh))) {	// no foundations for 0, 3, 6, 9, 12
+				int f = GetBridgeFoundation(ti->tileh, ti->map5 & 0x1);	// pass direction
+				if (f) DrawFoundation(ti, f);
+				
+				// default sloped sprites..
+				if (ti->tileh != 0) image = _track_sloped_sprites[ti->tileh - 1] + 0x3F3;
+			}
+
 			// bridge ending.
 			b = _bridge_sprite_table[_map2[ti->tile]>>4][6];
 			b += (tmp&(3<<1))*4; /* actually ((tmp>>2)&3)*8 */
@@ -875,7 +1036,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 			// get bridge sprites
 			b = _bridge_sprite_table[_map2[ti->tile]>>4][_map2[ti->tile]&0xF] + tmp * 4;
 
-			z = GetBridgeHeight(ti->tile) + 5;
+			z = GetBridgeHeight(ti) + 5;
 			
 			// draw rail
 			image = b[0];
@@ -929,9 +1090,14 @@ static uint GetSlopeZ_TunnelBridge(TileInfo *ti) {
 		if ( ti->map5 & 0x80 ) {
 			// bridge ending?
 			if (!(ti->map5 & 0x40)) {
-				if (ti->tileh != 0) {
-					return ti->z + 8;
-				} else if (!(ti->map5 & 0x20)) {
+				if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << ti->tileh))	// 7, 11, 13, 14
+					z += 8;
+
+				// no ramp for bridge ending
+				if ((BRIDGE_PARTLY_LEVELED_FOUNDATION & (1 << ti->tileh) || BRIDGE_NO_FOUNDATION & (1 << ti->tileh)) && ti->tileh != 0) {
+					return z + 8;
+
+				} else if (!(ti->map5 & 0x20)) { // northern / southern ending
 					// ramp
 					return (z + (x>>1) + 1);
 				} else {
@@ -950,7 +1116,7 @@ static uint GetSlopeZ_TunnelBridge(TileInfo *ti) {
 				
 				// actually on the bridge, but not yet in the shared area.
 				if (!IS_INT_INSIDE(x, 5, 10+1)) 
-					return GetBridgeHeight(ti->tile) + 8;
+					return GetBridgeHeight(ti) + 8;
 
 				// in the shared area, assume that we're below the bridge, cause otherwise the hint would've caught it.
 				// if rail or road below then it means it's possibly build on slope below the bridge.
@@ -980,8 +1146,6 @@ static uint GetSlopeZ_TunnelBridge(TileInfo *ti) {
 	// default case
 	z = ti->z;
 	return GetPartialZ(ti->x&0xF, ti->y&0xF, ti->tileh) + z;
-
-
 }
 
 static void GetAcceptedCargo_TunnelBridge(uint tile, AcceptedCargo *ac)
@@ -989,7 +1153,7 @@ static void GetAcceptedCargo_TunnelBridge(uint tile, AcceptedCargo *ac)
 	/* not used */
 }
 
-static const StringID _bridge_tile_str[16+16] = {
+static const StringID _bridge_tile_str[(MAX_BRIDGES + 3) + (MAX_BRIDGES + 3)] = {
 	STR_501F_WOODEN_RAIL_BRIDGE,
 	STR_5020_CONCRETE_RAIL_BRIDGE,
 	STR_501C_STEEL_GIRDER_RAIL_BRIDGE,
