@@ -8,6 +8,7 @@
 #include "command.h"
 #include "saveload.h"
 #include "player.h"
+#include "engine.h"
 
 #define INVALID_COORD (-0x8000)
 #define GEN_HASH(x,y) (((x & 0x1F80)>>7) + ((y & 0xFC0)))
@@ -39,6 +40,29 @@ bool EnsureNoVehicle(TileIndex tile)
 	return VehicleFromPos(tile, (void*)(int)tile, (VehicleFromPosProc*)EnsureNoVehicleProc) == NULL;
 }
 
+static void *EnsureNoVehicleProcZ(Vehicle *v, void *data)
+{
+	uint32 d = (uint32)data;	// max mapsize 4,096*4,096 if shifted 8 bits for byte
+	// with uint64 max mapsize is 2,6843,5456*2,6843,5456 :D
+	if (v->tile != (TileIndex)(d >> (sizeof(byte)*8)) || v->z_pos != (byte)d || v->type == VEH_Disaster)
+		return NULL;
+
+	VehicleInTheWayErrMsg(v);
+	return (void*)1;
+}
+
+bool EnsureNoVehicleZ(TileIndex tile, byte z)
+{
+	TileInfo ti;
+	FindLandscapeHeightByTile(&ti, tile);
+	// needs z correction for slope-type graphics that have the NORTHERN tile lowered
+	// 1, 2, 3, 4, 5, 6 and 7
+	if (CORRECT_Z(ti.tileh))
+		z += 8;
+
+	return VehicleFromPos(tile, (void*)((TileIndex)tile << (sizeof(byte)*8) | (byte)z), (VehicleFromPosProc*)EnsureNoVehicleProcZ) == NULL;
+}
+
 Vehicle *FindVehicleBetween(TileIndex from, TileIndex to, byte z)
 {
 	int x1 = GET_TILE_X(from);
@@ -111,7 +135,6 @@ void AfterLoadVehicles()
 			VehiclePositionChanged(v);
 
 			if (v->type == VEH_Train) {
-				v->u.rail.last_z = v->z_pos;
 				if (v->subtype == 0)
 					UpdateTrainAcceleration(v);
 			}
@@ -1307,6 +1330,33 @@ void AgeVehicle(Vehicle *v)
 	}
 }
 
+void MaybeRenewVehicle(Vehicle *v, int32 build_cost)
+{
+	Engine *e;
+
+	// When automatically renewing a vehicle we want to prevent the
+	// "getting old" messages so we renew it if it won't enter the
+	// depot during the next service sooner than half a year before
+	// the vehicle getting old (that's one year before it reaches
+	// the max_age, see AgeVehicle).
+	if (!(_patches.autorenew && v->max_age - v->age < 366 + 183 + v->service_interval))
+		return;
+
+	e = &_engines[v->engine_type];
+	v->reliability = e->reliability;
+	v->reliability_spd_dec = e->reliability_spd_dec;
+	v->age = 0;
+
+	v->date_of_last_service = _date;
+	v->build_year = _cur_year;
+
+	SET_EXPENSES_TYPE(EXPENSES_NEW_VEHICLES);
+	SubtractMoneyFromPlayer(build_cost - v->value);
+	v->value = build_cost;
+	
+	InvalidateWindow(WC_VEHICLE_DETAILS, v->index);
+}
+
 
 int32 CmdNameVehicle(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
@@ -1513,13 +1563,16 @@ const byte _common_veh_desc[] = {
 static const byte _train_desc[] = {
 	SLE_WRITEBYTE(Vehicle,type,VEH_Train, 0), // Train type. VEH_Train in mem, 0 in file.
 	SLE_INCLUDEX(0, INC_VEHICLE_COMMON),
-	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,crash_anim_pos),	SLE_UINT16),
-	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,force_proceed),		SLE_UINT8),
-	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,railtype),				SLE_UINT8),
-	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,track),						SLE_UINT8),
+	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,crash_anim_pos), SLE_UINT16),
+	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,force_proceed), SLE_UINT8),
+	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,railtype), SLE_UINT8),
+	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleRail,track), SLE_UINT8),
 
-	// reserve extra space in savegame here. (currently 16 bytes)
-	SLE_CONDARR(NullStruct,null,SLE_FILE_U64 | SLE_VAR_NULL, 2, 2, 255),
+	SLE_CONDVARX(offsetof(Vehicle,u)+offsetof(VehicleRail,flags), SLE_UINT8, 2, 255),
+	SLE_CONDVARX(offsetof(Vehicle,u)+offsetof(VehicleRail,days_since_order_progr), SLE_UINT16, 2, 255),
+
+	// reserve extra space in savegame here. (currently 13 bytes)
+	SLE_CONDARR(NullStruct,null,SLE_FILE_U8 | SLE_VAR_NULL, 13, 2, 255),
 
 	SLE_END()
 };
@@ -1556,12 +1609,14 @@ static const byte _aircraft_desc[] = {
 	SLE_WRITEBYTE(Vehicle,type,VEH_Aircraft, 3), // Aircraft type. VEH_Aircraft in mem, 3 in file.
 	SLE_INCLUDEX(0, INC_VEHICLE_COMMON),
 	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleAir,crashed_counter),	SLE_UINT16),
-	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleAir,unk0),							SLE_UINT8),
+	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleAir,pos),							SLE_UINT8),
 	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleAir,targetairport),		SLE_UINT8),
 	SLE_VARX(offsetof(Vehicle,u)+offsetof(VehicleAir,state),						SLE_UINT8),
+	
+	SLE_CONDVARX(offsetof(Vehicle,u)+offsetof(VehicleAir,previous_pos),			SLE_UINT8, 2, 255),
 
-	// reserve extra space in savegame here. (currently 16 bytes)
-	SLE_CONDARR(NullStruct,null,SLE_FILE_U64 | SLE_VAR_NULL, 2, 2, 255),
+	// reserve extra space in savegame here. (currently 15 bytes)
+	SLE_CONDARR(NullStruct,null,SLE_FILE_U8 | SLE_VAR_NULL, 15, 2, 255),
 
 	SLE_END()
 };
@@ -1660,11 +1715,26 @@ static void Save_VEHS()
 static void Load_VEHS()
 {
 	int index;
+	Vehicle *v;
+
 	while ((index = SlIterateArray()) != -1) {
 		Vehicle *v = &_vehicles[index];
 		v->next_in_chain_old = INVALID_VEHICLE;
 		SlObject(v, _veh_descs[SlReadByte()]);
 		v->next = v->next_in_chain_old == INVALID_VEHICLE ? NULL : &_vehicles[v->next_in_chain_old];
+		if (v->type == VEH_Train)
+			v->u.rail.first_engine = 0xffff;
+	}
+
+	// Iterate through trains and set first_engine appropriately.
+	FOR_ALL_VEHICLES(v) {
+		Vehicle *w;
+
+		if (v->type != VEH_Train || v->subtype != 0)
+			continue;
+
+		for (w = v->next; w; w = w->next)
+			w->u.rail.first_engine = v->engine_type;
 	}
 }
 
@@ -1698,6 +1768,10 @@ static const byte _checkpoint_desc[] = {
 	SLE_VAR(Checkpoint,xy,								SLE_UINT16),
 	SLE_VAR(Checkpoint,city_or_string,		SLE_UINT16),
 	SLE_VAR(Checkpoint,deleted,						SLE_UINT8),
+
+	SLE_CONDVAR(Checkpoint, build_date, SLE_UINT16, 3, 255),
+	SLE_CONDVAR(Checkpoint, stat_id, SLE_UINT8, 3, 255),
+
 	SLE_END()
 };
 
